@@ -1,4 +1,4 @@
-#!/bin/sh
+#!/bin/sh -e
 
 while getopts "q" flag
 do
@@ -15,6 +15,17 @@ do
 done
 
 echo "Updating configuration..."
+
+echo "Running libtoolize"
+if which libtoolize > /dev/null 2>&1; then
+    libtoolize -c -f
+elif which glibtoolize > /dev/null 2>&1; then
+    glibtoolize -c -f
+else
+  echo "No libtoolize found on your system (looked for libtoolize & glibtoolize)"
+  exit 1
+fi
+
 echo "Running aclocal"
 if which aclocal > /dev/null 2>&1; then
   aclocal -I src/cf -I src/external/rra-c-util/m4
@@ -36,7 +47,7 @@ autoheader
 #automake
 
 echo "Deleting autom4te.cache directory"
-rm -r autom4te.cache
+rm -rf autom4te.cache
 
 if [ $skipman ] ; then
     echo "Skipping man page build"
