@@ -1,12 +1,16 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
  */
 
+#include <afsconfig.h>
+#include <afs/param.h>
+#include <roken.h>
+
 #include <osi.h>
 #include "afsd.h"
 
@@ -36,7 +40,7 @@ long ReadData(cm_scache_t *scp, osi_hyper_t offset, long count, char *op,
     /* start by looking up the file's end */
     code = cm_SyncOp(scp, NULL, userp, &req, 0,
                       CM_SCACHESYNC_NEEDCALLBACK | CM_SCACHESYNC_GETSTATUS);
-    if (code) 
+    if (code)
         goto done;
 
     cm_SyncOpDone(scp, NULL, CM_SCACHESYNC_NEEDCALLBACK | CM_SCACHESYNC_GETSTATUS);
@@ -60,7 +64,7 @@ long ReadData(cm_scache_t *scp, osi_hyper_t offset, long count, char *op,
             count = 0;
         else
             count = thyper.LowPart;
-    }       
+    }
 
     *readp = count;
 
@@ -82,7 +86,7 @@ long ReadData(cm_scache_t *scp, osi_hyper_t offset, long count, char *op,
             }
             lock_ReleaseWrite(&scp->rw);
 
-            code = buf_Get(scp, &thyper, &bufferp);
+            code = buf_Get(scp, &thyper, &req, &bufferp);
 
             lock_ObtainWrite(&scp->rw);
             if (code) goto done;
@@ -93,7 +97,7 @@ long ReadData(cm_scache_t *scp, osi_hyper_t offset, long count, char *op,
                 code = cm_SyncOp(scp, bufferp, userp, &req, 0,
                                   CM_SCACHESYNC_NEEDCALLBACK
                                   | CM_SCACHESYNC_READ);
-                if (code) 
+                if (code)
                     goto done;
 
 		cm_SyncOpDone(scp, bufferp, CM_SCACHESYNC_NEEDCALLBACK | CM_SCACHESYNC_READ);
@@ -134,14 +138,14 @@ long ReadData(cm_scache_t *scp, osi_hyper_t offset, long count, char *op,
   done:
     lock_ReleaseWrite(&scp->rw);
     //lock_ReleaseMutex(&fidp->mx);
-    if (bufferp) 
+    if (bufferp)
         buf_Release(bufferp);
 
     if (code == 0 && sequential)
         cm_ConsiderPrefetch(scp, &lastByte, *readp, userp, &req);
 
     return code;
-}       
+}
 
 
 long WriteData(cm_scache_t *scp, osi_hyper_t offset, long count, char *op,
@@ -174,9 +178,9 @@ long WriteData(cm_scache_t *scp, osi_hyper_t offset, long count, char *op,
                      CM_SCACHESYNC_NEEDCALLBACK
                       | CM_SCACHESYNC_SETSTATUS
                       | CM_SCACHESYNC_GETSTATUS);
-    if (code) 
+    if (code)
         goto done;
-    
+
     cm_SyncOpDone(scp, NULL, CM_SCACHESYNC_NEEDCALLBACK | CM_SCACHESYNC_SETSTATUS | CM_SCACHESYNC_GETSTATUS);
 
 #if 0
@@ -186,7 +190,7 @@ long WriteData(cm_scache_t *scp, osi_hyper_t offset, long count, char *op,
     goto done;
     }
 #endif
-	
+
     /* now we have the entry locked, look up the length */
     fileLength = scp->length;
     minLength = fileLength;
@@ -240,14 +244,14 @@ long WriteData(cm_scache_t *scp, osi_hyper_t offset, long count, char *op,
                 lock_ReleaseMutex(&bufferp->mx);
                 buf_Release(bufferp);
                 bufferp = NULL;
-            }	
+            }
             lock_ReleaseWrite(&scp->rw);
 
-            code = buf_Get(scp, &thyper, &bufferp);
+            code = buf_Get(scp, &thyper, &req, &bufferp);
 
             lock_ObtainMutex(&bufferp->mx);
             lock_ObtainWrite(&scp->rw);
-            if (code) 
+            if (code)
                 goto done;
 
             bufferOffset = thyper;
@@ -258,19 +262,19 @@ long WriteData(cm_scache_t *scp, osi_hyper_t offset, long count, char *op,
                                   CM_SCACHESYNC_NEEDCALLBACK
                                   | CM_SCACHESYNC_WRITE
                                   | CM_SCACHESYNC_BUFLOCKED);
-                if (code) 
+                if (code)
                     goto done;
-                       
-		cm_SyncOpDone(scp, bufferp, 
-			       CM_SCACHESYNC_NEEDCALLBACK 
-			       | CM_SCACHESYNC_WRITE 
+
+		cm_SyncOpDone(scp, bufferp,
+			       CM_SCACHESYNC_NEEDCALLBACK
+			       | CM_SCACHESYNC_WRITE
 			       | CM_SCACHESYNC_BUFLOCKED);
 
                 /* If we're overwriting the entire buffer, or
                  * if we're writing at or past EOF, mark the
                  * buffer as current so we don't call
                  * cm_GetBuffer.  This skips the fetch from the
-                 * server in those cases where we're going to 
+                 * server in those cases where we're going to
                  * obliterate all the data in the buffer anyway,
                  * or in those cases where there is no useful
                  * data at the server to start with.
@@ -299,7 +303,7 @@ long WriteData(cm_scache_t *scp, osi_hyper_t offset, long count, char *op,
                 lock_ReleaseWrite(&scp->rw);
                 lock_ObtainMutex(&bufferp->mx);
                 lock_ObtainWrite(&scp->rw);
-                if (code) 
+                if (code)
                     break;
             }
             if (code) {
@@ -317,12 +321,12 @@ long WriteData(cm_scache_t *scp, osi_hyper_t offset, long count, char *op,
 
         /* and figure out how many bytes we want from this buffer */
         nbytes = buf_bufferSize - bufIndex;	/* what remains in buffer */
-        if (nbytes > count) 
+        if (nbytes > count)
             nbytes = count;	/* don't go past end of request */
 
         /* now copy the data */
-        memcpy(bufferp->datap + bufIndex, op, nbytes);
-        buf_SetDirty(bufferp, bufIndex, nbytes, userp);
+	memcpy(bufferp->datap + bufIndex, op, nbytes);
+        buf_SetDirty(bufferp, &req, bufIndex, nbytes, userp);
 
         /* adjust counters, pointers, etc. */
         op += nbytes;
@@ -346,8 +350,8 @@ long WriteData(cm_scache_t *scp, osi_hyper_t offset, long count, char *op,
         lock_ReleaseWrite(&scp->rw);
         if (code == 0)
             cm_QueueBKGRequest(scp, cm_BkgStore, writeBackOffset.LowPart,
-                               writeBackOffset.HighPart, cm_chunkSize, 0, userp);
-    }   
+                               writeBackOffset.HighPart, cm_chunkSize, 0, userp, &req);
+    }
 
     /* cm_SyncOpDone is called when cm_BkgStore completes */
     return code;
