@@ -108,8 +108,8 @@ typedef struct _AFS_COMM_RESULT_BLOCK
 //
 
 #define AFS_REDIR_INIT_FLAG_HIDE_DOT_FILES          0x00000001
-
 #define AFS_REDIR_INIT_NO_PAGING_FILE               0x00000002
+#define AFS_REDIR_INIT_FLAG_DISABLE_SHORTNAMES	    0x00000004
 
 typedef struct _AFS_REDIR_INIT_INFO_CB
 {
@@ -214,6 +214,8 @@ typedef struct _AFS_DIR_ENUM_ENTRY
 
     WCHAR           ShortName[12];
 
+    ULONG           NTStatus;           /* Error code during enumeration */
+
     /* Long Filename and Target (Symlink and MountPoint only) to follow without NULs */
 
 } AFSDirEnumEntry;
@@ -224,6 +226,10 @@ typedef struct _AFS_DIR_ENUM_RESP
 
     ULONG_PTR       EnumHandle;
 
+    LARGE_INTEGER   SnapshotDataVersion; /* DV at time Name/FID list was generated */
+
+    LARGE_INTEGER   CurrentDataVersion;   /* DV at time this header was constructed */
+
     AFSDirEnumEntry Entry[ 1];     /* Each entry is Quad aligned */
 
 } AFSDirEnumResp;
@@ -235,9 +241,9 @@ typedef struct _AFS_DIR_ENUM_RESP
 typedef struct _AFS_VOLUME_INFORMATION
 {
 
-    LARGE_INTEGER   TotalAllocationUnits;       /* Partition Max Blocks */
+    LARGE_INTEGER   TotalAllocationUnits;       /* Volume Max Blocks (Partition or Quota) */
 
-    LARGE_INTEGER   AvailableAllocationUnits;   /* Partition Blocks Avail */
+    LARGE_INTEGER   AvailableAllocationUnits;   /* Volume Blocks Avail (Partition or Quota) */
 
     LARGE_INTEGER   VolumeCreationTime;         /* AFS Last Update - Not Creation */
 
@@ -258,10 +264,29 @@ typedef struct _AFS_VOLUME_INFORMATION
 
     ULONG           VolumeLabelLength;
 
-    WCHAR           VolumeLabel[20];            /* Volume:Cell */
+    WCHAR           VolumeLabel[128];            /* Volume:Cell */
 
 } AFSVolumeInfoCB;
 
+
+//
+// Volume size information CB passed used to satisfy
+// FileFsFullSizeInformation and FileFsSizeInformation
+//
+
+typedef struct _AFS_VOLUME_SIZE_INFORMATION
+{
+
+    LARGE_INTEGER   TotalAllocationUnits;       /* Max Blocks (Quota or Partition) */
+
+    LARGE_INTEGER   AvailableAllocationUnits;   /* Blocks Avail (Quota or Partition) */
+
+    ULONG           SectorsPerAllocationUnit;   /* = 1 */
+
+    ULONG           BytesPerSector;             /* = 1024 */
+
+} AFSVolumeSizeInfoCB;
+
 //
 // File create CB
 //
@@ -303,6 +328,10 @@ typedef struct _AFS_FILE_OPEN_CB
 
     ULONG           ShareAccess;
 
+    ULONGLONG       ProcessId;
+
+    ULONGLONG       Identifier;
+
 } AFSFileOpenCB;
 
 typedef struct _AFS_FILE_OPEN_RESULT_CB
@@ -310,8 +339,21 @@ typedef struct _AFS_FILE_OPEN_RESULT_CB
 
     ULONG           GrantedAccess;
 
+    ULONG           FileAccess;
+
 } AFSFileOpenResultCB;
 
+typedef struct _AFS_FILE_ACCESS_RELEASE_CB
+{
+
+    ULONG           FileAccess;
+
+    ULONGLONG       ProcessId;
+
+    ULONGLONG       Identifier;
+
+} AFSFileAccessReleaseCB;
+
 //
 // IO Interace control blocks for extent processing when performing
 // queries via the AFS_REQUEST_TYPE_REQUEST_FILE_EXTENTS or synchronous
@@ -463,6 +505,8 @@ typedef struct _AFS_EXTENT_FAILURE_CB
 
     ULONG           FailureStatus;
 
+    GUID            AuthGroup;      // Length: sizeof(GUID) */
+
 } AFSExtentFailureCB;
 
 //
@@ -499,6 +543,8 @@ typedef struct _AFS_FILE_UPDATE_CB
 typedef struct _AFS_FILE_UPDATE_RESULT_CB
 {
 
+    LARGE_INTEGER   ParentDataVersion;
+
     AFSDirEnumEntry DirEnum;
 
 } AFSFileUpdateResultCB;
@@ -571,6 +617,16 @@ typedef struct _AFS_FILE_EVAL_TARGET_CB
 } AFSEvalTargetCB;
 
 
+typedef struct _AFS_FILE_EVAL_RESULT_CB
+{
+
+    LARGE_INTEGER   ParentDataVersion;
+
+    AFSDirEnumEntry DirEnum;
+
+} AFSFileEvalResultCB;
+
+
 //
 // Control structure for read and write requests through the PIOCtl interface
 //
@@ -956,8 +1012,20 @@ typedef struct _AFS_FILE_CLEANUP_CB
 
     ULONGLONG       ProcessId;
 
+    ULONG           FileAccess;
+
+    ULONGLONG       Identifier;
+
 } AFSFileCleanupCB;
 
+typedef struct _AFS_FILE_CLEANUP_RESULT_CB
+{
+
+    LARGE_INTEGER   ParentDataVersion;
+
+} AFSFileCleanupResultCB;
+
+
 //
 // Trace configuration cb
 //
