@@ -186,24 +186,14 @@ afs_linux_key_alloc(struct key_type *type, const char *desc, uid_t uid,
 }
 
 # if defined(STRUCT_TASK_STRUCT_HAS_CRED)
-static inline struct key *
-afs_session_keyring(afs_ucred_t *cred)
-{
-#  if defined(STRUCT_CRED_HAS_SESSION_KEYRING)
-    return cred->session_keyring;
-#  else
-    return cred->tgcred->session_keyring;
-#  endif
-}
-
 static inline struct key*
 afs_linux_search_keyring(afs_ucred_t *cred, struct key_type *type)
 {
     key_ref_t key_ref;
 
-    if (afs_session_keyring(cred)) {
+    if (cred->tgcred->session_keyring) {
 	key_ref = keyring_search(
-		      make_key_ref(afs_session_keyring(cred), 1),
+		      make_key_ref(cred->tgcred->session_keyring, 1),
 		      type, "_pag");
 	if (IS_ERR(key_ref))
 	    return ERR_CAST(key_ref);
@@ -299,21 +289,39 @@ kernel_getsockopt(struct socket *sockp, int level, int name, char *val,
 #endif
 
 #ifdef HAVE_TRY_TO_FREEZE
-static inline void
+static inline int
 afs_try_to_freeze(void) {
 # ifdef LINUX_REFRIGERATOR_TAKES_PF_FREEZE
-    try_to_freeze(PF_FREEZE);
+    return try_to_freeze(PF_FREEZE);
 # else
-    try_to_freeze();
+    return try_to_freeze();
 # endif
 }
 #else
-static inline void
+static inline int
 afs_try_to_freeze(void) {
 # ifdef CONFIG_PM
     if (current->flags & PF_FREEZE) {
 	refrigerator(PF_FREEZE);
+	return 1;
     }
+# endif
+    return 0;
+}
+#endif
+
+/* The commit which changed refrigerator so that it takes no arguments
+ * also added freezing(), so if LINUX_REFRIGERATOR_TAKES_PF_FREEZE is
+ * true, the kernel doesn't have a freezing() function.
+ */
+#ifdef LINUX_REFRIGERATOR_TAKES_PF_FREEZE
+static inline int
+freezing(struct task_struct *p)
+{
+# ifdef CONFIG_PM
+    return p->flags & PF_FREEZE;
+# else
+    return 0;
 # endif
 }
 #endif
@@ -465,6 +473,41 @@ afs_get_dentry_ref(afs_linux_path_t *path, struct vfsmount **mnt, struct dentry
 #endif
 }
 
+/* wait_event_freezable appeared with 2.6.24 */
+
+/* These implement the original AFS wait behaviour, with respect to the
+ * refrigerator, rather than the behaviour of the current wait_event_freezable
+ * implementation.
+ */
+
+#ifndef wait_event_freezable
+# define wait_event_freezable(waitqueue, condition)				\
+({										\
+    int _ret;									\
+    do {									\
+	_ret = wait_event_interruptible(waitqueue, 				\
+					(condition) || freezing(current));	\
+	if (_ret && !freezing(current))					\
+	    break;								\
+	else if (!(condition))							\
+	    _ret = -EINTR;							\
+    } while (afs_try_to_freeze());						\
+    _ret;									\
+})
+
+# define wait_event_freezable_timeout(waitqueue, condition, timeout)		\
+({										\
+     int _ret;									\
+     do {									\
+	_ret = wait_event_interruptible_timeout(waitqueue,			\
+						(condition || 			\
+						 freezing(current)),		\
+						timeout);			\
+     } while (afs_try_to_freeze());						\
+     _ret;									\
+})
+#endif
+
 #if defined(STRUCT_TASK_STRUCT_HAS_CRED)
 static inline struct file *
 afs_dentry_open(struct dentry *dp, struct vfsmount *mnt, int flags, const struct cred *creds) {
@@ -481,7 +524,7 @@ afs_dentry_open(struct dentry *dp, struct vfsmount *mnt, int flags, const struct
 }
 #endif
 
-#if !defined(STRUCT_FILENAME_HAS_NAME)
+#if defined(HAVE_LINUX_PUTNAME)
 typedef char *afs_name_t;
 
 static inline char *
@@ -493,11 +536,6 @@ static inline void
 afs_putname(afs_name_t name) {
     putname((char *)name);
 }
-
-static inline void
-afs_set_name(afs_name_t name, char *string) {
-    name = string;
-}
 #else
 typedef struct filename *afs_name_t;
 
@@ -510,45 +548,6 @@ static inline void
 afs_putname(afs_name_t name) {
     kmem_cache_free(names_cachep, (void *)name);
 }
-
-static inline void
-afs_set_name(afs_name_t aname, char *string) {
-    aname->name = string;
-}
 #endif
 
-static_inline struct key *
-afs_set_session_keyring(struct key *keyring)
-{
-    struct key *old;
-#if defined(STRUCT_CRED_HAS_SESSION_KEYRING)
-    struct cred *new_creds;
-    old = current_session_keyring();
-    new_creds = prepare_creds();
-    rcu_assign_pointer(new_creds->session_keyring, keyring);
-    commit_creds(new_creds);
-#else
-    spin_lock_irq(&current->sighand->siglock);
-    old = task_session_keyring(current);
-    smp_wmb();
-    task_session_keyring(current) = keyring;
-    spin_unlock_irq(&current->sighand->siglock);
-#endif
-    return old;
-}
-
-static inline int
-afs_truncate(struct inode *inode, int len)
-{
-    int code;
-#if defined(STRUCT_INODE_OPERATIONS_HAS_TRUNCATE)
-    code = vmtruncate(inode, len);
-#else
-    code = inode_newsize_ok(inode, len);
-    if (!code)
-        truncate_setsize(inode, len);
-#endif
-    return code;
-}
-
 #endif /* AFS_LINUX_OSI_COMPAT_H */
