@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -11,7 +11,9 @@
 #define __AFS_SYSINCLUDESH__ 1
 
 #include  <stdio.h>
-#if !defined(AFS_USR_DARWIN_ENV) && !defined(AFS_USR_FBSD_ENV)	/* must be included after KERNEL undef'd */
+#include  <afs/opr.h>
+
+#if !defined(AFS_USR_DARWIN_ENV) && !defined(AFS_USR_FBSD_ENV) && !defined(AFS_USR_DFBSD_ENV) /* must be included after KERNEL undef'd */
 #include  <errno.h>
 #endif
 #include  <stdlib.h>
@@ -20,7 +22,7 @@
 #include  <assert.h>
 #include  <stdarg.h>
 
-#if !defined(AFS_USR_DARWIN_ENV) && !defined(AFS_USR_FBSD_ENV) /* must be included after KERNEL undef'd */
+#if !defined(AFS_USR_DARWIN_ENV) && !defined(AFS_USR_FBSD_ENV) && !defined(AFS_USR_DFBSD_ENV) /* must be included after KERNEL undef'd */
 #include  <unistd.h>
 #include  <ctype.h>
 #include  <sys/types.h>
@@ -114,7 +116,10 @@
 #define FREAD			0x0001
 #endif /* AFS_USR_LINUX22_ENV */
 
-#if defined(AFS_USR_DARWIN_ENV) || defined(AFS_USR_FBSD_ENV)
+#if defined(AFS_USR_DARWIN_ENV) || defined(AFS_USR_FBSD_ENV) || defined(AFS_USR_DFBSD_ENV)
+#ifdef _KERNEL
+#undef _KERNEL
+#endif
 #ifdef KERNEL
 #undef KERNEL
 #define AFS_USR_UNDEF_KERNEL_ENV 1
@@ -155,10 +160,11 @@
 typedef unsigned int fsblkcnt_t;
 #endif
 
-#ifdef AFS_AFSDB_ENV
 #include <arpa/nameser.h>
+#ifdef HAVE_ARPA_NAMESER_COMPAT_H
+#include <arpa/nameser_compat.h>
+#endif
 #include <resolv.h>
-#endif /* AFS_AFSDB_ENV */
 
 /* glibc 2.2 has pthread_attr_setstacksize */
 #if (defined(AFS_LINUX22_ENV) && !defined(AFS_USR_LINUX22_ENV)) || (defined(AFS_USR_LINUX22_ENV) && (__GLIBC_MINOR__ < 2))
@@ -198,9 +204,8 @@ typedef unsigned int fsblkcnt_t;
 
 #ifdef UKERNEL
 
-#ifdef AFS_USR_SGI_ENV
 #undef socket
-#endif /* AFS_USR_SGI_ENV */
+#undef flock
 
 #if defined(AFS_USR_DARWIN_ENV) || defined(AFS_USR_FBSD_ENV)
 #undef if_mtu
@@ -220,17 +225,15 @@ typedef unsigned int fsblkcnt_t;
 #define vattr			usr_vattr
 #define buf			usr_buf
 #define statfs			usr_statfs
-#define ucred			usr_ucred
-#define user			usr_user
-#define proc			usr_proc
 #define file			usr_file
 #define dirent			usr_dirent
 #define flock			usr_flock
 #define fid			usr_fid
 #define sysent			usr_sysent
+#define ifaddr                  usr_ifaddr
+#define ifnet                   usr_ifnet
 #define in_ifaddr		usr_in_ifaddr
-#define ifaddr			usr_ifaddr
-#define ifnet			usr_ifnet
+#undef socket
 #define socket			usr_socket
 #define crget			usr_crget
 #define crcopy			usr_crcopy
@@ -253,6 +256,7 @@ typedef unsigned int fsblkcnt_t;
 #define offset_t		usr_offset_t
 #define getpid()		usr_getpid()
 #define setpag(A,B,C,D)		usr_setpag((A),(B),(C),(D))
+#define osi_getpid()	        afs_pointer_to_int(usr_thread_self())
 #ifdef pid_t
 #undef pid_t
 #endif
@@ -339,26 +343,18 @@ typedef long usr_ino_t;
 #ifdef VFIFO
 #undef VFIFO
 #endif
-#ifdef VDOOR
-#undef VDOOR
-#endif
-#ifdef VBAD
-#undef VBAD
-#endif
 #ifdef VSOCK
 #undef VSOCK
 #endif
 
-#define VNON			0
-#define VREG			1
-#define VDIR			2
-#define VBLK			3
-#define VCHR			4
-#define VLNK			5
-#define VFIFO			6
-#define VDOOR			7
-#define VBAD			8
-#define VSOCK			9
+#define VNON 0
+#define VREG S_IFREG
+#define VDIR S_IFDIR
+#define VBLK S_IFBLK
+#define VCHR S_IFCHR
+#define VLNK S_IFLNK
+#define VSOCK S_IFSOCK
+#define VFIFO S_IFIFO
 
 typedef int usr_vtype_t;
 
@@ -840,7 +836,7 @@ enum usr_uio_rw { USR_UIO_READ, USR_UIO_WRITE };
 #endif
 #define NBPG			4096
 
-#define panic(S)		do{fprintf(stderr, S);assert(0);}while(0)
+#define panic(S)		do{fprintf(stderr, "%s", S);assert(0);}while(0)
 #define abort()			assert(0)
 #define usr_assert(A)		assert(A)
 
@@ -941,29 +937,29 @@ extern pthread_cond_t usr_sleep_cond;
 #define usr_thread_t		pthread_t
 #define usr_key_t		pthread_key_t
 
-#define usr_mutex_init(A)	assert(pthread_mutex_init(A,NULL) == 0)
-#define usr_mutex_destroy(A)	assert(pthread_mutex_destroy(A) == 0)
-#define usr_mutex_lock(A)	assert(pthread_mutex_lock(A) == 0)
+#define usr_mutex_init(A)	opr_Verify(pthread_mutex_init(A,NULL) == 0)
+#define usr_mutex_destroy(A)	opr_Verify(pthread_mutex_destroy(A) == 0)
+#define usr_mutex_lock(A)	opr_Verify(pthread_mutex_lock(A) == 0)
 #define usr_mutex_trylock(A)	((pthread_mutex_trylock(A)==0)?1:0)
-#define usr_mutex_unlock(A)	assert(pthread_mutex_unlock(A) == 0)
-#define usr_cond_init(A)	assert(pthread_cond_init(A,NULL) == 0)
-#define usr_cond_destroy(A)	assert(pthread_cond_destroy(A) == 0)
-#define usr_cond_signal(A)	assert(pthread_cond_signal(A) == 0)
-#define usr_cond_broadcast(A)	assert(pthread_cond_broadcast(A) == 0)
+#define usr_mutex_unlock(A)	opr_Verify(pthread_mutex_unlock(A) == 0)
+#define usr_cond_init(A)	opr_Verify(pthread_cond_init(A,NULL) == 0)
+#define usr_cond_destroy(A)	opr_Verify(pthread_cond_destroy(A) == 0)
+#define usr_cond_signal(A)	opr_Verify(pthread_cond_signal(A) == 0)
+#define usr_cond_broadcast(A)	opr_Verify(pthread_cond_broadcast(A) == 0)
 #define usr_cond_wait(A,B)	pthread_cond_wait(A,B)
 #define usr_cond_timedwait(A,B,C)  pthread_cond_timedwait(A,B,C)
 
 #define usr_thread_create(A,B,C) \
     do { \
 	pthread_attr_t attr; \
-	assert(pthread_attr_init(&attr) == 0); \
-	assert(pthread_attr_setstacksize(&attr, 124288) == 0); \
-	assert(pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED) == 0); \
-	assert(pthread_create((A), &attr, (B), (void *)(C)) == 0); \
-	assert(pthread_attr_destroy(&attr) == 0); \
+	opr_Verify(pthread_attr_init(&attr) == 0); \
+	opr_Verify(pthread_attr_setstacksize(&attr, 122880) == 0); \
+	opr_Verify(pthread_create((A), &attr, (B), (void *)(C)) == 0); \
+	opr_Verify(pthread_attr_destroy(&attr) == 0); \
     } while(0)
+#define usr_thread_join(A,B)	pthread_join(A, B)
 #define usr_thread_detach(A)	pthread_detach(A)
-#define usr_keycreate(A,B)	assert(pthread_key_create(A,B) == 0)
+#define usr_keycreate(A,B)	opr_Verify(pthread_key_create(A,B) == 0)
 #define usr_setspecific(A,B)	pthread_setspecific(A,B)
 #define usr_getspecific(A,B)	(*(B)=pthread_getspecific(A),0)
 #define usr_thread_self()	pthread_self()
@@ -979,9 +975,9 @@ extern pthread_cond_t usr_sleep_cond;
 	_sleep_ts.tv_sec += 1;						   \
 	_sleep_ts.tv_nsec -= 1000000000;				   \
     }									   \
-    assert(pthread_mutex_lock(&usr_sleep_mutex) == 0);			   \
+    opr_Verify(pthread_mutex_lock(&usr_sleep_mutex) == 0);			   \
     pthread_cond_timedwait(&usr_sleep_cond, &usr_sleep_mutex, &_sleep_ts); \
-    assert(pthread_mutex_unlock(&usr_sleep_mutex) == 0);		   \
+    opr_Verify(pthread_mutex_unlock(&usr_sleep_mutex) == 0);		   \
 }
 
 #define uprintf			printf
@@ -1011,7 +1007,7 @@ do { \
     AFS_ASSERT_GLOCK(); \
     usr_assert((vp)->v_count > 0); \
     if (--((vp)->v_count) == 0) \
-	afs_inactive(VTOAFS(vp), u.u_cred); \
+	afs_inactive(VTOAFS(vp), get_user_struct()->u_cred); \
 } while(0)
 
 struct usr_statfs {
@@ -1158,7 +1154,6 @@ extern struct usr_ucred *usr_crget(void);
 extern struct usr_ucred *usr_crcopy(struct usr_ucred *);
 extern int usr_crhold(struct usr_ucred *);
 extern int usr_crfree(struct usr_ucred *);
-extern struct usr_ucred *afs_global_ucredp;
 
 struct usr_proc {
     unsigned long p_flag;
@@ -1221,44 +1216,44 @@ struct usr_socket {
 ((sizeof (struct usr_dirent)+4 - (MAXNAMLEN+1)) + (((len)+1 + 3) &~ 3))
 
 struct vcache;
-#define AFS_UCRED struct usr_ucred
+#define afs_ucred_t struct usr_ucred
 #define AFS_FLOCK       flock
 
 
 struct usr_vnodeops {
-    int (*vn_open) (struct vcache **, afs_int32, AFS_UCRED *);
-    int (*vn_close) (struct vcache *, afs_int32, AFS_UCRED *);
-    int (*vn_rdwr) (struct usr_vnode *avc, struct usr_uio *uio, 
+    int (*vn_open) (struct vcache **, afs_int32, afs_ucred_t *);
+    int (*vn_close) (struct vcache *, afs_int32, afs_ucred_t *);
+    int (*vn_rdwr) (struct usr_vnode *avc, struct usr_uio *uio,
 		    int rw, int io, struct usr_ucred *cred);
     int (*vn_ioctl) (void);
     int (*vn_select) (void);
-    int (*vn_getattr) (struct vcache *avc, struct vattr *, AFS_UCRED *);
-    int (*vn_setattr) (struct vcache *avc, struct vattr *, AFS_UCRED *);
-    int (*vn_access) (struct vcache *avc, afs_int32, AFS_UCRED *);
-    int (*vn_lookup) (struct vcache *adp, char *, struct vcache **, 
-		      AFS_UCRED *, int);
-    int (*vn_create) (struct vcache *adp, char *, struct vattr *, 
-		      enum vcexcl, int, struct vcache **, AFS_UCRED *);
-    int (*vn_remove) (struct vcache *adp, char *, AFS_UCRED *);
-    int (*vn_link) (struct vcache *avc, struct vcache *adp, char *, 
-		    AFS_UCRED *);
+    int (*vn_getattr) (struct vcache *avc, struct vattr *, afs_ucred_t *);
+    int (*vn_setattr) (struct vcache *avc, struct vattr *, afs_ucred_t *);
+    int (*vn_access) (struct vcache *avc, afs_int32, afs_ucred_t *);
+    int (*vn_lookup) (struct vcache *adp, char *, struct vcache **,
+		      afs_ucred_t *, int);
+    int (*vn_create) (struct vcache *adp, char *, struct vattr *,
+		      enum vcexcl, int, struct vcache **, afs_ucred_t *);
+    int (*vn_remove) (struct vcache *adp, char *, afs_ucred_t *);
+    int (*vn_link) (struct vcache *avc, struct vcache *adp, char *,
+		    afs_ucred_t *);
     int (*vn_rename) (struct vcache *aodp, char *, struct vcache *, char *,
-		      AFS_UCRED *);
-    int (*vn_mkdir) (struct vcache *adp, char *, struct vattr *, 
-		     struct vcache **, AFS_UCRED *);
-    int (*vn_rmdir) (struct vcache *adp, char *, AFS_UCRED *);
-    int (*vn_readdir) (struct vcache *avc, struct uio *, AFS_UCRED *);
+		      afs_ucred_t *);
+    int (*vn_mkdir) (struct vcache *adp, char *, struct vattr *,
+		     struct vcache **, afs_ucred_t *);
+    int (*vn_rmdir) (struct vcache *adp, char *, afs_ucred_t *);
+    int (*vn_readdir) (struct vcache *avc, struct uio *, afs_ucred_t *);
     int (*vn_symlink) (struct vcache *adp, char *, struct vattr *, char *,
-		       AFS_UCRED *);
-    int (*vn_readlink) (struct vcache *avc, struct uio *, AFS_UCRED *);
-    int (*vn_fsync) (struct vcache *avc, AFS_UCRED *);
-    int (*vn_inactive) (struct vcache *avc, AFS_UCRED *acred);
+		       afs_ucred_t *);
+    int (*vn_readlink) (struct vcache *avc, struct uio *, afs_ucred_t *);
+    int (*vn_fsync) (struct vcache *avc, afs_ucred_t *);
+    int (*vn_inactive) (struct vcache *avc, afs_ucred_t *acred);
     int (*vn_bmap) (void);
     int (*vn_strategy) (void);
     int (*vn_bread) (void);
     int (*vn_brelse) (void);
     int (*vn_lockctl) (struct vcache *, struct AFS_FLOCK *, int,
-		       AFS_UCRED *);
+		       afs_ucred_t *);
     int (*vn_fid) (struct vcache *avc, struct fid **);
 };
 
@@ -1278,7 +1273,7 @@ extern struct usr_mount *getmp(unsigned long);
 typedef long usr_whymountroot_t;
 
 struct usr_vfsops {
-    int (*vfs_mount) (struct vfs *, char *, struct vfs *);
+    int (*vfs_mount) (struct vfs *, char *, void *);
     int (*vfs_unmount) (struct vfs *);
     int (*vfs_root) (struct vfs *, struct vnode **);
     int (*vfs_statfs) (struct vfs *, struct statfs *);
@@ -1297,12 +1292,6 @@ struct usr_vfs {
     struct usr_vfsops *vfs_op;
 };
 
-struct usr_sysent {
-    char sy_narg;
-    int (*sy_call) (void);
-};
-extern struct usr_sysent usr_sysent[];
-
 struct usr_ifnet {
     struct usr_ifnet *if_next;
     short if_flags;
@@ -1337,18 +1326,10 @@ struct usr_in_ifaddr {
 };
 extern struct usr_in_ifaddr *usr_in_ifaddr;
 
-extern usr_key_t afs_global_u_key;	/* for per thread authentication */
-
-#if defined(AFS_USR_OSF_ENV)
-extern char V;
-#else
-extern long V;
-#endif
-
 #endif /* UKERNEL */
 
 struct min_direct {
-#if defined(AFS_OFS_ENV) || defined(AFS_USR_OSF_ENV)
+#if defined(AFS_USR_OSF_ENV)
     unsigned int d_fileno;
 #else				/* AFS_OFS_ENV || AFS_USR_OSF_ENV */
     unsigned long d_fileno;
@@ -1397,7 +1378,6 @@ struct usr_user {
     } u_r;
 };
 #define u_rval1			u_r.r_val1
-#define u			(*(get_user_struct()))
 
 extern struct usr_user *get_user_struct(void);
 
