@@ -588,6 +588,18 @@ int afs_lockctl(struct vcache * avc, struct AFS_FLOCK * af, int acmd,
 #else
 	) {
 #endif
+
+    if ((avc->f.states & CRO)) {
+	/* for RO volumes, don't do anything for locks; the fileserver doesn't
+	 * even track them. A write lock should not be possible, though. */
+	if (af->l_type == F_WRLCK) {
+	    code = EBADF;
+	} else {
+	    code = 0;
+	}
+	goto done;
+    }
+
     /* Java VMs ask for l_len=(long)-1 regardless of OS/CPU */
     if ((sizeof(af->l_len) == 8) && (af->l_len == 0x7fffffffffffffffLL))
 	af->l_len = 0;
@@ -606,8 +618,8 @@ int afs_lockctl(struct vcache * avc, struct AFS_FLOCK * af, int acmd,
     else if (af->l_type == F_UNLCK)
 	code = LOCK_UN;
     else {
-	afs_PutFakeStat(&fakestate);
-	return EINVAL;		/* unknown lock type */
+	code = EINVAL;		/* unknown lock type */
+	goto done;
     }
     if (((acmd == F_SETLK)
 #if 	defined(AFS_SGI_ENV)
