@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -13,15 +13,13 @@
 #include <afsconfig.h>
 #include "afs/param.h"
 
-RCSID
-    ("$Header: /cvs/openafs/src/afs/afs_cell.c,v 1.34.4.11 2009/05/11 13:28:04 shadow Exp $");
 
 #include "afs/stds.h"
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
 #include "afsincludes.h"	/* Afs-based standard headers */
 #include "afs/afs_stats.h"	/* afs statistics */
 #include "afs/afs_osi.h"
-#include "afs/afs_md5.h"
+#include "hcrypto/md5.h"
 
 /* Local variables. */
 afs_rwlock_t afs_xcell;		/* Export for cmdebug peeking at locks */
@@ -35,7 +33,6 @@ afs_rwlock_t afs_xcell;		/* Export for cmdebug peeking at locks */
  * afs_LookupAFSDB: look up AFSDB for given cell name and create locally
  */
 
-#ifdef AFS_AFSDB_ENV
 afs_rwlock_t afsdb_client_lock;	/* Serializes client requests */
 afs_rwlock_t afsdb_req_lock;	/* Serializes client requests */
 static char afsdb_handler_running;	/* Protected by GLOCK */
@@ -60,7 +57,12 @@ afs_StopAFSDB(void)
 	afs_osi_Wakeup(&afsdb_req);
     } else {
 	afsdb_handler_shutdown = 1;
+#if defined(AFS_SUN5_ENV) || defined(RXK_LISTENER_ENV) || defined(RXK_UPCALL_ENV)
 	afs_termState = AFSOP_STOP_RXEVENT;
+#else
+	afs_termState = AFSOP_STOP_COMPLETE;
+#endif
+	afs_osi_Wakeup(&afs_termState);
     }
 }
 
@@ -76,7 +78,7 @@ int
 afs_AFSDBHandler(char *acellName, int acellNameLen, afs_int32 * kernelMsg)
 {
     afs_int32 timeout, code;
-    afs_int32 cellHosts[MAXCELLHOSTS];
+    afs_int32 cellHosts[AFS_MAXCELLHOSTS];
 
     if (afsdb_handler_shutdown)
 	return -2;
@@ -92,7 +94,7 @@ afs_AFSDBHandler(char *acellName, int acellNameLen, afs_int32 * kernelMsg)
 	if (timeout)
 	    timeout += osi_Time();
 
-	for (i = 0; i < MAXCELLHOSTS; i++) {
+	for (i = 0; i < AFS_MAXCELLHOSTS; i++) {
 	    if (i >= hostCount)
 		cellHosts[i] = 0;
 	    else
@@ -100,10 +102,10 @@ afs_AFSDBHandler(char *acellName, int acellNameLen, afs_int32 * kernelMsg)
 	}
 
 	if (hostCount)
-	    code = afs_NewCell(acellName, cellHosts, CNoSUID, NULL, 0, 0, 
+	    code = afs_NewCell(acellName, cellHosts, CNoSUID, NULL, 0, 0,
 			       timeout);
 
-	if (!hostCount || (code && code != EEXIST)) 
+	if (!hostCount || (code && code != EEXIST))
 	    /* null out the cellname if the lookup failed */
 	    afsdb_req.cellname = NULL;
 	else
@@ -182,25 +184,22 @@ afs_GetCellHostsAFSDB(char *acellName)
     } else
 	return ENOENT;
 }
-#endif
 
 
-/*! 
+/*!
  * Look up AFSDB for given cell name and create locally.
- * \param acellName Cell name. 
+ * \param acellName Cell name.
  */
 void
 afs_LookupAFSDB(char *acellName)
 {
-#ifdef AFS_AFSDB_ENV
     int code;
     char *cellName = afs_strdup(acellName);
 
     code = afs_GetCellHostsAFSDB(cellName);
-    afs_Trace2(afs_iclSetp, CM_TRACE_AFSDB, ICL_TYPE_STRING, cellName, 
+    afs_Trace2(afs_iclSetp, CM_TRACE_AFSDB, ICL_TYPE_STRING, cellName,
                ICL_TYPE_INT32, code);
     afs_osi_FreeStr(cellName);
-#endif
 }
 
 /*
@@ -234,7 +233,8 @@ afs_cellname_new(char *name, afs_int32 cellnum)
     if (cellnum == 0)
 	cellnum = afs_cellnum_next;
 
-    cn = (struct cell_name *)afs_osi_Alloc(sizeof(*cn));
+    cn = afs_osi_Alloc(sizeof(*cn));
+    osi_Assert(cn != NULL);
     cn->next = afs_cellname_head;
     cn->cellnum = cellnum;
     cn->cellname = afs_strdup(name);
@@ -250,7 +250,7 @@ afs_cellname_new(char *name, afs_int32 cellnum)
 /*!
  * Look up a cell name by id.
  * \param cellnum
- * \return 
+ * \return
  */
 static struct cell_name *
 afs_cellname_lookup_id(afs_int32 cellnum)
@@ -267,7 +267,7 @@ afs_cellname_lookup_id(afs_int32 cellnum)
 /*!
  * Look up a cell name.
  * \param name Cell name.
- * \return 
+ * \return
  */
 static struct cell_name *
 afs_cellname_lookup_name(char *name)
@@ -297,7 +297,7 @@ afs_cellname_ref(struct cell_name *cn)
 /*!
  * \brief Load the list of cells from given inode.
  * \param inode Source inode.
- * \param lookupcode 
+ * \param lookupcode
  * \return 0 for success. < 0 for error.
  */
 int
@@ -331,7 +331,6 @@ afs_cellname_init(afs_dcache_id_t *inode, int lookupcode)
 
     while (1) {
 	afs_int32 cellnum, clen, magic;
-	struct cell_name *cn;
 	char *cellname;
 
 	cc = afs_osi_Read(tfile, off, &magic, sizeof(magic));
@@ -369,7 +368,7 @@ afs_cellname_init(afs_dcache_id_t *inode, int lookupcode)
 	    break;
 	}
 
-	cn = afs_cellname_new(cellname, cellnum);
+	afs_cellname_new(cellname, cellnum);
 	afs_osi_Free(cellname, clen + 1);
     }
 
@@ -453,7 +452,7 @@ static int afs_CellOrAliasExists_nl(char *aname);	/* Forward declaration */
 
 /*!
  * Look up cell alias by alias name.
- * \param  alias 
+ * \param  alias
  * \return Found struct or NULL.
  */
 static struct cell_alias *
@@ -469,7 +468,7 @@ afs_FindCellAlias(char *alias)
 
 /*!
  * Get cell alias by index (starting at 0).
- * \param index Cell index. 
+ * \param index Cell index.
  * \return Found struct or null.
  */
 struct cell_alias *
@@ -489,8 +488,8 @@ afs_GetCellAlias(int index)
 
  /*!
   * Put back a cell alias returned by Find or Get.
-  * \param a Alias. 
-  * \return 
+  * \param a Alias.
+  * \return
   */
 void
 afs_PutCellAlias(struct cell_alias *a)
@@ -502,7 +501,7 @@ afs_PutCellAlias(struct cell_alias *a)
  * Create new cell alias entry and update dynroot vnode.
  * \param alias
  * \param cell
- * \return 
+ * \return
  */
 afs_int32
 afs_NewCellAlias(char *alias, char *cell)
@@ -516,7 +515,8 @@ afs_NewCellAlias(char *alias, char *cell)
     }
 
     UpgradeSToWLock(&afs_xcell, 682);
-    tc = (struct cell_alias *)afs_osi_Alloc(sizeof(struct cell_alias));
+    tc = afs_osi_Alloc(sizeof(struct cell_alias));
+    osi_Assert(tc != NULL);
     tc->alias = afs_strdup(alias);
     tc->cell = afs_strdup(cell);
     tc->next = afs_cellalias_head;
@@ -554,12 +554,12 @@ afs_NewCellAlias(char *alias, char *cell)
  */
 
 struct afs_q CellLRU;		/* Export for kdump */
-static char *afs_thiscell;
+static char *afs_thiscell = NULL;
 afs_int32 afs_cellindex;	/* Export for kdump */
 
 /*!
  * Bump given cell up to the front of the LRU queue.
- * \param c Cell to set. 
+ * \param c Cell to set.
  */
 static void
 afs_UpdateCellLRU(struct cell *c)
@@ -573,7 +573,7 @@ afs_UpdateCellLRU(struct cell *c)
 /*!
  * Look up cell information in AFSDB if timeout expired
  * \param ac Cell to be refreshed.
- * \return 
+ * \return
  */
 static void
 afs_RefreshCell(struct cell *ac)
@@ -584,7 +584,7 @@ afs_RefreshCell(struct cell *ac)
 	afs_LookupAFSDB(ac->cellName);
 }
 
-/*! 
+/*!
  * Execute a callback for each existing cell, without a lock on afs_xcell.
  * Iterate on CellLRU, and execute a callback for each cell until given arguments are met.
  * \see afs_TraverseCells
@@ -621,7 +621,7 @@ afs_TraverseCells_nl(void *(*cb) (struct cell *, void *), void *arg)
  * Execute a callback for each existing cell, with a lock on afs_xcell.
  * \see afs_TraverseCells_nl
  * \param cb Traversal callback for each cell.
- * \param arg 
+ * \param arg
  * \return Found data or NULL.
  */
 void *
@@ -638,7 +638,7 @@ afs_TraverseCells(void *(*cb) (struct cell *, void *), void *arg)
 
 /*!
  * Useful traversal callback: Match by name.
- * \param cell 
+ * \param cell
  * \param arg Cell name (compared with cell->cellName).
  * \return Returns found cell or NULL.
  */
@@ -655,7 +655,7 @@ afs_choose_cell_by_name(struct cell *cell, void *arg)
 
 /*!
  * Useful traversal callback: Match by handle.
- * \param cell 
+ * \param cell
  * \param arg Cell handle (compared with cell->cellHandle).
  * \return Returns found cell or NULL.
  */
@@ -672,7 +672,7 @@ afs_choose_cell_by_handle(struct cell *cell, void *arg)
 
 /*!
  * Useful traversal callback: Match by cell number.
- * \param cell 
+ * \param cell
  * \param arg Cell number (compared with cell->cellNum).
  * \return Returns found cell or NULL.
  */
@@ -684,7 +684,7 @@ afs_choose_cell_by_num(struct cell *cell, void *arg)
 
 /*!
  * Useful traversal callback: Match by index.
- * \param cell 
+ * \param cell
  * \param arg Cell index (compared with cell->cellIndex).
  * \return Returns found cell or NULL.
  */
@@ -699,7 +699,7 @@ afs_choose_cell_by_index(struct cell *cell, void *arg)
  * Does not check AFSDB.
  * \param acellName Cell name.
  * \param locktype Type of lock to be used (not used).
- * \return 
+ * \return
  */
 static struct cell *
 afs_FindCellByName_nl(char *acellName, afs_int32 locktype)
@@ -712,7 +712,7 @@ afs_FindCellByName_nl(char *acellName, afs_int32 locktype)
  * Does not check AFSDB.
  * \param acellName Cell name.
  * \param locktype Type of lock to be used.
- * \return 
+ * \return
  */
 static struct cell *
 afs_FindCellByName(char *acellName, afs_int32 locktype)
@@ -724,7 +724,7 @@ afs_FindCellByName(char *acellName, afs_int32 locktype)
  * Same as FindCellByName but tries AFSDB if not found.
  * \param acellName Cell name.
  * \param locktype Type of lock to be used.
- * \return 
+ * \return
  */
 struct cell *
 afs_GetCellByName(char *acellName, afs_int32 locktype)
@@ -748,8 +748,8 @@ afs_GetCellByName(char *acellName, afs_int32 locktype)
 /*!
  * Return a cell with a given cell number.
  * \param cellnum Cell number.
- * \param locktype Lock to be used. 
- * \return 
+ * \param locktype Lock to be used.
+ * \return
  */
 struct cell *
 afs_GetCell(afs_int32 cellnum, afs_int32 locktype)
@@ -774,7 +774,7 @@ afs_GetCell(afs_int32 cellnum, afs_int32 locktype)
  * Same as GetCell, but does not try to refresh the data.
  * \param cellnum Cell number.
  * \param locktype What lock should be used.
- * \return 
+ * \return
  */
 struct cell *
 afs_GetCellStale(afs_int32 cellnum, afs_int32 locktype)
@@ -793,7 +793,7 @@ afs_GetCellStale(afs_int32 cellnum, afs_int32 locktype)
  * Return a cell with a given index number (starting at 0). Update CellLRU as well.
  * \param index
  * \param locktype Type of lock used.
- * \return 
+ * \return
  */
 struct cell *
 afs_GetCellByIndex(afs_int32 index, afs_int32 locktype)
@@ -810,7 +810,7 @@ afs_GetCellByIndex(afs_int32 index, afs_int32 locktype)
  * Return a cell with a given handle..
  * \param index
  * \param locktype Type of lock used.
- * \return 
+ * \return
  */
 struct cell *
 afs_GetCellByHandle(void *handle, afs_int32 locktype)
@@ -826,7 +826,7 @@ afs_GetCellByHandle(void *handle, afs_int32 locktype)
 /*!
  * Return primary cell, if any.
  * \param locktype Type of lock used.
- * \return 
+ * \return
  */
 struct cell *
 afs_GetPrimaryCell(afs_int32 locktype)
@@ -834,10 +834,28 @@ afs_GetPrimaryCell(afs_int32 locktype)
     return afs_GetCellByName(afs_thiscell, locktype);
 }
 
+/*!
+ * Return number of the primary cell.
+ * \return
+ *    Cell number, or 0 if primary cell not found
+ */
+afs_int32
+afs_GetPrimaryCellNum(void)
+{
+    struct cell *cell;
+    afs_int32 cellNum = 0;
+    cell = afs_GetPrimaryCell(READ_LOCK);
+    if (cell) {
+	cellNum = cell->cellNum;
+	afs_PutCell(cell, READ_LOCK);
+    }
+    return cellNum;
+}
+
 /*!
  * Returns true if the given cell is the primary cell.
  * \param cell
- * \return 
+ * \return
  */
 int
 afs_IsPrimaryCell(struct cell *cell)
@@ -858,8 +876,8 @@ afs_IsPrimaryCell(struct cell *cell)
 
 /*!
  * Returns afs_IsPrimaryCell(afs_GetCell(cellnum)).
- * \param cellnum 
- * \return 
+ * \param cellnum
+ * \return
  */
 int
 afs_IsPrimaryCellNum(afs_int32 cellnum)
@@ -878,7 +896,7 @@ afs_IsPrimaryCellNum(afs_int32 cellnum)
 
 /*!
  * Set the primary cell name to the given cell name.
- * \param acellName Cell name. 
+ * \param acellName Cell name.
  * \return 0 for success, < 0 for error.
  */
 afs_int32
@@ -893,15 +911,15 @@ afs_SetPrimaryCell(char *acellName)
 }
 
 /*!
- * Create or update a cell entry. 
+ * Create or update a cell entry.
  * \param acellName Name of cell.
  * \param acellHosts Array of hosts that this cell has.
  * \param aflags Cell flags.
- * \param linkedcname 
+ * \param linkedcname
  * \param fsport File server port.
  * \param vlport Volume server port.
  * \param timeout Cell timeout value, 0 means static AFSDB entry.
- * \return 
+ * \return
  */
 afs_int32
 afs_NewCell(char *acellName, afs_int32 * acellHosts, int aflags,
@@ -909,6 +927,7 @@ afs_NewCell(char *acellName, afs_int32 * acellHosts, int aflags,
 {
     struct cell *tc, *tcl = 0;
     afs_int32 i, newc = 0, code = 0;
+    struct md5 m;
 
     AFS_STATCNT(afs_NewCell);
 
@@ -918,13 +937,16 @@ afs_NewCell(char *acellName, afs_int32 * acellHosts, int aflags,
     if (tc) {
 	aflags &= ~CNoSUID;
     } else {
-	tc = (struct cell *)afs_osi_Alloc(sizeof(struct cell));
-	memset((char *)tc, 0, sizeof(*tc));
+	tc = afs_osi_Alloc(sizeof(struct cell));
+	osi_Assert(tc != NULL);
+	memset(tc, 0, sizeof(*tc));
 	tc->cellName = afs_strdup(acellName);
 	tc->fsport = AFS_FSPORT;
 	tc->vlport = AFS_VLPORT;
-	AFS_MD5_String(tc->cellHandle, tc->cellName, strlen(tc->cellName));
-	RWLOCK_INIT(&tc->lock, "cell lock");
+	MD5_Init(&m);
+	MD5_Update(&m, tc->cellName, strlen(tc->cellName));
+	MD5_Final(tc->cellHandle, &m);
+	AFS_RWLOCK_INIT(&tc->lock, "cell lock");
 	newc = 1;
 	aflags |= CNoSUID;
     }
@@ -944,7 +966,7 @@ afs_NewCell(char *acellName, afs_int32 * acellHosts, int aflags,
     /* we don't want to keep pinging old vlservers which were down,
      * since they don't matter any more.  It's easier to do this than
      * to remove the server from its various hash tables. */
-    for (i = 0; i < MAXCELLHOSTS; i++) {
+    for (i = 0; i < AFS_MAXCELLHOSTS; i++) {
 	if (!tc->cellHosts[i])
 	    break;
 	tc->cellHosts[i]->flags &= ~SRVR_ISDOWN;
@@ -975,23 +997,23 @@ afs_NewCell(char *acellName, afs_int32 * acellHosts, int aflags,
     }
     tc->states |= aflags;
     tc->timeout = timeout;
-    
-    memset((char *)tc->cellHosts, 0, sizeof(tc->cellHosts));
-    for (i = 0; i < MAXCELLHOSTS; i++) {
-	/* Get server for each host and link this cell in.*/	
+
+    memset(tc->cellHosts, 0, sizeof(tc->cellHosts));
+    for (i = 0; i < AFS_MAXCELLHOSTS; i++) {
+	/* Get server for each host and link this cell in.*/
 	struct server *ts;
 	afs_uint32 temp = acellHosts[i];
 	if (!temp)
 	    break;
-	ts = afs_GetServer(&temp, 1, 0, tc->vlport, WRITE_LOCK, NULL, 0);
+	ts = afs_GetServer(&temp, 1, 0, tc->vlport, WRITE_LOCK, NULL, 0, NULL);
 	ts->cell = tc;
 	ts->flags &= ~SRVR_ISGONE;
 	/* Set the server as a host of the new cell. */
 	tc->cellHosts[i] = ts;
 	afs_PutServer(ts, WRITE_LOCK);
     }
-    afs_SortServers(tc->cellHosts, MAXCELLHOSTS);	/* randomize servers */
-	
+    afs_SortServers(tc->cellHosts, AFS_MAXCELLHOSTS);	/* randomize servers */
+
     /* New cell: Build and add to LRU cell queue. */
     if (newc) {
 	struct cell_name *cn;
@@ -1010,7 +1032,8 @@ afs_NewCell(char *acellName, afs_int32 * acellHosts, int aflags,
     ReleaseWriteLock(&tc->lock);
     ReleaseWriteLock(&afs_xcell);
     afs_PutCell(tc, 0);
-    afs_DynrootInvalidate();
+    if (!(aflags & CHush))
+	afs_DynrootInvalidate();
     return 0;
 
   bad:
@@ -1040,11 +1063,16 @@ afs_NewCell(char *acellName, afs_int32 * acellHosts, int aflags,
 void
 afs_CellInit(void)
 {
-    RWLOCK_INIT(&afs_xcell, "afs_xcell");
-#ifdef AFS_AFSDB_ENV
-    RWLOCK_INIT(&afsdb_client_lock, "afsdb_client_lock");
-    RWLOCK_INIT(&afsdb_req_lock, "afsdb_req_lock");
-#endif
+    static char CellInit_done = 0;
+
+    if (CellInit_done)
+	return;
+
+    CellInit_done = 1;
+
+    AFS_RWLOCK_INIT(&afs_xcell, "afs_xcell");
+    AFS_RWLOCK_INIT(&afsdb_client_lock, "afsdb_client_lock");
+    AFS_RWLOCK_INIT(&afsdb_req_lock, "afsdb_req_lock");
     QInit(&CellLRU);
 
     afs_cellindex = 0;
@@ -1060,7 +1088,12 @@ shutdown_cell(void)
     struct afs_q *cq, *tq;
     struct cell *tc;
 
-    RWLOCK_INIT(&afs_xcell, "afs_xcell");
+#ifdef AFS_CACHE_VNODE_PATH
+    if (cacheDiskType != AFS_FCACHE_TYPE_MEM) {
+	afs_osi_FreeStr(afs_cellname_inode.ufs);
+    }
+#endif
+    AFS_RWLOCK_INIT(&afs_xcell, "afs_xcell");
 
     for (cq = CellLRU.next; cq != &CellLRU; cq = tq) {
 	tc = QTOC(cq);
@@ -1087,7 +1120,7 @@ shutdown_cell(void)
 /*!
  * Remove a server from a cell's server list.
  * \param srvp Server to be removed.
- * \return 
+ * \return
  */
 void
 afs_RemoveCellEntry(struct server *srvp)
@@ -1101,7 +1134,7 @@ afs_RemoveCellEntry(struct server *srvp)
 
     /* Remove the server structure from the cell list - if there */
     ObtainWriteLock(&tc->lock, 200);
-    for (j = k = 0; j < MAXCELLHOSTS; j++) {
+    for (j = k = 0; j < AFS_MAXCELLHOSTS; j++) {
 	if (!tc->cellHosts[j])
 	    break;
 	if (tc->cellHosts[j] != srvp) {
@@ -1111,7 +1144,7 @@ afs_RemoveCellEntry(struct server *srvp)
     if (k == 0) {
 	/* What do we do if we remove the last one? */
     }
-    for (; k < MAXCELLHOSTS; k++) {
+    for (; k < AFS_MAXCELLHOSTS; k++) {
 	tc->cellHosts[k] = 0;
     }
     ReleaseWriteLock(&tc->lock);
@@ -1119,8 +1152,8 @@ afs_RemoveCellEntry(struct server *srvp)
 
 /*!
  * Check if the given name exists as a cell or alias. Does not lock afs_xcell.
- * \param aname 
- * \return 
+ * \param aname
+ * \return
  */
 static int
 afs_CellOrAliasExists_nl(char *aname)
@@ -1146,7 +1179,7 @@ afs_CellOrAliasExists_nl(char *aname)
 /*!
  * Check if the given name exists as a cell or alias. Locks afs_xcell.
  * \param aname
- * \return 
+ * \return
  */
 int
 afs_CellOrAliasExists(char *aname)
@@ -1162,7 +1195,7 @@ afs_CellOrAliasExists(char *aname)
 
 /*!
  * Check if a cell number is valid (also set the used flag).
- * \param cellnum 
+ * \param cellnum
  * \return 1 - true, 0 - false
  */
 int
