@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -98,6 +98,7 @@
 #define PTS_LstOwnEvent    "AFS_PTS_LstOwn"
 #define PTS_IsMemOfEvent   "AFS_PTS_IsMemOf"
 #define PTS_UpdEntEvent    "AFS_PTS_UpdEnt"
+#define PTS_LstSGrps       "AFS_PTS_LstSGrps"
 
 #define BUDB_StartEvent    "AFS_BUDB_Start"
 #define BUDB_FinishEvent   "AFS_BUDB_Finish"
@@ -290,4 +291,8 @@
 /* prototypes for audit functions */
 int osi_audit(char *audEvent, afs_int32 errCode, ...);
 int osi_auditU(struct rx_call *call, char *audEvent, int errCode, ...);
-int osi_audit_file(FILE *out);
+int osi_audit_file(const char *filename);
+void osi_audit_init(void);
+int osi_audit_interface(const char *interface);
+void osi_audit_set_user_check(void *rock, int (*islocal)(void *rock, char *name, char *inst, char *cell));
+void audit_PrintStats(FILE *out);
