@@ -94,8 +94,11 @@ struct afsconf_dir {
     struct afsconf_entry *entries;	/* list of cell entries */
     struct opr_queue keyList;		/* list of keys */
     afs_int32 timeRead;		/* time stamp of file last read */
+    afs_int32 timeCheck;	/* time of last check for update */
     struct afsconf_aliasentry *alias_entries;	/* cell aliases */
     afsconf_secflags securityFlags;
+    struct afsconf_realms *local_realms;        /* local realms */
+    struct afsconf_realms *exclusions;          /* excluded principals */
 };
 
 extern afs_int32 afsconf_FindService(const char *aname);
@@ -251,6 +254,24 @@ extern int afsconf_SuperIdentity(struct afsconf_dir *, struct rx_call *,
 				 struct rx_identity **);
 extern int afsconf_IsSuperIdentity(struct afsconf_dir *, struct rx_identity *);
 
+/* realms.c */
+extern int afsconf_SetLocalRealm(const char *realm);
+extern int afsconf_IsLocalRealmMatch(struct afsconf_dir *dir, afs_int32 * local,
+				const char *name, const char *instance,
+				const char *cell);
+
+/* netrestrict.c */
+
+extern int afsconf_ParseNetRestrictFile(afs_uint32 outAddrs[],
+					afs_uint32 * mask, afs_uint32 * mtu,
+					afs_uint32 maxAddrs, afs_uint32 * nAddrs,
+					char reason[], const char *fileName);
+
+extern int afsconf_ParseNetFiles(afs_uint32 addrbuf[], afs_uint32 maskbuf[],
+				 afs_uint32 mtubuf[], afs_uint32 max,
+				 char reason[], const char *niFileName,
+				 const char *nrFileName);
+
 /* some well-known ports and their names; new additions to table in cellconfig.c, too */
 #define	AFSCONF_FILESERVICE		"afs"
 #define	AFSCONF_FILEPORT		7000
