@@ -88,9 +88,6 @@ static char lcell[MAXCELLCHARS];
 #define BUFSIZ 4096
 #endif
 
-#ifdef	AFS_HPUX_ENV
-#include <unistd.h>
-#endif
 #if	defined(AFS_AIX_ENV) || defined(AFS_SUN5_ENV)
 static struct flock fileWlock = { F_WRLCK, 0, 0, 0, 0, 0 };
 static struct flock fileRlock = { F_RDLCK, 0, 0, 0, 0, 0 };
@@ -490,7 +487,10 @@ ktc_GetTokenEx(char *cellName, struct ktc_setTokenData **tokenSet) {
 	memset(ktcToken, 0, sizeof(struct ktc_token));
 
 	strcpy(server.name, "afs");
-	strcpy(server.cell, cellName);
+
+	if (cellName != NULL)
+	    strcpy(server.cell, cellName);
+
 	code = GetToken(&server, ktcToken, sizeof(struct ktc_token),
 			 NULL /*client*/, &viceid);
 	if (code == 0) {
@@ -1109,10 +1109,6 @@ ktc_curpag(void)
   */
 
 #if 0
-#include <stdio.h>
-#include <errno.h>
-#include <sys/types.h>
-#include <sys/stat.h>
 #include <sys/file.h>
 #include <krb.h>
 #endif
@@ -1827,7 +1823,7 @@ extern char **environ;
 
     for (senv = environ, numenv = 0; *senv; senv++)
 	numenv++;
-    newenv = (char **)malloc((numenv + 2) * sizeof(char *));
+    newenv = malloc((numenv + 2) * sizeof(char *));
 
     for (senv = environ, denv = newenv; *senv; senv++) {
 	if (strncmp(*senv, "KRBTKFILE=", 10) != 0 &&
