@@ -45,6 +45,7 @@
 
 #include "bosint.h"
 #include "bnode.h"
+#include "bnode_internal.h"
 #include "bosprototypes.h"
 
 #define BOZO_LWP_STACKSIZE	16000
@@ -54,6 +55,10 @@ struct afsconf_dir *bozo_confdir = 0;	/* bozo configuration dir */
 static PROCESS bozo_pid;
 const char *bozo_fileName;
 FILE *bozo_logFile;
+#ifndef AFS_NT40_ENV
+static int bozo_argc = 0;
+static char** bozo_argv = NULL;
+#endif
 
 const char *DoCore;
 int DoLogging = 0;
@@ -95,67 +100,36 @@ bozo_rxstat_userok(struct rx_call *call)
     return afsconf_SuperUser(bozo_confdir, call, NULL);
 }
 
+/**
+ * Return true if this name is a member of the local realm.
+ */
+int
+bozo_IsLocalRealmMatch(void *rock, char *name, char *inst, char *cell)
+{
+    struct afsconf_dir *dir = (struct afsconf_dir *)rock;
+    afs_int32 islocal = 0;	/* default to no */
+    int code;
+
+    code = afsconf_IsLocalRealmMatch(dir, &islocal, name, inst, cell);
+    if (code) {
+	bozo_Log("Failed local realm check; code=%d, name=%s, inst=%s, cell=%s\n",
+		 code, name, inst, cell);
+    }
+    return islocal;
+}
+
 /* restart bozo process */
 int
 bozo_ReBozo(void)
 {
 #ifdef AFS_NT40_ENV
-    /* exit with restart code; SCM integrator process will restart bosserver */
-    int status = BOSEXIT_RESTART;
-
-    /* if noauth flag is set, pass "-noauth" to new bosserver */
-    if (afsconf_GetNoAuthFlag(bozo_confdir)) {
-	status |= BOSEXIT_NOAUTH_FLAG;
-    }
-    /* if logging is on, pass "-log" to new bosserver */
-    if (DoLogging) {
-	status |= BOSEXIT_LOGGING_FLAG;
-    }
-    /* if rxbind is set, pass "-rxbind" to new bosserver */
-    if (rxBind) {
-	status |= BOSEXIT_RXBIND_FLAG;
-    }
-    exit(status);
+    /* exit with restart code; SCM integrator process will restart bosserver with
+       the same arguments */
+    exit(BOSEXIT_RESTART);
 #else
     /* exec new bosserver process */
-    char *argv[4];
     int i = 0;
 
-    argv[i] = (char *)AFSDIR_SERVER_BOSVR_FILEPATH;
-    i++;
-
-    /* if noauth flag is set, pass "-noauth" to new bosserver */
-    if (afsconf_GetNoAuthFlag(bozo_confdir)) {
-	argv[i] = "-noauth";
-	i++;
-    }
-    /* if logging is on, pass "-log" to new bosserver */
-    if (DoLogging) {
-	argv[i] = "-log";
-	i++;
-    }
-    /* if rxbind is set, pass "-rxbind" to new bosserver */
-    if (rxBind) {
-	argv[i] = "-rxbind";
-	i++;
-    }
-#ifndef AFS_NT40_ENV
-    /* if syslog logging is on, pass "-syslog" to new bosserver */
-    if (DoSyslog) {
-	char *arg = (char *)malloc(40);	/* enough for -syslog=# */
-	if (DoSyslogFacility != LOG_DAEMON) {
-	    snprintf(arg, 40, "-syslog=%d", DoSyslogFacility);
-	} else {
-	    strcpy(arg, "-syslog");
-	}
-	argv[i] = arg;
-	i++;
-    }
-#endif
-
-    /* null-terminate argument list */
-    argv[i] = NULL;
-
     /* close random fd's */
     for (i = 3; i < 64; i++) {
 	close(i);
@@ -163,7 +137,7 @@ bozo_ReBozo(void)
 
     unlink(AFSDIR_SERVER_BOZRXBIND_FILEPATH);
 
-    execv(argv[0], argv);	/* should not return */
+    execv(bozo_argv[0], bozo_argv);	/* should not return */
     _exit(1);
 #endif /* AFS_NT40_ENV */
 }
@@ -295,16 +269,14 @@ ReadBozoFile(char *aname)
 	/* if BozoInit exists and BosConfig doesn't, try a rename */
 	if (access(AFSDIR_SERVER_BOZINIT_FILEPATH, 0) == 0
 	    && access(AFSDIR_SERVER_BOZCONF_FILEPATH, 0) != 0) {
-	    code =
-		renamefile(AFSDIR_SERVER_BOZINIT_FILEPATH,
-			   AFSDIR_SERVER_BOZCONF_FILEPATH);
+	    code = rk_rename(AFSDIR_SERVER_BOZINIT_FILEPATH,
+			     AFSDIR_SERVER_BOZCONF_FILEPATH);
 	    if (code < 0)
 		perror("bosconfig rename");
 	}
 	if (access(AFSDIR_SERVER_BOZCONFNEW_FILEPATH, 0) == 0) {
-	    code =
-		renamefile(AFSDIR_SERVER_BOZCONFNEW_FILEPATH,
-			   AFSDIR_SERVER_BOZCONF_FILEPATH);
+	    code = rk_rename(AFSDIR_SERVER_BOZCONFNEW_FILEPATH,
+			     AFSDIR_SERVER_BOZCONF_FILEPATH);
 	    if (code < 0)
 		perror("bosconfig rename");
 	}
@@ -419,7 +391,7 @@ ReadBozoFile(char *aname)
 		goto fail;	/* no "parm " either */
 	    }
 	    if (!parms[i])	/* make sure there's space */
-		parms[i] = (char *)malloc(BOZO_BSSIZE);
+		parms[i] = malloc(BOZO_BSSIZE);
 	    strcpy(parms[i], tbuffer + 5);	/* remember the parameter for later */
 	    thisparms[i] = parms[i];
 	}
@@ -493,7 +465,7 @@ WriteBozoFile(char *aname)
 	unlink(tbuffer);
 	return -1;
     }
-    code = renamefile(tbuffer, aname);
+    code = rk_rename(tbuffer, aname);
     if (code) {
 	unlink(tbuffer);
 	return -1;
@@ -713,28 +685,18 @@ static char *
 make_pid_filename(char *ainst, char *aname)
 {
     char *buffer = NULL;
-    int length;
 
-    length = strlen(DoPidFiles) + strlen(ainst) + 6;
     if (aname && *aname) {
-	length += strlen(aname) + 1;
-    }
-    buffer = malloc(length * sizeof(char));
-    if (!buffer) {
-	if (aname) {
+	asprintf(&buffer, "%s/%s.%s.pid", DoPidFiles, ainst, aname);
+	if (buffer == NULL)
 	    bozo_Log("Failed to alloc pid filename buffer for %s.%s.\n",
 		     ainst, aname);
-	} else {
-	    bozo_Log("Failed to alloc pid filename buffer for %s.\n", ainst);
-	}
     } else {
-	if (aname && *aname) {
-	    snprintf(buffer, length, "%s/%s.%s.pid", DoPidFiles, ainst,
-		     aname);
-	} else {
-	    snprintf(buffer, length, "%s/%s.pid", DoPidFiles, ainst);
-	}
+	asprintf(&buffer, "%s/%s.pid", DoPidFiles, ainst);
+	if (buffer == NULL)
+	    bozo_Log("Failed to alloc pid filename buffer for %s.\n", ainst);
     }
+
     return buffer;
 }
 
@@ -806,16 +768,17 @@ bozo_CreateRxBindFile(afs_uint32 host)
     char buffer[16];
     FILE *fp;
 
-    if (host == htonl(INADDR_ANY)) {
-	host = htonl(0x7f000001);
-    }
-
     afs_inet_ntoa_r(host, buffer);
     bozo_Log("Listening on %s:%d\n", buffer, AFSCONF_NANNYPORT);
     if ((fp = fopen(AFSDIR_SERVER_BOZRXBIND_FILEPATH, "w")) == NULL) {
 	bozo_Log("Unable to open rxbind address file: %s, code=%d\n",
 		 AFSDIR_SERVER_BOZRXBIND_FILEPATH, errno);
     } else {
+	/* If listening on any interface, write the loopback interface
+	   to the rxbind file to give local scripts a usable addresss. */
+	if (host == htonl(INADDR_ANY)) {
+	    afs_inet_ntoa_r(htonl(0x7f000001), buffer);
+	}
 	fprintf(fp, "%s\n", buffer);
 	fclose(fp);
     }
@@ -902,8 +865,23 @@ main(int argc, char **argv, char **envp)
     }
 #endif
 
+#ifndef AFS_NT40_ENV
+    /* save args for restart */
+    bozo_argc = argc;
+    bozo_argv = malloc((argc+1) * sizeof(char*));
+    if (!bozo_argv) {
+	fprintf(stderr, "%s: Failed to allocate argument list.\n", argv[0]);
+	exit(1);
+    }
+    bozo_argv[0] = (char*)AFSDIR_SERVER_BOSVR_FILEPATH; /* expected path */
+    bozo_argv[bozo_argc] = NULL; /* null terminate list */
+#endif	/* AFS_NT40_ENV */
+
     /* parse cmd line */
     for (code = 1; code < argc; code++) {
+#ifndef AFS_NT40_ENV
+	bozo_argv[code] = argv[code];
+#endif	/* AFS_NT40_ENV */
 	if (strcmp(argv[code], "-noauth") == 0) {
 	    /* set noauth flag */
 	    noAuth = 1;
@@ -970,7 +948,7 @@ main(int argc, char **argv, char **envp)
 #ifndef AFS_NT40_ENV
 	    printf("Usage: bosserver [-noauth] [-log] "
 		   "[-auditlog <log path>] "
-		   "[-audit-interafce <file|sysvmq> (default is file)] "
+		   "[-audit-interface <file|sysvmq> (default is file)] "
 		   "[-rxmaxmtu <bytes>] [-rxbind] [-allow-dotted-principals]"
 		   "[-syslog[=FACILITY]] "
 		   "[-restricted] "
@@ -981,7 +959,7 @@ main(int argc, char **argv, char **envp)
 #else
 	    printf("Usage: bosserver [-noauth] [-log] "
 		   "[-auditlog <log path>] "
-		   "[-audit-interafce <file|sysvmq> (default is file)] "
+		   "[-audit-interface <file|sysvmq> (default is file)] "
 		   "[-rxmaxmtu <bytes>] [-rxbind] [-allow-dotted-principals]"
 		   "[-restricted] "
 		   "[-enable_peer_stats] [-enable_process_stats] "
@@ -1047,7 +1025,7 @@ main(int argc, char **argv, char **envp)
 	) {
 	strcpy(namebuf, AFSDIR_BOZLOG_FILE);
 	strcat(namebuf, ".old");
-	renamefile(AFSDIR_BOZLOG_FILE, namebuf);	/* try rename first */
+	rk_rename(AFSDIR_BOZLOG_FILE, namebuf);	/* try rename first */
 	bozo_logFile = fopen(AFSDIR_BOZLOG_FILE, "a");
 	if (!bozo_logFile) {
 	    printf("bosserver: can't initialize log file (%s).\n",
@@ -1084,10 +1062,10 @@ main(int argc, char **argv, char **envp)
 	if (AFSDIR_SERVER_NETRESTRICT_FILEPATH ||
 	    AFSDIR_SERVER_NETINFO_FILEPATH) {
 	    char reason[1024];
-	    ccode = parseNetFiles(SHostAddrs, NULL, NULL,
-	                          ADDRSPERSITE, reason,
-	                          AFSDIR_SERVER_NETINFO_FILEPATH,
-	                          AFSDIR_SERVER_NETRESTRICT_FILEPATH);
+	    ccode = afsconf_ParseNetFiles(SHostAddrs, NULL, NULL,
+			                  ADDRSPERSITE, reason,
+	                                  AFSDIR_SERVER_NETINFO_FILEPATH,
+	                                  AFSDIR_SERVER_NETRESTRICT_FILEPATH);
         } else {
             ccode = rx_getAllAddr(SHostAddrs, ADDRSPERSITE);
         }
@@ -1112,6 +1090,16 @@ main(int argc, char **argv, char **envp)
 	exit(code);
     }
 
+    /* Disable jumbograms */
+    rx_SetNoJumbo();
+
+    if (rxMaxMTU != -1) {
+	if (rx_SetMaxMTU(rxMaxMTU) != 0) {
+	    bozo_Log("bosserver: rxMaxMTU %d is invalid\n", rxMaxMTU);
+	    exit(1);
+	}
+    }
+
     code = LWP_CreateProcess(BozoDaemon, BOZO_LWP_STACKSIZE, /* priority */ 1,
 			     /* param */ NULL , "bozo-the-clown",
 			     &bozo_pid);
@@ -1151,6 +1139,9 @@ main(int argc, char **argv, char **envp)
 	}
     }
 
+    /* initialize audit user check */
+    osi_audit_set_user_check(tdir, bozo_IsLocalRealmMatch);
+
     /* read init file, starting up programs */
     if ((code = ReadBozoFile(0))) {
 	bozo_Log
@@ -1174,16 +1165,6 @@ main(int argc, char **argv, char **envp)
 	bozo_CreatePidFile("bosserver", NULL, getpid());
     }
 
-    /* Disable jumbograms */
-    rx_SetNoJumbo();
-
-    if (rxMaxMTU != -1) {
-	if (rx_SetMaxMTU(rxMaxMTU) != 0) {
-	    bozo_Log("bosserver: rxMaxMTU %d is invalid\n", rxMaxMTU);
-	    exit(1);
-	}
-    }
-
     tservice = rx_NewServiceHost(host, 0, /* service id */ 1,
 			         "bozo", securityClasses, numClasses,
 				 BOZO_ExecuteRequest);
