@@ -17,8 +17,10 @@
 #include <lwp.h>
 #include <rx/rx.h>
 #include <afs/afsutil.h>
+#include <opr/queue.h>
 
 #include "bnode.h"
+#include "bnode_internal.h"
 #include "bosprototypes.h"
 
 extern char *DoPidFiles;
@@ -291,7 +293,7 @@ SetSalFlag(struct fsbnode *abnode, int aflag)
     if (abnode->salsrvcmd == NULL) {
 	abnode->needsSalvage = aflag;
 	strcompose(tbuffer, AFSDIR_PATH_MAX, AFSDIR_SERVER_LOCAL_DIRPATH, "/",
-		   SALFILE, abnode->b.name, NULL);
+		   SALFILE, abnode->b.name, (char *)NULL);
 	if (aflag) {
 	    fd = open(tbuffer, O_CREAT | O_TRUNC | O_RDWR, 0666);
 	    close(fd);
@@ -313,7 +315,7 @@ RestoreSalFlag(struct fsbnode *abnode)
 	abnode->needsSalvage = 0;
     } else {
 	strcompose(tbuffer, AFSDIR_PATH_MAX, AFSDIR_SERVER_LOCAL_DIRPATH, "/",
-		   SALFILE, abnode->b.name, NULL);
+		   SALFILE, abnode->b.name, (char *)NULL);
 	if (access(tbuffer, 0) == 0) {
 	    /* file exists, so need to salvage */
 	    abnode->needsSalvage = 1;
@@ -324,15 +326,6 @@ RestoreSalFlag(struct fsbnode *abnode)
     return 0;
 }
 
-char *
-copystr(char *a)
-{
-    char *b;
-    b = (char *)malloc(strlen(a) + 1);
-    strcpy(b, a);
-    return b;
-}
-
 static int
 fs_delete(struct bnode *bn)
 {
@@ -439,12 +432,11 @@ fs_create(char *ainstance, char *afilecmd, char *avolcmd, char *asalcmd,
 	}
     }
 
-    te = (struct fsbnode *)malloc(sizeof(struct fsbnode));
+    te = calloc(1, sizeof(struct fsbnode));
     if (te == NULL) {
 	bailout = 1;
 	goto done;
     }
-    memset(te, 0, sizeof(struct fsbnode));
     te->filecmd = fileCmdpath;
     te->volcmd = volCmdpath;
     te->salsrvcmd = NULL;
@@ -458,7 +450,7 @@ fs_create(char *ainstance, char *afilecmd, char *avolcmd, char *asalcmd,
 	goto done;
     }
     bnode_SetTimeout(fsbnode2bnode(te), POLLTIME);
-    		/* ask for timeout activations every 10 seconds */
+		/* ask for timeout activations every 20 seconds */
     RestoreSalFlag(te);		/* restore needsSalvage flag based on file's existence */
     SetNeedsClock(te);		/* compute needsClock field */
 
@@ -568,12 +560,11 @@ dafs_create(char *ainstance, char *afilecmd, char *avolcmd,
 	}
     }
 
-    te = (struct fsbnode *)malloc(sizeof(struct fsbnode));
+    te = calloc(1, sizeof(struct fsbnode));
     if (te == NULL) {
 	bailout = 1;
 	goto done;
     }
-    memset(te, 0, sizeof(struct fsbnode));
     te->filecmd = fileCmdpath;
     te->volcmd = volCmdpath;
     te->salsrvcmd = salsrvCmdpath;
@@ -587,7 +578,7 @@ dafs_create(char *ainstance, char *afilecmd, char *avolcmd,
 	goto done;
     }
     bnode_SetTimeout(fsbnode2bnode(te), POLLTIME);
-    		/* ask for timeout activations every 10 seconds */
+		/* ask for timeout activations every 20 seconds */
     RestoreSalFlag(te);		/* restore needsSalvage flag based on file's existence */
     SetNeedsClock(te);		/* compute needsClock field */
 
@@ -666,6 +657,16 @@ fs_timeout(struct bnode *bn)
 		 SDTIME);
 	}
     }
+
+    if ((abnode->b.flags & BNODE_ERRORSTOP) && !abnode->salRunning
+	&& !abnode->volRunning && !abnode->fileRunning && !abnode->scanRunning
+	&& !abnode->salsrvRunning) {
+	bnode_SetStat(bn, BSTAT_NORMAL);
+    }
+    else {
+	bnode_ResetErrorCount(bn);
+    }
+
     SetNeedsClock(abnode);
     return 0;
 }
@@ -770,17 +771,38 @@ fs_procexit(struct bnode *bn, struct bnode_proc *aproc)
 static void
 SetNeedsClock(struct fsbnode *ab)
 {
-    if (ab->b.goal == 1 && ab->fileRunning && ab->volRunning
+    afs_int32 timeout = POLLTIME;
+
+    if ((ab->fileSDW && !ab->fileKillSent) || (ab->volSDW && !ab->volKillSent)
+	|| (ab->scanSDW && !ab->scanKillSent) || (ab->salSDW && !ab->salKillSent)
+	|| (ab->salsrvSDW && !ab->salsrvKillSent)) {
+	/* SIGQUIT sent, will send SIGKILL if process does not exit */
+	ab->needsClock = 1;
+    } else if (ab->b.goal == 1 && ab->fileRunning && ab->volRunning
 	&& (!ab->scancmd || ab->scanRunning)
-	&& (!ab->salsrvcmd || ab->salsrvRunning))
-	ab->needsClock = 0;	/* running normally */
-    else if (ab->b.goal == 0 && !ab->fileRunning && !ab->volRunning
-	     && !ab->salRunning && !ab->scanRunning && !ab->salsrvRunning)
-	ab->needsClock = 0;	/* halted normally */
-    else
+	&& (!ab->salsrvcmd || ab->salsrvRunning)) {
+	if (ab->b.errorStopCount) {
+	    /* reset error count after running for a bit */
+	    ab->needsClock = 1;
+	} else {
+	    ab->needsClock = 0;	/* running normally */
+	}
+    } else if ((ab->b.goal == 0) && !ab->fileRunning && !ab->volRunning
+	       && !ab->salRunning && !ab->scanRunning && !ab->salsrvRunning) {
+	if (ab->b.flags & BNODE_ERRORSTOP && ab->b.errorStopDelay) {
+	    bozo_Log("%s will retry start in %d seconds\n", ab->b.name,
+		     ab->b.errorStopDelay);
+	    ab->needsClock = 1;	/* halted for errors, retry later */
+	    timeout = ab->b.errorStopDelay;
+	} else {
+	    ab->needsClock = 0;	/* halted normally */
+	}
+    } else
 	ab->needsClock = 1;	/* other */
-    if (ab->needsClock && !bnode_PendingTimeout(fsbnode2bnode(ab)))
-	bnode_SetTimeout(fsbnode2bnode(ab), POLLTIME);
+
+    if (ab->needsClock && (!bnode_PendingTimeout(fsbnode2bnode(ab))
+			   || ab->b.period != timeout))
+	bnode_SetTimeout(fsbnode2bnode(ab), timeout);
     if (!ab->needsClock)
 	bnode_SetTimeout(fsbnode2bnode(ab), 0);
 }
