@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -11,27 +11,19 @@
 
 #include <afsconfig.h>
 #include <afs/param.h>
-
-RCSID
-    ("$Header: /cvs/openafs/src/budb/ol_verify.c,v 1.13.14.3 2008/04/02 19:51:55 shadow Exp $");
-
-#include <stdio.h>
-#ifdef AFS_NT40_ENV
-#include <winsock2.h>
-#else
-#include <netinet/in.h>
-#include <netdb.h>
-#endif
-#include <string.h>
 #include <afs/stds.h>
-#include <sys/types.h>
+
+#include <roken.h>
+
 #include <lock.h>
 #include <ubik.h>
+#include <afs/cellconfig.h>
+#include <afs/audit.h>
+
 #include "database.h"
 #include "error_macros.h"
 #include "budb_errs.h"
-#include <afs/cellconfig.h>
-#include "afs/audit.h"
+#include "budb_internal.h"
 
 #undef min
 #undef max
@@ -39,8 +31,8 @@ RCSID
 /* notes
  * 1) volInfo structures refering to a volume of the same name are
  *	chained together, i.e. the volumes described differ in volid, partition
- *	etc. The structure at the head of this list (the sameNameChain) is 
- *	treated specially. When a delete volInfo request is processed, heads 
+ *	etc. The structure at the head of this list (the sameNameChain) is
+ *	treated specially. When a delete volInfo request is processed, heads
  *	are not deleted unless all other items on the sameNameChain are gone.
  *
  *	The result is that volInfo (head) structures may be present
@@ -53,7 +45,7 @@ RCSID
  *	is only for the off-line case.
  */
 
-/* flags associated with each structure. These are set and checked in 
+/* flags associated with each structure. These are set and checked in
  * the blockMap entries
  */
 
@@ -168,10 +160,10 @@ int hashBlockType[HT_MAX_FUNCTION + 1] = {
 struct mapCompatability {
     short trigger;		/* these bits trigger this element */
 } mapC[] = {
-MAP_FREE, MAP_HTBLOCK, MAP_DUMPHASH | MAP_IDHASH,
-	MAP_TAPEHASH | MAP_TAPEONDUMP, MAP_VOLINFOONNAME,
-	MAP_VOLINFONAMEHEAD | MAP_VOLHASH,
-	MAP_VOLFRAGONTAPE | MAP_VOLFRAGONVOL, MAP_TEXTBLOCK};
+	{MAP_FREE}, {MAP_HTBLOCK}, {MAP_DUMPHASH | MAP_IDHASH},
+	{MAP_TAPEHASH | MAP_TAPEONDUMP}, {MAP_VOLINFOONNAME},
+	{MAP_VOLINFONAMEHEAD | MAP_VOLHASH},
+	{MAP_VOLFRAGONTAPE | MAP_VOLFRAGONVOL}, {MAP_TEXTBLOCK}};
 
 /* no. of entries in the mapC array */
 int NMAPCs = (sizeof(mapC) / sizeof(mapC[0]));
@@ -187,6 +179,11 @@ char *textName[TB_NUM] = {
 extern int sizeFunctions[];
 extern int nHTBuckets;
 
+afs_int32 DbVerify(struct rx_call *call, afs_int32 *status,
+		   afs_int32 *orphans, afs_int32 *host);
+afs_int32 verifyTextChain(struct ubik_trans *ut, struct textBlock *tbPtr);
+
+
 #define DBBAD BUDB_DATABASEINCONSISTENT
 
 /* ------------------------------------
@@ -202,7 +199,7 @@ extern int nHTBuckets;
  */
 
 afs_int32
-BumpErrors()
+BumpErrors(void)
 {
     if (++miscData.errors >= miscData.maxErrors)
 	return (1);
@@ -221,11 +218,8 @@ BumpErrors()
  */
 
 afs_int32
-checkDiskAddress(address, type, blockIndexPtr, entryIndexPtr)
-     unsigned long address;
-     int type;
-     int *blockIndexPtr;
-     int *entryIndexPtr;
+checkDiskAddress(unsigned long address, int type, int *blockIndexPtr,
+		 int *entryIndexPtr)
 {
     int index, offset;
 
@@ -269,10 +263,7 @@ checkDiskAddress(address, type, blockIndexPtr, entryIndexPtr)
  */
 
 afs_int32
-ConvertDiskAddress(address, blockIndexPtr, entryIndexPtr)
-     afs_uint32 address;
-     int *blockIndexPtr;
-     int *entryIndexPtr;
+ConvertDiskAddress(afs_uint32 address, int *blockIndexPtr, int *entryIndexPtr)
 {
     int index, type;
     afs_int32 code;
@@ -285,8 +276,7 @@ ConvertDiskAddress(address, blockIndexPtr, entryIndexPtr)
 }
 
 char *
-TypeName(index)
-     int index;
+TypeName(int index)
 {
     static char error[36];
 
@@ -318,16 +308,15 @@ getDumpID(struct ubik_trans *ut,
  */
 
 /* verifyDumpEntry
- *      Follow the tapes entries hanging off of a dump and verify they belong 
+ *      Follow the tapes entries hanging off of a dump and verify they belong
  *      to the dump.
  */
 afs_int32
-verifyDumpEntry(ut, dumpAddr, ai, ao, dumpPtr)
-     struct ubik_trans *ut;
-     afs_int32 dumpAddr;
-     int ai, ao;
-     struct dump *dumpPtr;
+verifyDumpEntry(struct ubik_trans *ut, afs_int32 dumpAddr, int ai, int ao,
+		void *param)
 {
+    struct dump *dumpPtr = (struct dump *)param;
+
     struct tape tape;
     afs_int32 tapeAddr, tapeCount = 0, volCount = 0, appDumpCount = 0;
     afs_int32 appDumpAddr, appDumpIndex, appDumpOffset;
@@ -452,16 +441,14 @@ verifyDumpEntry(ut, dumpAddr, ai, ao, dumpPtr)
 
 /*
  * verifyTapeEntry
- *      Follw the volume fragments hanging off of a tape entry and verify 
+ *      Follw the volume fragments hanging off of a tape entry and verify
  *      they belong to the tape.
  */
 afs_int32
-verifyTapeEntry(ut, tapeAddr, ai, ao, tapePtr)
-     struct ubik_trans *ut;
-     afs_int32 tapeAddr;
-     int ai, ao;
-     struct tape *tapePtr;
+verifyTapeEntry(struct ubik_trans *ut, afs_int32 tapeAddr, int ai, int ao,
+     		void *param)
 {
+    struct tape *tapePtr = (struct tape *) param;
     int volCount = 0, ccheck = 1;
     afs_int32 volFragAddr;
     int blockIndex, entryIndex;
@@ -535,12 +522,10 @@ verifyTapeEntry(ut, tapeAddr, ai, ao, tapePtr)
  *      So no check is done agaist it.
  */
 afs_int32
-verifyVolFragEntry(ut, va, ai, ao, v)
-     struct ubik_trans *ut;
-     afs_int32 va;
-     int ai, ao;
-     struct volFragment *v;
+verifyVolFragEntry(struct ubik_trans *ut, afs_int32 va, int ai, int ao,
+		   void *param)
 {
+    /* struct volFragment *v = (struct volFragment *)param; */
     misc->nVolFrag++;
     return 0;
 }
@@ -552,12 +537,11 @@ verifyVolFragEntry(ut, va, ai, ao, v)
  *      also verify all entries are also on the chain.
  */
 afs_int32
-verifyVolInfoEntry(ut, volInfoAddr, ai, ao, volInfo)
-     struct ubik_trans *ut;
-     afs_int32 volInfoAddr;
-     int ai, ao;
-     struct volInfo *volInfo;
+verifyVolInfoEntry(struct ubik_trans *ut, afs_int32 volInfoAddr, int ai,
+		   int ao, void *param)
 {
+    struct volInfo *volInfo = (struct volInfo *) param;
+
     int volCount = 0, ccheck = 1;
     afs_int32 volFragAddr;
     int blockIndex, entryIndex;
@@ -612,7 +596,7 @@ verifyVolInfoEntry(ut, volInfoAddr, ai, ao, volInfo)
     if (volCount > misc->maxVolsPerVolInfo)
 	misc->maxVolsPerVolInfo = volCount;
 
-    /* Check that all volInfo structures with same name point to the same 
+    /* Check that all volInfo structures with same name point to the same
      * head. If sameNameHead == 0, this is the head structure so we check,
      * otherwise ignore
      */
@@ -679,12 +663,11 @@ verifyVolInfoEntry(ut, volInfoAddr, ai, ao, volInfo)
  */
 
 /* verifyBlocks
- *     Read each block header of every 2K block and remember it in our global 
+ *     Read each block header of every 2K block and remember it in our global
  *     blockMap array. Also check that the type of block is good.
  */
 afs_int32
-verifyBlocks(ut)
-     struct ubik_trans *ut;
+verifyBlocks(struct ubik_trans *ut)
 {
     struct block block;
     int blocktype;
@@ -716,10 +699,9 @@ verifyBlocks(ut)
 	bmsize =
 	    sizeof(*ablockMap) + (blockEntries[blocktype] -
 				  1) * sizeof(ablockMap->entries[0]);
-	ablockMap = (struct blockMap *)malloc(bmsize);
+	ablockMap = calloc(1, bmsize);
 	if (!ablockMap)
 	    ERROR(BUDB_NOMEM);
-	memset(ablockMap, 0, bmsize);
 
 	ablockMap->nEntries = blockEntries[blocktype];
 
@@ -736,18 +718,17 @@ int minvols, maxvols, ttlvols;
 
 /* verifyHashTableBlock
  *      Take a 2K hash table block and traverse its entries. Make sure each entry
- *      is of the correct type for the hash table, is hashed into the correct 
+ *      is of the correct type for the hash table, is hashed into the correct
  *      entry and is not threaded on multiple lists.
  */
 afs_int32
-verifyHashTableBlock(ut, mhtPtr, htBlockPtr, old, length, index, mapBit)
-     struct ubik_trans *ut;
-     struct memoryHashTable *mhtPtr;
-     struct htBlock *htBlockPtr;
-     int old;
-     afs_int32 length;		/* size of whole hash table */
-     int index;			/* base index of this block */
-     int mapBit;
+verifyHashTableBlock(struct ubik_trans *ut,
+		     struct memoryHashTable *mhtPtr,
+		     struct htBlock *htBlockPtr,
+		     int old,
+		     afs_int32 length,	/* size of whole hash table */
+		     int index,		/* base index of this block */
+     		     int mapBit)
 {
     int type;			/* hash table type */
     int entrySize;		/* hashed entry size */
@@ -837,16 +818,14 @@ verifyHashTableBlock(ut, mhtPtr, htBlockPtr, old, length, index, mapBit)
 }
 
 /* verifyHashTable
- *      Read each 2K block a hashtable has (both its old hastable and 
+ *      Read each 2K block a hashtable has (both its old hastable and
  *      new hashtable) and verify the block has not been read before.
  *      Will also make call to verify entries within each 2K block of
  *      the hash table.
  */
 afs_int32
-verifyHashTable(ut, mhtPtr, mapBit)
-     struct ubik_trans *ut;
-     struct memoryHashTable *mhtPtr;
-     int mapBit;
+verifyHashTable(struct ubik_trans *ut, struct memoryHashTable *mhtPtr,
+		int mapBit)
 {
     struct hashTable *htPtr = mhtPtr->ht;
 
@@ -944,18 +923,19 @@ verifyHashTable(ut, mhtPtr, mapBit)
  *	the integrity of each allocated structure.
  */
 afs_int32
-verifyEntryChains(ut)
-     struct ubik_trans *ut;
+verifyEntryChains(struct ubik_trans *ut)
 {
     afs_int32 code;
     afs_int32 offset;
+    afs_int32 start;
     int blockIndex, entryIndex;
     char entry[sizeof(struct block)];
     int entrySize;
     int type;
     int nFree;
 
-    static afs_int32(*checkEntry[NBLOCKTYPES]) ()
+    static afs_int32(*checkEntry[NBLOCKTYPES]) (struct ubik_trans *,
+		    				afs_int32, int, int, void *)
 	= {
 	/* FIXME: this list does not match typeName[] and may be incorrect */
 	0,			/* free block */
@@ -979,7 +959,8 @@ verifyEntryChains(ut)
 		return BUDB_IO;
 
 	    /* check if entry is free by looking at the first "afs_int32" of the structure */
-	    if (*((afs_int32 *) & entry[0]) == 0) {	/* zero is free */
+	    memcpy(&start, entry, sizeof(start));
+	    if (start == 0) {	/* zero is free */
 		/* Is it on any hash chain? */
 		if (blockMap[blockIndex]->entries[entryIndex] & MAP_HASHES) {
 		    Log("Entry: blockindex %d, entryindex %d - marked free but hashed 0x%x\n", blockIndex, entryIndex, blockMap[blockIndex]->entries[entryIndex]);
@@ -1013,7 +994,7 @@ verifyEntryChains(ut)
 
 
 afs_int32
-verifyFreeLists()
+verifyFreeLists(void)
 {
     int i;
     afs_int32 addr;
@@ -1035,7 +1016,6 @@ verifyFreeLists()
 		Log("     Skipping remainder of free chain\n");
 		if (BumpErrors())
 		    return (DBBAD);
-		code = 0;
 		break;
 	    }
 
@@ -1078,7 +1058,7 @@ verifyFreeLists()
  *	checking the bits for compatibility.
  */
 afs_int32
-verifyMapBits()
+verifyMapBits(void)
 {
     int blockIndex, entryIndex, i, entrySize, type, bits;
     afs_int32 offset;
@@ -1178,12 +1158,10 @@ verifyMapBits()
 }
 
 afs_int32
-verifyText(ut)
-     struct ubik_trans *ut;
+verifyText(struct ubik_trans *ut)
 {
     int i;
     afs_int32 code;
-    extern afs_int32 verifyTextChain();
 
     /* check each of the text types in use */
     for (i = 0; i < TB_NUM; i++) {
@@ -1199,9 +1177,7 @@ verifyText(ut)
  *	check the integrity of a text chain. Also checks the new chain.
  */
 afs_int32
-verifyTextChain(ut, tbPtr)
-     struct ubik_trans *ut;
-     struct textBlock *tbPtr;
+verifyTextChain(struct ubik_trans *ut, struct textBlock *tbPtr)
 {
     dbadr blockAddr;
     int blockIndex, entryIndex;
@@ -1262,16 +1238,14 @@ verifyTextChain(ut, tbPtr)
  */
 
 afs_int32
-verifyDatabase(ut, recreateFile)
-     struct ubik_trans *ut;
-     FILE *recreateFile;	/* not used */
+verifyDatabase(struct ubik_trans *ut,
+     	       FILE *recreateFile)	/* not used */
 {
     afs_int32 eof;
     int bmsize;
-    afs_int32 code = 0, tcode;
+    afs_int32 code = 0, tcode = 0;
 
     extern int nBlocks;		/* no. blocks in database */
-    extern struct ubik_dbase *BU_dbase;
 
     /* clear verification statistics */
     misc = &miscData;
@@ -1306,10 +1280,9 @@ verifyDatabase(ut, recreateFile)
 
     /* construct block map - first level is the array of pointers */
     bmsize = nBlocks * sizeof(struct blockMap *);
-    blockMap = (struct blockMap **)malloc(bmsize);
+    blockMap = calloc(1, bmsize);
     if (!blockMap)
 	ERROR(BUDB_NOMEM);
-    memset(blockMap, 0, bmsize);
 
     /* verify blocks and construct the block map */
     Log("Read header of every block\n");
@@ -1426,13 +1399,9 @@ verifyDatabase(ut, recreateFile)
  *	orphans - no. of orphan blocks
  *	host - address of host that did verification
  */
-afs_int32 DbVerify();
 afs_int32
-SBUDB_DbVerify(call, status, orphans, host)
-     struct rx_call *call;
-     afs_int32 *status;
-     afs_int32 *orphans;
-     afs_int32 *host;
+SBUDB_DbVerify(struct rx_call *call, afs_int32 *status, afs_int32 *orphans,
+	       afs_int32 *host)
 {
     afs_int32 code;
 
@@ -1442,11 +1411,8 @@ SBUDB_DbVerify(call, status, orphans, host)
 }
 
 afs_int32
-DbVerify(call, status, orphans, host)
-     struct rx_call *call;
-     afs_int32 *status;
-     afs_int32 *orphans;
-     afs_int32 *host;
+DbVerify(struct rx_call *call, afs_int32 *status, afs_int32 *orphans,
+	 afs_int32 *host)
 {
     struct ubik_trans *ut = 0;
     afs_int32 code = 0, tcode;
@@ -1495,9 +1461,8 @@ DbVerify(call, status, orphans, host)
 /* check_header
  *	do a simple sanity check on the database header
  */
-
-check_header(callerst)
-     char *callerst;
+void
+check_header(char *callerst)
 {
     static int iteration_count = 0;
     afs_int32 eof;
