@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -10,21 +10,8 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/butc/dbentries.c,v 1.8.14.4 2007/12/11 16:27:06 shadow Exp $");
+#include <roken.h>
 
-#include <sys/types.h>
-#ifdef AFS_NT40_ENV
-#include <winsock2.h>
-#else
-#include <sys/file.h>
-#include <netinet/in.h>
-#include <sys/socket.h>
-#include <sys/time.h>
-#include <netdb.h>
-#endif
-#include <stdlib.h>
-#include <string.h>
 #include <rx/xdr.h>
 #include <rx/rx.h>
 #include <lwp.h>
@@ -33,12 +20,13 @@ RCSID
 #include <afs/bubasics.h>
 #include <afs/budb_client.h>
 #include <afs/vldbint.h>
-
 #include <afs/vlserver.h>
 #include <afs/volser.h>
 #include <afs/volint.h>
 #include <afs/cellconfig.h>
+#include <afs/bucoord_prototypes.h>
 
+#include "butc_internal.h"
 #include "error_macros.h"
 
 dlqlinkT savedEntries;
@@ -47,15 +35,15 @@ dlqlinkT entries_to_flush;
 int dbWatcherinprogress;
 
 afs_int32
-threadEntryDir(char *anEntry, afs_int32 size, afs_int32 type)
+threadEntryDir(void *anEntry, afs_int32 size, afs_int32 type)
 {
     dlqlinkP entryPtr;
-    char *entry = NULL;
+    void *entry = NULL;
     int tried;
 
     for (tried = 0; tried < 5; tried++) {
-	entryPtr = (dlqlinkP) malloc(sizeof(dlqlinkT));
-	entry = (char *)malloc(size);
+	entryPtr = malloc(sizeof(dlqlinkT));
+	entry = malloc(size);
 	if (entryPtr && entry)
 	    break;
 
@@ -85,20 +73,20 @@ threadEntryDir(char *anEntry, afs_int32 size, afs_int32 type)
 /*
  * threadEntry.
  *     Creates an entry and puts it onto the savedEntries list.
- *     Will retry up to 5 times if not enough memory. Hopfully, the 
+ *     Will retry up to 5 times if not enough memory. Hopfully, the
  *     Watcher thread will free up some memory for it to continue.
  */
 
 afs_int32
-threadEntry(char *anEntry, afs_int32 size, afs_int32 type)
+threadEntry(void *anEntry, afs_int32 size, afs_int32 type)
 {
     dlqlinkP entryPtr;
-    char *entry = NULL;
+    void *entry = NULL;
     int tried;
 
     for (tried = 0; tried < 5; tried++) {
 	entryPtr = (dlqlinkP) malloc(sizeof(dlqlinkT));
-	entry = (char *)malloc(size);
+	entry = malloc(size);
 	if (entryPtr && entry)
 	    break;
 
@@ -158,7 +146,9 @@ finishDump(struct budb_dumpEntry *aDumpEntryPtr)
  *     Creates a tape entry and puts it onto the savedEntries list.
  */
 afs_int32
-useTape(struct budb_tapeEntry *aTapeEntryPtr, afs_int32 dumpID, char *tapename, afs_int32 tapeSeq, afs_int32 useCount, Date written, Date expiration, afs_int32 tapepos)
+useTape(struct budb_tapeEntry *aTapeEntryPtr, afs_int32 dumpID,
+	char *tapename, afs_int32 tapeSeq, afs_int32 useCount,
+	Date written, Date expiration, afs_int32 tapepos)
 {
     afs_int32 code = 0;
 
@@ -201,7 +191,10 @@ finishTape(struct budb_tapeEntry *aTapeEntryPtr, afs_int32 useKBytes)
  *     Creates a volume entry and puts it onto the savedEntries list.
  */
 afs_int32
-addVolume(struct budb_volumeEntry *aVolEntryPtr, afs_int32 dumpID, char *tapename, char *volname, afs_int32 volid, Date cloneDate, afs_int32 startPos, afs_int32 volBytes, int fragment, afs_int32 flags)
+addVolume(struct budb_volumeEntry *aVolEntryPtr, afs_int32 dumpID,
+	  char *tapename, char *volname, afs_int32 volid,
+	  Date cloneDate, afs_int32 startPos, afs_int32 volBytes,
+	  int fragment, afs_int32 flags)
 {
     afs_int32 code = 0;
     int allo = 0;
@@ -235,9 +228,16 @@ addVolume(struct budb_volumeEntry *aVolEntryPtr, afs_int32 dumpID, char *tapenam
     return (code);
 }
 
+static_inline int
+freeEntry(void *e)
+{
+    free(e);
+    return 0;
+}
+
 /*
  * flushSavedEntries
- *     Runs through the list of savedEntries and adds the volumes and 
+ *     Runs through the list of savedEntries and adds the volumes and
  *     tapes to the database.
  *     A status of DUMP_NORETRYEOT means the tape(s) contains no useful data,
  *     and tapes and volumes should not be added to the DB.
@@ -268,11 +268,11 @@ flushSavedEntries(afs_int32 status)
 	    free(entryPtr);
     }
 
-    /* 
-     * Add dump, tape, and volume entries to the list for the dbWatcher to 
+    /*
+     * Add dump, tape, and volume entries to the list for the dbWatcher to
      * flush. Volume entries are not added if the volume failed to dump.
      */
-    while (entryPtr = dlqUnlinkf(&savedEntries)) {
+    while ((entryPtr = dlqUnlinkf(&savedEntries))) {
 	if ((entryPtr->dlq_type == DLQ_VOLENTRY) && (status != DUMP_SUCCESS)) {
 	    volPtr = (struct budb_volumeEntry *)entryPtr->dlq_structPtr;
 	    if (volPtr)
@@ -286,12 +286,12 @@ flushSavedEntries(afs_int32 status)
 
   error_exit:
     /* Free anything that remains on dlq */
-    dlqTraverseQueue(&savedEntries, free, free);
+    dlqTraverseQueue(&savedEntries, freeEntry, freeEntry);
     return (code);
 }
 
 void
-waitDbWatcher()
+waitDbWatcher(void)
 {
     int message = 0;
 
@@ -316,8 +316,8 @@ waitDbWatcher()
 #define MAXVOLUMESTOADD 100
 int addvolumes = 1;
 
-void
-dbWatcher()
+void *
+dbWatcher(void *unused)
 {
     dlqlinkP entryPtr;
     struct budb_dumpEntry *dumpPtr;
@@ -327,6 +327,7 @@ dbWatcher()
     afs_int32 code = 0;
     int i, c, addedDump;
 
+    afs_pthread_setname_self("dbWatcher");
     dlqInit(&entries_to_flush);
     dlqInit(&savedEntries);
 
@@ -335,7 +336,7 @@ dbWatcher()
     while (1) {
 	/*while */
 	/* Add tape and volume enties to the backup database */
-	while (entryPtr = dlqUnlinkf(&entries_to_flush)) {
+	while ((entryPtr = dlqUnlinkf(&entries_to_flush))) {
 	    dbWatcherinprogress = 1;
 
 	    if (!entryPtr->dlq_structPtr) {
@@ -393,7 +394,7 @@ dbWatcher()
 		    tapePtr =
 			(struct budb_tapeEntry *)entryPtr->dlq_structPtr;
 		    if (addedDump) {
-			code = bcdb_FinishTape(tapePtr, &new);
+			code = bcdb_FinishTape(tapePtr);
 			if (code) {
 			    ErrorLog(0, 0, code, 0,
 				     "Warning: Can't finish tape %s of DumpID %u in backup database\n",
@@ -485,4 +486,5 @@ dbWatcher()
 	IOMGR_Sleep(2);
 #endif
     }
+    return NULL;
 }
