@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -12,24 +12,13 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+#include <afs/procmgmt.h>
+#include <roken.h>
 
-#include <sys/types.h>
-#include <errno.h>
-#ifdef AFS_NT40_ENV
-#include <winsock2.h>
-#else
-#include <sys/file.h>
-#include <netinet/in.h>
-#endif
-#include <rx/xdr.h>
+#include <afs/opr.h>
 #include <rx/rx.h>
 #include <afs/afsint.h>
-#include <stdio.h>
-#include <string.h>
-#include <afs/procmgmt.h>
-#include <afs/assert.h>
 #include <afs/prs_fs.h>
-#include <sys/stat.h>
 #include <afs/nfs.h>
 #include <lwp.h>
 #include <lock.h>
@@ -39,6 +28,7 @@
 #include <afs/tcdata.h>
 #include <afs/budb_client.h>
 #include <afs/bucoord_prototypes.h>
+
 #include "error_macros.h"
 #include "butc_xbsa.h"
 #include "butc_prototypes.h"
@@ -156,7 +146,7 @@ STC_LabelTape(struct rx_call *acid, struct tc_tapeLabel *label, afs_uint32 *task
     if (callPermitted(acid) == 0)
 	return (TC_NOTPERMITTED);
 
-    ptr = (struct labelTapeIf *)malloc(sizeof(*ptr));
+    ptr = malloc(sizeof(*ptr));
     if (!ptr)
 	ERROR_EXIT(TC_NOMEMORY);
     memcpy(&ptr->label, label, sizeof(ptr->label));
@@ -236,18 +226,13 @@ STC_PerformDump(struct rx_call *rxCallId, struct tc_dumpInterface *tcdiPtr, tc_d
     /*set up the parameters in the node, to be used by LWP */
     strcpy(newNode->dumpSetName, tcdiPtr->dumpName);
 
-    newNode->dumpName = (char *)malloc(strlen(tcdiPtr->dumpPath) + 1);
-    strcpy(newNode->dumpName, tcdiPtr->dumpPath);
-
-    newNode->volumeSetName =
-	(char *)malloc(strlen(tcdiPtr->volumeSetName) + 1);
-    strcpy(newNode->volumeSetName, tcdiPtr->volumeSetName);
+    newNode->dumpName = strdup(tcdiPtr->dumpPath);
+    newNode->volumeSetName = strdup(tcdiPtr->volumeSetName);
 
     CopyTapeSetDesc(&(newNode->tapeSetDesc), &tcdiPtr->tapeSet);
 
-    newNode->dumps = (struct tc_dumpDesc *)
-	malloc(sizeof(struct tc_dumpDesc) *
-	       tc_dumpArrayPtr->tc_dumpArray_len);
+    newNode->dumps = malloc(sizeof(struct tc_dumpDesc) *
+			    tc_dumpArrayPtr->tc_dumpArray_len);
     newNode->arraySize = tc_dumpArrayPtr->tc_dumpArray_len;
     CopyDumpDesc(newNode->dumps, tc_dumpArrayPtr);
 
@@ -328,9 +313,8 @@ STC_PerformRestore(struct rx_call *acid, char *dumpSetName, tc_restoreArray *are
     /* this creates a node in list, alots an id for it and prepares it for locking */
     CreateNode(&newNode);
 
-    newNode->restores = (struct tc_restoreDesc *)
-	malloc(sizeof(struct tc_restoreDesc) *
-	       arestores->tc_restoreArray_len);
+    newNode->restores = malloc(sizeof(struct tc_restoreDesc) *
+		               arestores->tc_restoreArray_len);
     newNode->arraySize = arestores->tc_restoreArray_len;
     CopyRestoreDesc(newNode->restores, arestores);
     *taskID = newNode->taskID;
@@ -446,11 +430,11 @@ STC_RestoreDb(struct rx_call *rxCall, afs_uint32 *taskId)
 	ERROR_EXIT(code);
 
     AFS_SIGSET_CLEAR();
-    code = pthread_create(&pid, &tattr, restoreDbFromTape, (void *)*taskId);
+    code = pthread_create(&pid, &tattr, restoreDbFromTape, (void *)(intptr_t)*taskId);
     AFS_SIGSET_RESTORE();
 #else
     code =
-	LWP_CreateProcess(restoreDbFromTape, 32768, 1, (void *)*taskId,
+	LWP_CreateProcess(restoreDbFromTape, 32768, 1, (void *)(intptr_t)*taskId,
 			  "Db restore", &pid);
 #endif
 
@@ -491,7 +475,7 @@ STC_SaveDb(struct rx_call *rxCall, Date archiveTime, afs_uint32 *taskId)
 
     *taskId = allocTaskId();
 
-    ptr = (struct saveDbIf *)malloc(sizeof(struct saveDbIf));
+    ptr = malloc(sizeof(struct saveDbIf));
     if (!ptr)
 	ERROR_EXIT(TC_NOMEMORY);
     ptr->archiveTime = archiveTime;
@@ -570,7 +554,7 @@ STC_ScanDumps(struct rx_call *acid, afs_int32 addDbFlag, afs_uint32 *taskId)
 
     *taskId = allocTaskId();
 
-    ptr = (struct scanTapeIf *)malloc(sizeof(*ptr));
+    ptr = malloc(sizeof(*ptr));
     if (!ptr)
 	ERROR_EXIT(TC_NOMEMORY);
     ptr->addDbFlag = addDbFlag;
@@ -658,7 +642,7 @@ STC_DeleteDump(struct rx_call *acid, afs_uint32 dumpID, afs_uint32 *taskId)
     if (callPermitted(acid) == 0)
 	return (TC_NOTPERMITTED);
 
-    ptr = (struct deleteDumpIf *)malloc(sizeof(*ptr));
+    ptr = malloc(sizeof(*ptr));
     if (!ptr)
 	ERROR_EXIT(TC_NOMEMORY);
 
