@@ -1,22 +1,21 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
  */
+#include <afsconfig.h>
+#include <afs/param.h>
+
+#include <roken.h>
 
-#include <sys/types.h>
-#include <sys/file.h>
-#include <sys/stat.h>
 #ifdef AFS_AIX_ENV
 #include <sys/statfs.h>
 #endif
-#include <netdb.h>
-#include <sys/errno.h>
+
 #include <lock.h>
-#include <netinet/in.h>
 #include <rx/xdr.h>
 #include <rx/rx.h>
 #include <rx/rx_globals.h>
@@ -29,9 +28,6 @@
 #include <afs/cmd.h>
 #include <rx/rxkad.h>
 #include <afs/tcdata.h>
-#ifdef	AFS_AIX32_ENV
-#include <signal.h>
-#endif
 
 #define SERVERNAME "server1"
 
@@ -46,9 +42,10 @@ int rxInitDone = 0;
 
 struct rx_connection *
 UV_Bind(aserver, port)
-     afs_int32 aserver, port;
+     afs_uint32 aserver;
+     afs_int32 port;
 {
-    register struct rx_connection *tc;
+    struct rx_connection *tc;
     struct rx_securityClass *uvclass;
 
     uvclass = rxnull_NewClientSecurityObject();
@@ -58,14 +55,14 @@ UV_Bind(aserver, port)
 
 
 /* return host address in network byte order */
-afs_int32
+afs_uint32
 GetServer(aname)
      char *aname;
 {
-    register struct hostent *th;
-    afs_int32 addr;
+    struct hostent *th;
+    afs_uint32 addr;
     char b1, b2, b3, b4;
-    register afs_int32 code;
+    afs_int32 code;
 
     code = sscanf(aname, "%d.%d.%d.%d", &b1, &b2, &b3, &b4);
     if (code == 4) {
@@ -80,12 +77,11 @@ GetServer(aname)
 }
 
 
-static
-PerformDump(as)
-     register struct cmd_syndesc *as;
+static int
+PerformDump(struct cmd_syndesc *as, void *arock)
 {
     struct rx_connection *aconn;
-    afs_int32 server;
+    afs_uint32 server;
     FILE *fopen(), *fp;
     struct tc_dumpDesc *ptr;
     int i;
@@ -107,8 +103,7 @@ PerformDump(as)
     strcpy(ttapeSet.format, "tapeName%u");
     strcpy(ttapeSet.tapeServer, "diskTapes");
     tdumps.tc_dumpArray_val =
-	(struct tc_dumpDesc
-	 *)(malloc(tdumps.tc_dumpArray_len * sizeof(struct tc_dumpDesc)));
+	malloc(tdumps.tc_dumpArray_len * sizeof(struct tc_dumpDesc));
     ptr = tdumps.tc_dumpArray_val;
     for (i = 0; i < tdumps.tc_dumpArray_len; i++) {
 	fscanf(fp, "%s\n", ptr->name);
@@ -127,14 +122,15 @@ PerformDump(as)
 	exit(1);
     }
     printf("dumpid returned %u\n", tdumpID);
+
+    return 0;
 }
 
-static
-PerformRestore(as)
-     register struct cmd_syndesc *as;
+static int
+PerformRestore(struct cmd_syndesc *as, void *arock)
 {
     struct rx_connection *aconn;
-    afs_int32 server;
+    afs_uint32 server;
     int i;
     FILE *fopen(), *fp;
     struct tc_restoreDesc *ptr;
@@ -149,9 +145,9 @@ PerformRestore(as)
     strcpy(tdumpSetName, "Test");
     fp = fopen("restoreScr", "r");
     fscanf(fp, "%u\n", &trestores.tc_restoreArray_len);
-    trestores.tc_restoreArray_val =
-	(struct tc_restoreDesc *)malloc(trestores.tc_restoreArray_len *
-					sizeof(struct tc_restoreDesc));
+    trestores.tc_restoreArray_val
+	= malloc(trestores.tc_restoreArray_len *
+		 sizeof(struct tc_restoreDesc));
     ptr = trestores.tc_restoreArray_val;
     for (i = 0; i < trestores.tc_restoreArray_len; i++) {
 	fscanf(fp, "%s\n", ptr->oldName);
@@ -169,14 +165,14 @@ PerformRestore(as)
 	exit(1);
     }
     printf("dumpid returned %u\n", tdumpID);
+    return 0;
 }
 
-static
-CheckDump(as)
-     register struct cmd_syndesc *as;
+static int
+CheckDump(struct cmd_syndesc *as, void *arock)
 {
     struct rx_connection *aconn;
-    afs_int32 server;
+    afs_uint32 server;
     server = GetServer(SERVERNAME);
     if (!server) {
 	printf("cant get server id \n");
@@ -189,14 +185,14 @@ CheckDump(as)
 	printf("call to TC_CheckDump failed %u\n", code);
 	exit(1);
     }
+    return 0;
 }
 
-static
-AbortDump(as)
-     register struct cmd_syndesc *as;
+static int
+AbortDump(struct cmd_syndesc *as, void *arock)
 {
     struct rx_connection *aconn;
-    afs_int32 server;
+    afs_uint32 server;
     server = GetServer(SERVERNAME);
     if (!server) {
 	printf("cant get server id \n");
@@ -209,14 +205,14 @@ AbortDump(as)
 	printf("call to TC_AbortDump failed %u\n", code);
 	exit(1);
     }
+    return 0;
 }
 
-static
-WaitForDump(as)
-     register struct cmd_syndesc *as;
+static int
+WaitForDump(struct cmd_syndesc *as, void *arock)
 {
     struct rx_connection *aconn;
-    afs_int32 server;
+    afs_uint32 server;
     server = GetServer(SERVERNAME);
     if (!server) {
 	printf("cant get server id \n");
@@ -229,14 +225,14 @@ WaitForDump(as)
 	printf("call to TC_WaitForDump failed %u\n", code);
 	exit(1);
     }
+    return 0;
 }
 
-static
-EndDump(as)
-     register struct cmd_syndesc *as;
+static int
+EndDump(struct cmd_syndesc *as, void *arock)
 {
     struct rx_connection *aconn;
-    afs_int32 server;
+    afs_uint32 server;
     server = GetServer(SERVERNAME);
     if (!server) {
 	printf("cant get server id \n");
@@ -249,12 +245,11 @@ EndDump(as)
 	printf("call to TC_EndDump failed %u\n", code);
 	exit(1);
     }
+    return 0;
 }
 
-static
-MyBeforeProc(as, arock)
-     struct cmd_syndesc *as;
-     char *arock;
+static int
+MyBeforeProc(struct cmd_syndesc *as, void *arock)
 {
     afs_int32 code;
 
@@ -274,14 +269,14 @@ main(argc, argv)
      int argc;
      char **argv;
 {
-    register afs_int32 code;
+    afs_int32 code;
 
-    register struct cmd_syndesc *ts;
+    struct cmd_syndesc *ts;
 
 #ifdef	AFS_AIX32_ENV
     /*
-     * The following signal action for AIX is necessary so that in case of a 
-     * crash (i.e. core is generated) we can include the user's data section 
+     * The following signal action for AIX is necessary so that in case of a
+     * crash (i.e. core is generated) we can include the user's data section
      * in the core dump. Unfortunately, by default, only a partial core is
      * generated which, in many cases, isn't too useful.
      */
@@ -295,20 +290,20 @@ main(argc, argv)
 #endif
     cmd_SetBeforeProc(MyBeforeProc, NULL);
 
-    ts = cmd_CreateSyntax("dump", PerformDump, 0, "perform a dump");
+    ts = cmd_CreateSyntax("dump", PerformDump, NULL, "perform a dump");
 
-    ts = cmd_CreateSyntax("restore", PerformRestore, 0, "perform a restore");
+    ts = cmd_CreateSyntax("restore", PerformRestore, NULL, "perform a restore");
 
-    ts = cmd_CreateSyntax("check", CheckDump, 0, "check a dump");
+    ts = cmd_CreateSyntax("check", CheckDump, NULL, "check a dump");
     cmd_AddParm(ts, "-id", CMD_SINGLE, 0, "dump id");
 
-    ts = cmd_CreateSyntax("abort", AbortDump, 0, "abort a dump");
+    ts = cmd_CreateSyntax("abort", AbortDump, NULL, "abort a dump");
     cmd_AddParm(ts, "-id", CMD_SINGLE, 0, "dump id");
 
-    ts = cmd_CreateSyntax("wait", WaitForDump, 0, "wait for a dump");
+    ts = cmd_CreateSyntax("wait", WaitForDump, NULL, "wait for a dump");
     cmd_AddParm(ts, "-id", CMD_SINGLE, 0, "dump id");
 
-    ts = cmd_CreateSyntax("end", EndDump, 0, "end a dump");
+    ts = cmd_CreateSyntax("end", EndDump, NULL, "end a dump");
     cmd_AddParm(ts, "-id", CMD_SINGLE, 0, "dump id");
 
     code = cmd_Dispatch(argc, argv);
