@@ -65,6 +65,7 @@
 #define AFS_USR_LINUX26_ENV	1
 
 #define AFS_ENV			1
+#define AFS_64BIT_CLIENT	1
 #undef  AFS_NONFSTRANS
 #define AFS_NONFSTRANS 		1
 #define AFS_MOUNT_AFS 		"afs"	/* The name of the filesystem type. */
@@ -75,7 +76,6 @@
 #define AFS_GCPAGS		0	/* if nonzero, garbage collect PAGs */
 #define AFS_HAVE_FFS		1	/* Use system's ffs. */
 #define AFS_HAVE_STATVFS	0	/* System doesn't support statvfs */
-#define AFS_VM_RDWR_ENV		1	/* read/write implemented via VM */
 
 #define	afsio_iov	uio_iov
 #define	afsio_iovcnt	uio_iovcnt
@@ -100,6 +100,13 @@
 #include <features.h>
 #endif
 
+#if defined(HAVE_LINUX_ERRQUEUE_H) && defined(HAVE_SETSOCKOPT_IP_RECVERR)
+# define AFS_RXERRQ_ENV
+#endif
+#ifdef AFS_RXERRQ_ENV
+# define AFS_ADAPT_PMTU
+#endif
+
 #ifdef __GLIBC__
 #if (__GLIBC__ > 2) || (__GLIBC__ == 2 && __GLIBC_MINOR__ > 3)
 #define USE_UCONTEXT
