@@ -28,6 +28,7 @@
 #include <rx/xdr.h>
 #include <rx/rx.h>
 #include <rx/rxkad.h>
+#include <rx/rxkad_convert.h>
 #include <lock.h>
 #define UBIK_LEGACY_CALLITER 1
 #include <ubik.h>
@@ -554,8 +555,8 @@ Unlock(struct cmd_syndesc *as, void *arock)
 	if (code && (code != UNOSERVERS)) {
 	    server = 0;
 	    if (conn && conn->conns[count - 1]
-		&& conn->conns[count - 1]->peer) {
-		server = conn->conns[count - 1]->peer->host;
+		&& rx_PeerOf(conn->conns[count - 1])) {
+		server = rx_HostOf(rx_PeerOf(conn->conns[count - 1]));
 	    }
 	    afs_com_err(whoami, code,
 		    "so %s.%s may still be locked (on server %d.%d.%d.%d)",
@@ -1521,10 +1522,9 @@ MyBeforeProc(struct cmd_syndesc *as, void *arock)
 			afs_com_err(whoami, code, "prompting for %s", p + 1);
 			return code;
 		    }
-		    ip = (struct cmd_item *)malloc(sizeof(struct cmd_item));
-		    ip->data = (char *)malloc(strlen(password) + 1);
+		    ip = malloc(sizeof(struct cmd_item));
+		    ip->data = strdup(password);
 		    ip->next = 0;
-		    strcpy(ip->data, password);
 		    as->parms[i].items = ip;
 		}
 	    }
