@@ -33,6 +33,7 @@
 #include <rx/xdr.h>
 #include <rx/rx.h>
 #include <rx/rxkad.h>
+#include <rx/rxkad_convert.h>
 #include <afs/cellconfig.h>
 #include <afs/auth.h>
 #include <afs/com_err.h>
@@ -647,8 +648,8 @@ kamCreateUser(struct rx_call *call, char *aname, char *ainstance,
 	return code;
     }
     code = ubik_EndTrans(tt);
-    KALOG(aname, ainstance, NULL, NULL, NULL, call->conn->peer->host,
-	  LOG_CRUSER);
+    KALOG(aname, ainstance, NULL, NULL, NULL,
+	  rx_HostOf(rx_PeerOf(rx_ConnectionOf(call))), LOG_CRUSER);
     return code;
 }
 
@@ -923,8 +924,8 @@ kamSetPassword(struct rx_call *call, char *aname, char *ainstance,
 	goto abort;
 
     code = ubik_EndTrans(tt);
-    KALOG(aname, ainstance, NULL, NULL, NULL, call->conn->peer->host,
-	  LOG_CHPASSWD);
+    KALOG(aname, ainstance, NULL, NULL, NULL,
+	  rx_HostOf(rx_PeerOf(rx_ConnectionOf(call))), LOG_CHPASSWD);
     return code;
 
   abort:
@@ -1246,15 +1247,15 @@ Authenticate(int version, struct rx_call *call, char *aname, char *ainstance,
     DES_pcbc_encrypt(oanswer->SeqBody, oanswer->SeqBody, oanswer->SeqLen,
 		     &user_schedule, ktc_to_cblockptr(&tentry.key), ENCRYPT);
     code = ubik_EndTrans(tt);
-    KALOG(aname, ainstance, sname, sinst, NULL, call->conn->peer->host,
-	  LOG_AUTHENTICATE);
+    KALOG(aname, ainstance, sname, sinst, NULL,
+	  rx_HostOf(rx_PeerOf(rx_ConnectionOf(call))), LOG_AUTHENTICATE);
     return code;
 
   abort:
     COUNT_ABO;
     ubik_AbortTrans(tt);
-    KALOG(aname, ainstance, sname, sinst, NULL, call->conn->peer->host,
-	  LOG_AUTHFAILED);
+    KALOG(aname, ainstance, sname, sinst, NULL,
+	  rx_HostOf(rx_PeerOf(rx_ConnectionOf(call))), LOG_AUTHFAILED);
     return code;
 }
 
@@ -1455,8 +1456,8 @@ kamSetFields(struct rx_call *call,
 	goto abort;
 
     code = ubik_EndTrans(tt);
-    KALOG(aname, ainstance, NULL, NULL, NULL, call->conn->peer->host,
-	  LOG_SETFIELDS);
+    KALOG(aname, ainstance, NULL, NULL, NULL,
+	  rx_HostOf(rx_PeerOf(rx_ConnectionOf(call))), LOG_SETFIELDS);
     return code;
 
   abort:
@@ -1528,8 +1529,8 @@ kamDeleteUser(struct rx_call *call, char *aname, char *ainstance)
 	goto abort;
 
     code = ubik_EndTrans(tt);
-    KALOG(aname, ainstance, NULL, NULL, NULL, call->conn->peer->host,
-	  LOG_DELUSER);
+    KALOG(aname, ainstance, NULL, NULL, NULL,
+	  rx_HostOf(rx_PeerOf(rx_ConnectionOf(call))), LOG_DELUSER);
     return code;
 }
 
@@ -1631,7 +1632,7 @@ kamGetEntry(struct rx_call *call,
      * only return user's key if security disabled or if admin and
      * we have an encrypted connection to the user
      */
-    rxkad_GetServerInfo(call->conn, &enc_level, 0, 0, 0, 0, 0);
+    rxkad_GetServerInfo(rx_ConnectionOf(call), &enc_level, 0, 0, 0, 0, 0);
     if ((noAuthenticationRequired)
 	|| (callerIsAdmin && enc_level == rxkad_crypt))
 	memcpy(&aentry->key, &tentry.key, sizeof(struct ktc_encryptionKey));
@@ -1932,7 +1933,7 @@ GetTicket(int version,
 		     &schedule, ktc_to_cblockptr(&authSessionKey), ENCRYPT);
     code = ubik_EndTrans(tt);
     KALOG(name, instance, sname, sinstance, (import ? authDomain : NULL),
-	  call->conn->peer->host, LOG_GETTICKET);
+	  rx_HostOf(rx_PeerOf(rx_ConnectionOf(call))), LOG_GETTICKET);
     return code;
 
   abort:
@@ -2258,8 +2259,8 @@ SKAM_Unlock(struct rx_call *call,
     kaux_write(to, 0, 0);	/* zero failure counters at this offset */
 
     code = ubik_EndTrans(tt);
-    KALOG(aname, ainstance, NULL, NULL, NULL, call->conn->peer->host,
-	  LOG_UNLOCK);
+    KALOG(aname, ainstance, NULL, NULL, NULL,
+	  rx_HostOf(rx_PeerOf(rx_ConnectionOf(call))), LOG_UNLOCK);
     goto exit;
 
   abort:
