@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -19,8 +19,6 @@
 #ifndef __KAUTILS__
 #define __KAUTILS__
 
-#if !defined(UKERNEL)
-#include <des.h>
 #include <afs/auth.h>
 
 #ifndef KAMAJORVERSION
@@ -39,12 +37,6 @@
 #include <afs/cellconfig.h>
 #include <afs/afsutil.h>
 
-#else
-#include "ubik.h"
-#include "afs/auth.h"
-#include "afs/cellconfig.h"
-#endif /* !defined(UKERNEL) */
-
 
 #define KA_TIMESTR_LEN 30
 #define Date afs_uint32
@@ -69,7 +61,7 @@ extern afs_int32 ka_GetAdminToken(char *name, char *instance, char *cell,
 extern afs_int32 ka_VerifyUserToken(char *name, char *instance, char *cell,
 				    struct ktc_encryptionKey *key);
 
-extern void ka_ExplicitCell(char *cell, afs_int32 serverList[]
+extern void ka_ExplicitCell(char *cell, afs_uint32 serverList[]
     );
 
 extern afs_int32 ka_GetServers(char *cell, struct afsconf_cell *cellinfo);
@@ -132,7 +124,7 @@ extern char *ka_LocalCell(void
 
 extern int ka_ExpandCell(char *cell, char *fullCell, int *alocal);
 
-extern int ka_CellToRealm(char *cell, char *realm, int *local);
+extern int ka_CellToRealm(char *cell, char *realm, int *local) AFS_NONNULL((2));
 
 extern void ka_PrintUserID(char *prefix, char *name, char *instance,
 			   char *postfix);
@@ -147,10 +139,14 @@ extern int umin(afs_uint32 a, afs_uint32 b);
 
 extern afs_int32 ka_KeyCheckSum(char *key, afs_uint32 * cksumP);
 
-extern int ka_KeyIsZero(register char *akey, register int alen);
+extern int ka_KeyIsZero(char *akey, int alen);
 
 extern void ka_timestr(afs_int32 time, char *tstr, afs_int32 tlen);
 
+extern void ka_debugKeyCache(struct ka_debugInfo *info);
+
+extern void save_principal(char *p, char *n, char *i, char *c);
+
 extern afs_int32 ka_GetAFSTicket(char *name, char *instance, char *realm,
 				 Date lifetime, afs_int32 flags);
 
@@ -287,7 +283,7 @@ struct ka_getTicketAnswer {
 };
 
 #ifndef ERROR_TABLE_BASE_KA
-#define ka_ErrorString error_message
+#define ka_ErrorString afs_error_message
 #undef  KAMINERROR
 #define KAMINERROR ERROR_TABLE_BASE_KA
 #define KAMAXERROR (KAMINERROR+255)
