@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -11,60 +11,35 @@
  * package. */
 
 #include <afsconfig.h>
-#if defined(UKERNEL)
-#include "afs/param.h"
-#else
 #include <afs/param.h>
-#endif
-
-RCSID
-    ("$Header: /cvs/openafs/src/kauth/user.c,v 1.12.8.1 2007/04/10 18:39:52 shadow Exp $");
-
-#if defined(UKERNEL)
-#include "afs/sysincludes.h"
-#include "afsincludes.h"
-#include "afs/stds.h"
-#include "afs/com_err.h"
-#include "afs/cellconfig.h"
-#include "afs/auth.h"
-#include "afs/ptint.h"
-#include "afs/pterror.h"
-#include "afs/ptserver.h"
-#include "rx/rx.h"
-#include "rx/rx_globals.h"
-#include "rx/rxkad.h"
-#include "afs/kauth.h"
-#include "afs/kautils.h"
-#include "afs/afsutil.h"
-#else /* defined(UKERNEL) */
 #include <afs/stds.h>
-#include <signal.h>
+
+#include <roken.h>
+
+#include <hcrypto/des.h>
+#include <hcrypto/ui.h>
+
 #include <afs/com_err.h>
-#ifdef AFS_NT40_ENV
-#include <winsock2.h>
-#else
-#include <netinet/in.h>
-#endif
-#ifdef HAVE_STRING_H
-#include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
 #include <afs/cellconfig.h>
 #include <afs/auth.h>
 #include <afs/ptint.h>
 #include <afs/pterror.h>
+#include <afs/ptuser.h>
 #include <afs/ptserver.h>
 #include <afs/afsutil.h>
+
+#ifndef UKERNEL
+#include <afs/sys_prototypes.h>
+#endif
+
 #include <rx/rx.h>
 #include <rx/rx_globals.h>
 #include <rx/rxkad.h>		/* max ticket lifetime */
+#include <rx/rxkad_convert.h>
+
 #include "kauth.h"
 #include "kautils.h"
-#endif /* defined(UKERNEL) */
-
+#include <afs/ktc.h>
 
 afs_int32
 GetTickets(char *name, char *instance, char *realm,
@@ -72,7 +47,6 @@ GetTickets(char *name, char *instance, char *realm,
 	   afs_int32 * pwexpires, afs_int32 flags)
 {
     afs_int32 code;
-    struct ktc_token token;
 
     code = ka_GetAuthToken(name, instance, realm, key, lifetime, pwexpires);
     memset(key, 0, sizeof(*key));
@@ -82,7 +56,7 @@ GetTickets(char *name, char *instance, char *realm,
     return code;
 }
 
-/* 
+/*
  * Requires that you already possess a TGT.
  */
 afs_int32
@@ -153,13 +127,17 @@ ka_GetAFSTicket(char *name, char *instance, char *realm, Date lifetime,
 #endif
 
 afs_int32
-ka_UserAuthenticateGeneral(afs_int32 flags, char *name, char *instance, char *realm, char *password, Date lifetime, afs_int32 * password_expires,	/* days 'til, or don't change if not set */
+ka_UserAuthenticateGeneral(afs_int32 flags, char *name, char *instance,
+			   char *realm, char *password, Date lifetime,
+			   afs_int32 * password_expires,	/* days 'til, or don't change if not set */
 			   afs_int32 spare2, char **reasonP)
 {
     int remainingTime = 0;
     struct ktc_encryptionKey key;
     afs_int32 code, dosetpag = 0;
-    int (*old) ();
+#if !defined(AFS_NT40_ENV) && !defined(AFS_LINUX20_ENV) && !defined(AFS_USR_LINUX20_ENV) && !defined(AFS_XBSD_ENV) || defined(AFS_FBSD_ENV)
+    void (*old)(int);
+#endif
 
     if (reasonP)
 	*reasonP = "";
@@ -176,7 +154,7 @@ ka_UserAuthenticateGeneral(afs_int32 flags, char *name, char *instance, char *re
 
     ka_StringToKey(password, realm, &key);
 
-/* 
+/*
  * alarm is set by klogin and kpasswd only so ignore for
  * NT
  */
@@ -191,9 +169,9 @@ ka_UserAuthenticateGeneral(afs_int32 flags, char *name, char *instance, char *re
     }
 #endif
 
-#if !defined(AFS_NT40_ENV) && !defined(AFS_LINUX20_ENV) && !defined(AFS_USR_LINUX20_ENV) && !defined(AFS_XBSD_ENV)
+#if !defined(AFS_NT40_ENV) && !defined(AFS_LINUX20_ENV) && !defined(AFS_USR_LINUX20_ENV) && (!defined(AFS_XBSD_ENV) || defined(AFS_FBSD_ENV))
     /* handle smoothly the case where no AFS system calls exists (yet) */
-    old = (int (*)())signal(SIGSYS, SIG_IGN);
+    old = signal(SIGSYS, SIG_IGN);
 #endif
 #ifdef	AFS_DECOSF_ENV
     (void)signal(SIGTRAP, SIG_IGN);
@@ -203,7 +181,7 @@ ka_UserAuthenticateGeneral(afs_int32 flags, char *name, char *instance, char *re
     if (flags & KA_USERAUTH_ONLY_VERIFY) {
 	code = ka_VerifyUserToken(name, instance, realm, &key);
 	if (code == KABADREQUEST) {
-	    des_string_to_key(password, &key);
+	    DES_string_to_key(password, ktc_to_cblockptr(&key));
 	    code = ka_VerifyUserToken(name, instance, realm, &key);
 	}
     } else {
@@ -228,44 +206,11 @@ ka_UserAuthenticateGeneral(afs_int32 flags, char *name, char *instance, char *re
 	    GetTickets(name, instance, realm, &key, lifetime,
 		       password_expires, dosetpag);
 	if (code == KABADREQUEST) {
-	    des_string_to_key(password, &key);
+	    DES_string_to_key(password, ktc_to_cblockptr(&key));
 	    code =
 		GetTickets(name, instance, realm, &key, lifetime,
 			   password_expires, dosetpag);
 	}
-
-/* By the time 3.3 comes out, these "old-style" passwd programs should be 
- * well and truly obsolete.  Any passwords set with such a program
- * OUGHT to have been changed years ago.  Having 2 -or- 3
- * authentication RPCs generated for every klog plays hob with the
- * "failed login limits" code in the kaserver, and it's hard to
- * explain to admins just how to set the limit properly.  By removing 
- * this function, we can just double it internally in the kaserver, and 
- * not document anything.  kpasswd had the TRUNCATEPASSWORD "feature"
- * disabled on 10/02/90.
- */
-#ifdef OLDCRUFT
-	if ((code == KABADREQUEST) && (strlen(password) > 8)) {
-	    /* try with only the first 8 characters incase they set their password
-	     * with an old style passwd program. */
-	    char pass8[9];
-	    strncpy(pass8, password, 8);
-	    pass8[8] = 0;
-	    ka_StringToKey(pass8, realm, &key);
-	    memset(pass8, 0, sizeof(pass8));
-	    code =
-		GetTickets(name, instance, realm, &key, lifetime,
-			   password_expires, dosetpag);
-	    if (code == 0) {
-		fprintf(stderr, "%s %s\n%s %s\n%s\n",
-			"Warning: you have typed a password longer than 8",
-			"characters, but only the",
-			"first 8 characters were actually significant.  If",
-			"you change your password",
-			"again this warning message will go away.\n");
-	    }
-	}
-#endif /* OLDCRUFT */
     }
 
 #ifndef AFS_NT40_ENV
@@ -313,7 +258,7 @@ ka_UserReadPassword(char *prompt, char *password, int plen, char **reasonP)
     code = ka_Init(0);
     if (code)
 	return code;
-    code = read_pw_string(password, plen, prompt, 0);
+    code = UI_UTIL_read_pw_string(password, plen, prompt, 0);
     if (code)
 	code = KAREADPW;
     else if (strlen(password) == 0)
