@@ -14,6 +14,8 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+#include <roken.h>
+
 #include <ctype.h>
 
 #include "bos.h"
@@ -65,7 +67,7 @@ static char *day[] = {
 static int
 LocalFreeTokens(struct token *alist)
 {
-    register struct token *nlist;
+    struct token *nlist;
     for (; alist; alist = nlist) {
 	nlist = alist->next;
 	free(alist->key);
@@ -90,8 +92,8 @@ LocalParseLine(char *aline, struct token **alist)
     char *tptr = NULL;
     int inToken;
     struct token *first, *last;
-    register struct token *ttok;
-    register int tc;
+    struct token *ttok;
+    int tc;
 
     inToken = 0;		/* not copying token chars at start */
     first = NULL;
@@ -102,10 +104,9 @@ LocalParseLine(char *aline, struct token **alist)
 	    if (inToken) {
 		inToken = 0;	/* end of this token */
 		*tptr++ = 0;
-		ttok = (struct token *)malloc(sizeof(struct token));
+		ttok = malloc(sizeof(struct token));
 		ttok->next = NULL;
-		ttok->key = (char *)malloc(strlen(tbuffer) + 1);
-		strcpy(ttok->key, tbuffer);
+		ttok->key = strdup(tbuffer);
 		if (last) {
 		    last->next = ttok;
 		    last = ttok;
@@ -154,8 +155,8 @@ ParseTime(bos_RestartTime_p ak, char *astr)
 {
     int field;
     short temp;
-    register char *tp;
-    register int tc;
+    char *tp;
+    int tc;
 
     field = 0;			/* 0=hour, 1=min, 2=sec */
     temp = 0;
@@ -194,7 +195,7 @@ int
 ktime_ParsePeriodic(char *adate, bos_RestartTime_p ak)
 {
     struct token *tt;
-    register afs_int32 code;
+    afs_int32 code;
     struct ptemp *tp;
 
     memset(ak, 0, sizeof(*ak));
@@ -1270,13 +1271,14 @@ DoBosHostList(struct cmd_syndesc *as, void *arock)
 	    (cellHandle, as->parms[SERVER].items->data, &bos_server, &st)) {
 	    ERR_ST_EXT("bos_ServerOpen", st);
 	}
+
+        printf("Listing hosts at server %s\n", as->parms[SERVER].items->data);
     }
 
     if (!bos_HostGetBegin(bos_server, &iter, &st)) {
 	ERR_ST_EXT("bos_HostGetBegin", st);
     }
 
-    printf("Listing hosts at server %s\n", as->parms[SERVER].items->data);
 
     while (bos_HostGetNext(iter, host, &st)) {
 	printf("\t%s\n", host);
@@ -1363,9 +1365,9 @@ DoBosExecutableTimestampGet(struct cmd_syndesc *as, void *arock)
     afs_status_t st = 0;
     void *bos_server = NULL;
     const char *executable = NULL;
-    unsigned long new_time = 0;
-    unsigned long old_time = 0;
-    unsigned long bak_time = 0;
+    afs_int32 new_time = 0;
+    afs_int32 old_time = 0;
+    afs_int32 bak_time = 0;
 
     if (as->parms[SERVER].items) {
 	if (!bos_ServerOpen
