@@ -29,19 +29,10 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+#include <roken.h>
+#include <afs/opr.h>
 
-#include <stdio.h>
-#ifdef	AFS_AIX32_ENV
-#include <signal.h>
-#endif
-
-#include <string.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <errno.h>
-#include <sys/ioctl.h>
 #include <afs/vice.h>
-#include <sys/file.h>
 
 #include <afs/auth.h>
 #include <afs/cellconfig.h>
@@ -74,7 +65,7 @@ CommandProc(struct cmd_syndesc *as, void *arock)
 
     if (as->parms[0].items) {	/* A cell is provided */
 	for (itp = as->parms[0].items; itp; itp = itp->next) {
-	    if (i >= MAXCELLS) {
+	    if (i > MAXCELLS) {
 		printf
 		    ("The maximum number of cells (%d) is exceeded; the rest are ignored\n",
 		     MAXCELLS);
@@ -152,8 +143,7 @@ unlog_ForgetCertainTokens(char **list, int listSize)
 	code = ktc_ListTokens(count, &count, &serviceName);
     } while (!code);
 
-    tokenInfoP =
-	(struct tokenInfo *)malloc((sizeof(struct tokenInfo) * count));
+    tokenInfoP = malloc((sizeof(struct tokenInfo) * count));
     if (!tokenInfoP) {
 	perror("unlog_ForgetCertainTokens -- osi_Alloc failed");
 	exit(1);
