@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -10,18 +10,20 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+#include <roken.h>
+#include <afs/opr.h>
 
-#include <stdio.h>
 #ifdef AFS_NT40_ENV
 #include <WINNT/afsevent.h>
 #endif
 #include <ctype.h>
-#include <string.h>
+
 #include <rx/rx.h>
 #include <rx/xdr.h>
 #include <afs/cellconfig.h>
 #include <afs/afsutil.h>
 #include <afs/com_err.h>
+
 #include "ptclient.h"
 #include "ptuser.h"
 #include "pterror.h"
@@ -60,7 +62,7 @@ osi_audit(void)
 int
 main(int argc, char **argv)
 {
-    register afs_int32 code;
+    afs_int32 code;
     char name[PR_MAXNAMELEN];
     char gname[PR_MAXNAMELEN];
     char owner[PR_MAXNAMELEN];
@@ -84,7 +86,7 @@ main(int argc, char **argv)
 	    verbose = 1;
 	else {
 	    if (!strcmp(argv[i], "-c")) {
-		cellname = (char *)malloc(100);
+		cellname = malloc(100);
 		strncpy(cellname, argv[++i], 100);
 	    } else
 		strncpy(buf, argv[i], 150);
@@ -146,8 +148,7 @@ main(int argc, char **argv)
 		    } else {
 			/* add the members of a group to the group */
 			if (verbose)
-			    printf("Adding %s to %s.\n",
-				   lnames.namelist_val[i], gname);
+			    printf("Adding %s to %s.\n", name, gname);
 			code = pr_ListMembers(name, &lnames);
 			if (code) {
 			    fprintf(stderr,
