@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -10,17 +10,18 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+#include <roken.h>
 
-#include <stdio.h>
 #ifdef AFS_NT40_ENV
 #include <WINNT/afsevent.h>
 #endif
-#include <string.h>
+
 #include <rx/rx.h>
 #include <rx/xdr.h>
 #include <afs/cellconfig.h>
 #include <afs/afsutil.h>
 #include <afs/com_err.h>
+
 #include "ptclient.h"
 #include "ptuser.h"
 #include "ptprototypes.h"
@@ -41,7 +42,7 @@ int
 main(afs_int32 argc, char **argv)
 {
 
-    register afs_int32 code;
+    afs_int32 code;
     char name[PR_MAXNAMELEN];
     afs_int32 id;
     char buf[150];
@@ -64,7 +65,7 @@ main(afs_int32 argc, char **argv)
 	    verbose = 1;
 	else {
 	    if (!strcmp(argv[i], "-c")) {
-		cellname = (char *)malloc(100);
+		cellname = malloc(100);
 		strncpy(cellname, argv[++i], 100);
 	    } else
 		strncpy(buf, argv[i], 150);
