@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -10,41 +10,30 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/ptserver/testpt.c,v 1.14.14.5 2008/10/27 23:41:46 shadow Exp $");
+#include <roken.h>
+#include <afs/opr.h>
 
 #include <ctype.h>
-#include <errno.h>
-#include <sys/types.h>
+#include <math.h>
+
 #ifdef AFS_NT40_ENV
-#include <winsock2.h>
 #include <WINNT/afsevent.h>
-#else
-#include <sys/socket.h>
-#include <netdb.h>
-#include <netinet/in.h>
-#endif
-#include <string.h>
-#ifdef HAVE_FCNTL_H
-#include <fcntl.h>
-#endif
-#ifdef HAVE_SYS_STAT_H
-#include <sys/stat.h>
-#endif
-#ifdef HAVE_SYS_TYPES_H
-#include <sys/types.h>
 #endif
+
 #include <rx/rx.h>
 #include <rx/xdr.h>
 #include <afs/rxgen_consts.h>
 #include <afs/cmd.h>
 #include <afs/auth.h>
 #include <afs/cellconfig.h>
-#include "ptclient.h"
-#include "pterror.h"
 #include <afs/afsutil.h>
 #include <afs/com_err.h>
 
+#include "ptclient.h"
+#include "pterror.h"
+#include "ptuser.h"
+#include "ptprototypes.h"
+
 static char *whoami = "testpr";
 static struct afsconf_dir *conf;	/* cell info, set by MyBeforeProc */
 static char conf_dir[100];
@@ -109,7 +98,7 @@ ListUsedIds(struct cmd_syndesc *as, void *arock)
     printf("Checking for %d %sused ids starting at %d.\n", number,
 	   (unused ? "un" : ""), startId);
 #define NUM 100
-    lids.idlist_val = (afs_int32 *) malloc(sizeof(afs_int32) * NUM);
+    lids.idlist_val = malloc(sizeof(afs_int32) * NUM);
     lnames.namelist_len = 0;
     lnames.namelist_val = 0;
     while (number) {
@@ -178,11 +167,11 @@ afs_int32 *groupOwners;		/* ids of owners of groups */
 int nUsers, nGroups, nAdds, nRems, nUDels, nGDels;
 
 int
-IdCmp(afs_int32 *a, afs_int32 *b)
+IdCmp(const void *a, const void *b)
 {
-    if (*a > *b) {
+    if (*(afs_int32 *)a > *(afs_int32 *)b) {
 	return 1;
-    } else if (*a == *b) {
+    } else if (*(afs_int32 *)a == *(afs_int32 *)b) {
 	return 0;
     } else /* (*a < *b) */ {
 	return -1;
@@ -199,7 +188,6 @@ static int
 GetGroupLimit(int N, int x)
 {
     int y;
-    double sqrt();
 
     if ((x >= N) || (x < 0)) {
 	printf("GetGroupLimit: input value out of range %d (%d)\n", x, N);
@@ -271,7 +259,6 @@ CreateGroup(int g)
     afs_int32 code;
     char name[16];
     afs_int32 id = 0;
-    afs_int32 flags = PRGRP;
     afs_int32 owner = 0;
     char *ownerName = NULL;
     int ownerType;		/* type of ownership */
@@ -344,7 +331,7 @@ DeleteRandomId(afs_int32 *list)
     k = random();		/* random starting point */
     for (j = 0; j < number; j++) {	/* find an undeleted id */
 	m = (k + j) % number;
-	if (id = list[m]) {
+	if ((id = list[m])) {
 	    code = ubik_PR_Delete(pruclient, 0, id);
 	    if (code) {
 		afs_com_err(whoami, code, "Couldn't delete %di", id);
@@ -502,22 +489,17 @@ TestManyMembers(struct cmd_syndesc *as, void *arock)
 
     srandom(seed);
 
-    users = (afs_int32 *) malloc(number * sizeof(afs_int32));
-    groups = (afs_int32 *) malloc(number * sizeof(afs_int32));
-    filled = (char *)malloc(number * sizeof(char));
-    cleaned = (char *)malloc(number * sizeof(char));
-    population = (char *)malloc(sqr(number) * sizeof(char));
+    users = calloc(number, sizeof(afs_int32));
+    groups = calloc(number, sizeof(afs_int32));
+    filled = calloc(number, sizeof(char));
+    cleaned = calloc(number, sizeof(char));
+    population = calloc(sqr(number), sizeof(char));
 
     nFilled = 0;
-    memset(filled, 0, number);
     nCleaned = 0;
-    memset(cleaned, 0, number);
-    memset(population, 0, sqr(number));
-    memset(users, 0, number * sizeof(afs_int32));
-    memset(groups, 0, number * sizeof(afs_int32));
 
     ownerUser = lastGroup = 0;
-    groupOwners = (afs_int32 *) malloc(number * sizeof(afs_int32));
+    groupOwners = malloc(number * sizeof(afs_int32));
     nUsers = nGroups = nAdds = nRems = nUDels = nGDels = 0;
 
     while ((nFilled < number) || (nCleaned < number)) {
@@ -576,27 +558,29 @@ TestManyMembers(struct cmd_syndesc *as, void *arock)
 
     /* check the membership list of all users for correctness */
     printf("Starting check of memberships\n");
-    glist = (afs_int32 *) malloc(number * sizeof(afs_int32));
+    glist = malloc(number * sizeof(afs_int32));
     for (u = 0; u < number; u++) {
 	afs_int32 ui = users[u];
 	if (ui) {
 	    int i;
 	    int ng;		/* number groups */
-	    int (*proc) ();	/* membership listing procedure */
 	    int over;
+	    int (*proc)(struct ubik_client *, afs_int32, afs_int32, prlist *,
+	    		afs_int32 *);
 	    prlist alist;
 
 	    alist.prlist_len = 0;
 	    alist.prlist_val = 0;
-	    if (random() & 4)
-		proc = PR_ListElements;
-	    else
-		proc = PR_GetCPS;
-	    code = ubik_Call(proc, pruclient, 0, ui, &alist, &over);
+	    if (random() & 4) {
+	    	proc = ubik_PR_ListElements;
+	    } else {
+	    	proc = ubik_PR_GetCPS;
+	    }
+	    code = (*proc)(pruclient, 0, ui, &alist, &over);
 	    if (code) {
 		afs_com_err(whoami, code,
 			"getting membership list of (%di) using %s", ui,
-			((proc == PR_GetCPS) ? "GetCPR" : "ListElements"));
+			(proc == ubik_PR_ListElements?"ListElements":"GetCPS"));
 		exit(24);
 	    }
 	    if (over) {
@@ -607,7 +591,7 @@ TestManyMembers(struct cmd_syndesc *as, void *arock)
 		if (population[u * number + i])
 		    glist[ng++] = groups[i];
 	    qsort(glist, ng, sizeof(afs_int32), IdCmp);
-	    if (ng != (alist.prlist_len - ((proc == PR_GetCPS) ? 3 : 0))) {
+	    if (ng != (alist.prlist_len - ((proc == ubik_PR_GetCPS) ? 3 : 0))) {
 		fprintf(stderr,
 			"Membership list for %di of unexpected length: was %d but expected %d\n",
 			ui, alist.prlist_len, ng);
@@ -624,7 +608,7 @@ TestManyMembers(struct cmd_syndesc *as, void *arock)
 		}
 	    if (code)
 		exit(21);
-	    if (proc == PR_GetCPS) {
+	    if (proc == ubik_PR_GetCPS) {
 		if ((alist.prlist_val[i /* =ng */ ] != AUTHUSERID) ||
 		    (alist.prlist_val[++i] != ANYUSERID)
 		    || (alist.prlist_val[++i] != ui)) {
@@ -852,7 +836,7 @@ TestPrServ(struct cmd_syndesc *as, void *arock)
     strcat(name, ":abcdefghijklmnopqrstuvwxyz");
     name[0] = 1;		/* bash the owner name */
     id = 0;
-    code = pr_CreateGroup(name, &id);
+    code = pr_CreateGroup(name, creator, &id);
     if (code != PRNOENT) {	/* owner doesn't exist */
 	afs_com_err(whoami, code, "succeeded creating %s", name);
 	exit(9);
@@ -1004,7 +988,7 @@ MyBeforeProc(struct cmd_syndesc *as, void *arock)
 	}
 
 	strcompose(tmp_conf_file, 128, tmp_conf_dir, "/",
-		   AFSDIR_CELLSERVDB_FILE, NULL);
+		   AFSDIR_CELLSERVDB_FILE, (char *)NULL);
 	f = fopen(tmp_conf_file, "w");
 	if (f == 0) {
 	  cantcreate:
@@ -1014,7 +998,7 @@ MyBeforeProc(struct cmd_syndesc *as, void *arock)
 	}
 	fprintf(f, ">%s\n", lcell);
 	for (i = 0; i < cellinfo.numServers; i++) {
-	    register unsigned char *tp =
+	    unsigned char *tp =
 		(unsigned char *)&cellinfo.hostAddr[i].sin_addr;
 	    fprintf(f, "%d.%d.%d.%d\t#%s\n", tp[0], tp[1], tp[2], tp[3],
 		    cellinfo.hostName[i]);
@@ -1027,7 +1011,7 @@ MyBeforeProc(struct cmd_syndesc *as, void *arock)
 	}
 
 	strcompose(tmp_cell_file, 128, tmp_conf_dir, "/",
-		   AFSDIR_THISCELL_FILE, NULL);
+		   AFSDIR_THISCELL_FILE, (char *)NULL);
 	f = fopen(tmp_cell_file, "w");
 	if (f == 0)
 	    goto cantcreate;
@@ -1036,7 +1020,7 @@ MyBeforeProc(struct cmd_syndesc *as, void *arock)
 	    goto cantclose;
 
 	strcompose(tmp_noauth_file, 128, tmp_conf_dir, "/",
-		   AFSDIR_NOAUTH_FILE, NULL);
+		   AFSDIR_NOAUTH_FILE, (char *)NULL);
 	if (noAuth) {
 	    code = creat(tmp_noauth_file, 0777);
 	    if (code && (errno != EEXIST))
@@ -1056,7 +1040,7 @@ MyBeforeProc(struct cmd_syndesc *as, void *arock)
 }
 
 static void
-add_std_args(register struct cmd_syndesc *ts)
+add_std_args(struct cmd_syndesc *ts)
 {
     cmd_Seek(ts, 12);
     cmd_AddParm(ts, "-confdir", CMD_SINGLE, CMD_OPTIONAL,
@@ -1067,7 +1051,7 @@ add_std_args(register struct cmd_syndesc *ts)
 }
 
 int
-osi_audit()
+osi_audit(void)
 {
 /* OK, this REALLY sucks bigtime, but I can't tell who is calling
  * afsconf_CheckAuth easily, and only *SERVERS* should be calling osi_audit
@@ -1078,7 +1062,7 @@ osi_audit()
 
 #include "AFS_component_version_number.c"
 
-int 
+int
 main(int argc, char *argv[])
 {
     afs_int32 code;
