@@ -1,27 +1,17 @@
-/* $Id: ubik.c,v 1.5 2003/07/15 23:16:04 shadow Exp $ */
+/* $Id$ */
 
 #include <afsconfig.h>
 #include <afs/param.h>
+#include <afs/stds.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/ptserver/ubik.c,v 1.5 2003/07/15 23:16:04 shadow Exp $");
-
-#include <sys/types.h>
-#include <netinet/in.h>
-
-#ifdef HAVE_STRING_H
-#include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
+#include <roken.h>
 
 #include <lock.h>
+#include <afs/cellconfig.h>
 #define UBIK_INTERNALS
-#include <afs/stds.h>
 #include <ubik.h>
 #include <rx/xdr.h>
+
 #include "ptint.h"
 #include "ptserver.h"
 
@@ -29,13 +19,15 @@ extern int dbase_fd;
 struct ubik_dbase *dbase;
 
 int
-ubik_ServerInit()
+ubik_ServerInit(afs_uint32 myHost, short myPort, afs_uint32 serverList[],
+                const char *pathName, struct ubik_dbase **dbase)
 {
     return (0);
 }
 
 int
-ubik_BeginTrans()
+ubik_BeginTrans(struct ubik_dbase *dbase, afs_int32 transMode,
+		struct ubik_trans **transPtr)
 {
     static int init = 0;
     struct ubik_hdr thdr;
@@ -44,7 +36,7 @@ ubik_BeginTrans()
 	thdr.version.epoch = htonl(2);
 	thdr.version.counter = htonl(0);
 	thdr.magic = htonl(UBIK_MAGIC);
-	thdr.size = htonl(HDRSIZE);
+	thdr.size = htons(HDRSIZE);
 	lseek(dbase_fd, 0, 0);
 	write(dbase_fd, &thdr, sizeof(thdr));
 	fsync(dbase_fd);
@@ -54,67 +46,73 @@ ubik_BeginTrans()
 }
 
 int
-ubik_BeginTransReadAny()
+ubik_BeginTransReadAny(struct ubik_dbase *dbase, afs_int32 transMode,
+                       struct ubik_trans **transPtr)
 {
     return (0);
 }
 
 int
-ubik_AbortTrans()
+ubik_AbortTrans(struct ubik_trans *transPtr)
 {
     return (0);
 }
 
 int
-ubik_EndTrans()
+ubik_EndTrans(struct ubik_trans *transPtr)
 {
     return (0);
 }
 
 int
-ubik_Tell()
+ubik_Tell(struct ubik_trans *transPtr, afs_int32 * fileid,
+          afs_int32 * position)
 {
     return (0);
 }
 
 int
-ubik_Truncate()
-{
-    return (0);
-}
-
-long
-ubik_SetLock()
+ubik_Truncate(struct ubik_trans *transPtr, afs_int32 length)
 {
     return (0);
 }
 
 int
-ubik_WaitVersion()
+ubik_SetLock(struct ubik_trans *atrans, afs_int32 apos, afs_int32 alen,
+             int atype)
 {
     return (0);
 }
 
 int
-ubik_CacheUpdate()
+ubik_WaitVersion(struct ubik_dbase *adatabase,
+                 struct ubik_version *aversion)
 {
     return (0);
 }
 
 int
-panic(a, b, c, d)
-     char *a, *b, *c, *d;
+ubik_CheckCache(struct ubik_trans *atrans, ubik_updatecache_func cbf, void *rock)
 {
-    printf(a, b, c, d);
+    return (*cbf)(atrans, rock);
+}
+
+void
+panic(char *format, ...)
+{
+    va_list ap;
+
+    va_start(ap, format);
+    vprintf(format, ap);
+    va_end(ap);
+
     abort();
     printf("BACK FROM ABORT\n");	/* shouldn't come back from floating pt exception */
     exit(1);			/* never know, though */
 }
 
 int
-ubik_GetVersion(dummy, ver)
-     int dummy;
-     struct ubik_version *ver;
+ubik_GetVersion(struct ubik_trans *dummy, struct ubik_version *ver)
 {
     memset(ver, 0, sizeof(struct ubik_version));
     return (0);
@@ -122,10 +120,7 @@ ubik_GetVersion(dummy, ver)
 
 
 int
-ubik_Seek(tt, afd, pos)
-     struct ubik_trans *tt;
-     long afd;
-     long pos;
+ubik_Seek(struct ubik_trans *tt, afs_int32 afd, afs_int32 pos)
 {
     if (lseek(dbase_fd, pos + HDRSIZE, 0) < 0) {
 	perror("ubik_Seek");
@@ -135,10 +130,7 @@ ubik_Seek(tt, afd, pos)
 }
 
 int
-ubik_Write(tt, buf, len)
-     struct ubik_trans *tt;
-     char *buf;
-     long len;
+ubik_Write(struct ubik_trans *tt, void *buf, afs_int32 len)
 {
     int status;
 
@@ -151,10 +143,7 @@ ubik_Write(tt, buf, len)
 }
 
 int
-ubik_Read(tt, buf, len)
-     struct ubik_trans *tt;
-     char *buf;
-     long len;
+ubik_Read(struct ubik_trans *tt, void *buf, afs_int32 len)
 {
     int status;
 
@@ -164,7 +153,7 @@ ubik_Read(tt, buf, len)
 	return (1);
     }
     if (status < len)
-	memset(&buf[status], 0, len - status);
+	memset((char *)buf + status, 0, len - status);
     return (0);
 }
 
@@ -182,7 +171,8 @@ struct rx_securityClass *ubik_sc[3];
 
 /* Other declarations */
 
-afsconf_GetNoAuthFlag()
+int
+afsconf_GetNoAuthFlag(struct afsconf_dir *adir)
 {
     return (1);
 }
@@ -190,5 +180,3 @@ afsconf_GetNoAuthFlag()
 
 char *prdir = "/dev/null";
 struct prheader cheader;
-int pr_realmNameLen;
-char *pr_realmName;
