@@ -12,8 +12,9 @@
 
 
 #include "rx/rx_kcommon.h"
-#include "rx/rx_packet.h"
 #include "rx/rx_atomic.h"
+#include "rx/rx_internal.h"
+#include "rx/rx_packet.h"
 #include "rx/rx_stats.h"
 
 #ifdef AFS_DARWIN80_ENV
@@ -37,6 +38,9 @@ rx_upcall(socket_t so, void *arg, __unused int waitflag)
     afs_int32 savelen;          /* was using rlen but had aliasing problems */
     size_t nbytes, resid, noffset;
 
+    /* See if a check for additional packets was issued */
+    rx_CheckPackets();
+
     p = rxi_AllocPacket(RX_PACKET_CLASS_RECEIVE);
     rx_computelen(p, tlen);
     rx_SetDataSize(p, tlen);    /* this is the size of the user data area */
