@@ -582,7 +582,7 @@ s_output(int argc, char *argv[], char *infile, char *define, int extend,
 	return;
     }
     if (nomain) {
-	write_programs((char *)NULL);
+	write_programs(NULL);
     } else {
 	write_most();
 	do_registers(argc, argv);
@@ -663,6 +663,10 @@ C_output(char *infile, char *define, int extend, char *outfile, int append)
 	    f_print(fout, "#include <afsconfig.h>\n");
 	    f_print(fout, "#include <afs/param.h>\n");
 	    f_print(fout, "#include <roken.h>\n");
+	    f_print(fout, "#include <afs/opr.h>\n");
+	    f_print(fout, "#ifdef AFS_PTHREAD_ENV\n");
+	    f_print(fout, "# include <opr/lock.h>\n");
+	    f_print(fout, "#endif\n");
 	    f_print(fout, "#include \"%s\"\n\n", include);
 	}
 	free(include);
