@@ -1,7 +1,7 @@
 /*
  * Copyright 2006, Sine Nomine Associates and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -18,30 +18,17 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+#include <roken.h>
 
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
-#include <dirent.h>
-#include <sys/stat.h>
-#include <time.h>
-#include <errno.h>
 #ifdef AFS_NT40_ENV
-#include <io.h>
 #include <WINNT/afsevent.h>
-#else
-#include <sys/param.h>
-#include <sys/file.h>
-#ifndef ITIMER_REAL
-#include <sys/time.h>
-#endif /* ITIMER_REAL */
 #endif
-#include <rx/xdr.h>
-#include <afs/afsint.h>
-#include <afs/assert.h>
-#include <afs/dir.h>
 
-#include <fcntl.h>
+#include <rx/xdr.h>
+#include <rx/rx_queue.h>
+#include <afs/afsint.h>
+#include <afs/opr_assert.h>
+#include <afs/dir.h>
 
 #ifndef AFS_NT40_ENV
 #include <afs/osi_inode.h>
@@ -51,15 +38,15 @@
 #include <afs/afsutil.h>
 #include <afs/fileutil.h>
 
-#include "nfs.h"
-#include "lwp.h"
-#include "lock.h"
-#include "ihandle.h"
-#include "vnode.h"
-#include "volume.h"
-#include "partition.h"
-#include "daemon_com.h"
-#include "salvsync.h"
+#include <afs/nfs.h>
+#include <lwp.h>
+#include <lock.h>
+#include <afs/ihandle.h>
+#include <afs/vnode.h>
+#include <afs/volume.h>
+#include <afs/partition.h>
+#include <afs/daemon_com.h>
+#include <afs/salvsync.h>
 #ifdef AFS_NT40_ENV
 #include <pthread.h>
 #endif
@@ -82,15 +69,15 @@ struct salv_state {
     char partName[16];
 };
 
-struct state {
+struct fssync_state {
     afs_int32 reason;
     struct salv_state * sop;
 };
 
-static int common_prolog(struct cmd_syndesc *, struct state *);
-static int common_salv_prolog(struct cmd_syndesc *, struct state *);
+static int common_prolog(struct cmd_syndesc *, struct fssync_state *);
+static int common_salv_prolog(struct cmd_syndesc *, struct fssync_state *);
 
-static int do_salvop(struct state *, afs_int32 command, SYNC_response * res);
+static int do_salvop(struct fssync_state *, afs_int32 command, SYNC_response * res);
 
 static char * response_code_to_string(afs_int32);
 static char * command_code_to_string(afs_int32);
@@ -174,9 +161,10 @@ main(int argc, char **argv)
 }
 
 static int
-common_prolog(struct cmd_syndesc * as, struct state * state)
+common_prolog(struct cmd_syndesc * as, struct fssync_state * state)
 {
-    register struct cmd_item *ti;
+    struct cmd_item *ti;
+    VolumePackageOptions opts;
 
 #ifdef AFS_NT40_ENV
     if (afs_winsockInit() < 0) {
@@ -184,13 +172,22 @@ common_prolog(struct cmd_syndesc * as, struct state * state)
     }
 #endif
 
-    VInitVolumePackage(debugUtility, 1, 1,
-		       DONT_CONNECT_FS, 0);
+    VOptDefaults(debugUtility, &opts);
+    if (VInitVolumePackage2(debugUtility, &opts)) {
+	/* VInitVolumePackage2 can fail on e.g. partition attachment errors,
+	 * but we don't really care, since all we're doing is trying to use
+	 * SALVSYNC */
+	fprintf(stderr, "errors encountered initializing volume package, but "
+	                "trying to continue anyway\n");
+    }
     DInit(1);
 
     if ((ti = as->parms[COMMON_PARMS_OFFSET].items)) {	/* -reason */
 	state->reason = atoi(ti->data);
+    } else {
+	state->reason = SALVSYNC_REASON_WHATEVER;
     }
+
     if ((ti = as->parms[COMMON_PARMS_OFFSET+1].items)) {	/* -programtype */
 	if (!strcmp(ti->data, "fileServer")) {
 	    programType = fileServer;
@@ -200,6 +197,10 @@ common_prolog(struct cmd_syndesc * as, struct state * state)
 	    programType = salvager;
 	} else if (!strcmp(ti->data, "salvageServer")) {
 	    programType = salvageServer;
+	} else if (!strcmp(ti->data, "volumeServer")) {
+	    programType = volumeServer;
+	} else if (!strcmp(ti->data, "volumeSalvager")) {
+	    programType = volumeSalvager;
 	} else {
 	    programType = (ProgramType) atoi(ti->data);
 	}
@@ -211,9 +212,9 @@ common_prolog(struct cmd_syndesc * as, struct state * state)
 }
 
 static int
-common_salv_prolog(struct cmd_syndesc * as, struct state * state)
+common_salv_prolog(struct cmd_syndesc * as, struct fssync_state * state)
 {
-    register struct cmd_item *ti;
+    struct cmd_item *ti;
 
     state->sop = (struct salv_state *) calloc(1, sizeof(struct salv_state));
     assert(state->sop != NULL);
@@ -240,7 +241,7 @@ common_salv_prolog(struct cmd_syndesc * as, struct state * state)
 }
 
 static int
-do_salvop(struct state * state, afs_int32 command, SYNC_response * res)
+do_salvop(struct fssync_state * state, afs_int32 command, SYNC_response * res)
 {
     afs_int32 code;
     SALVSYNC_response_hdr hdr_l, *hdr;
@@ -254,7 +255,7 @@ do_salvop(struct state * state, afs_int32 command, SYNC_response * res)
 	hdr = (SALVSYNC_response_hdr *) res->payload.buf;
     }
 
-    fprintf(stderr, "calling SALVSYNC_SalvageVolume with command code %d (%s)\n", 
+    fprintf(stderr, "calling SALVSYNC_SalvageVolume with command code %d (%s)\n",
 	    command, command_code_to_string(command));
 
     code = SALVSYNC_SalvageVolume(state->sop->volume,
@@ -273,9 +274,9 @@ do_salvop(struct state * state, afs_int32 command, SYNC_response * res)
     }
 
     fprintf(stderr, "SALVSYNC_SalvageVolume returned %d (%s)\n", code, response_code_to_string(code));
-    fprintf(stderr, "protocol response code was %d (%s)\n", 
+    fprintf(stderr, "protocol response code was %d (%s)\n",
 	    res->hdr.response, response_code_to_string(res->hdr.response));
-    fprintf(stderr, "protocol reason code was %d (%s)\n", 
+    fprintf(stderr, "protocol reason code was %d (%s)\n",
 	    res->hdr.reason, reason_code_to_string(res->hdr.reason));
 
     printf("state = {\n");
@@ -395,7 +396,7 @@ state_code_to_string(afs_int32 state)
 static int
 OpStats(struct cmd_syndesc * as, void * rock)
 {
-    struct state state;
+    struct fssync_state state;
 
     common_prolog(as, &state);
     common_salv_prolog(as, &state);
@@ -408,7 +409,7 @@ OpStats(struct cmd_syndesc * as, void * rock)
 static int
 OpSalvage(struct cmd_syndesc * as, void * rock)
 {
-    struct state state;
+    struct fssync_state state;
 
     common_prolog(as, &state);
     common_salv_prolog(as, &state);
@@ -421,7 +422,7 @@ OpSalvage(struct cmd_syndesc * as, void * rock)
 static int
 OpCancel(struct cmd_syndesc * as, void * rock)
 {
-    struct state state;
+    struct fssync_state state;
 
     common_prolog(as, &state);
     common_salv_prolog(as, &state);
@@ -434,7 +435,7 @@ OpCancel(struct cmd_syndesc * as, void * rock)
 static int
 OpCancelAll(struct cmd_syndesc * as, void * rock)
 {
-    struct state state;
+    struct fssync_state state;
 
     common_prolog(as, &state);
     common_salv_prolog(as, &state);
@@ -447,7 +448,7 @@ OpCancelAll(struct cmd_syndesc * as, void * rock)
 static int
 OpRaisePrio(struct cmd_syndesc * as, void * rock)
 {
-    struct state state;
+    struct fssync_state state;
 
     common_prolog(as, &state);
     common_salv_prolog(as, &state);
@@ -460,7 +461,7 @@ OpRaisePrio(struct cmd_syndesc * as, void * rock)
 static int
 OpQuery(struct cmd_syndesc * as, void * rock)
 {
-    struct state state;
+    struct fssync_state state;
 
     common_prolog(as, &state);
     common_salv_prolog(as, &state);
