@@ -11,12 +11,19 @@
 #include <afs/param.h>
 
 #include <roken.h>
+
+#include <assert.h>
+
 #include <afs/opr.h>
+#ifdef AFS_PTHREAD_ENV
+# include <opr/lock.h>
+#else
+# include <opr/lockstub.h>
+#endif
+
 #include <lock.h>
 #include <rx/xdr.h>
 #include <rx/rx.h>
-#include <errno.h>
-#include <assert.h>
 #include <afs/afsutil.h>
 
 #define UBIK_INTERNALS
@@ -559,7 +566,7 @@ SDISK_SendFile(struct rx_call *rxcall, afs_int32 file,
     memcpy(&ubik_dbase->version, avers, sizeof(struct ubik_version));
     udisk_Invalidate(dbase, file);	/* new dbase, flush disk buffers */
 #ifdef AFS_PTHREAD_ENV
-    assert(pthread_cond_broadcast(&dbase->version_cond) == 0);
+    opr_Assert(pthread_cond_broadcast(&dbase->version_cond) == 0);
 #else
     LWP_NoYieldSignal(&dbase->version);
 #endif
