@@ -18,18 +18,7 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-
-#include "uss_vol.h"		/*Interface to this module */
-#include "uss_common.h"		/*Common definitions */
-#include "uss_procs.h"		/*Defs from procs module */
-#include "uss_fs.h"		/*CacheManager ops */
-#include "uss_acl.h"
-#include <sys/stat.h>
-#include <pwd.h>
-#include <netdb.h>
-#include <errno.h>
-
-#include <string.h>
+#include <roken.h>
 
 #include <afs/com_err.h>
 #include <afs/vlserver.h>
@@ -38,11 +27,18 @@
 #include <afs/cellconfig.h>
 #include <rx/rx_globals.h>
 #include <afs/volser.h>
+#include <afs/volser_prototypes.h>
 #include <afs/volint.h>
 #include <afs/keys.h>
 #include <afs/afsutil.h>
 #include <ubik.h>
 
+#include "uss_vol.h"		/*Interface to this module */
+#include "uss_common.h"		/*Common definitions */
+#include "uss_procs.h"		/*Defs from procs module */
+#include "uss_fs.h"		/*CacheManager ops */
+#include "uss_acl.h"
+
 extern int line;
 
 /*
@@ -104,12 +100,11 @@ InitThisModule(int a_noAuthFlag, char *a_confDir, char *a_cellName)
 #ifdef USS_VOL_DB
     static char rn[] = "uss_vol:InitThisModule";
 #endif
-    register afs_int32 code;	/*Return code */
+    afs_int32 code;	/*Return code */
     struct afsconf_dir *tdir;	/*Ptr to conf dir info */
     struct afsconf_cell info;	/*Info about chosen cell */
-    struct ktc_principal sname;	/*Service name */
-    struct ktc_token ttoken;	/*Service ticket */
     afs_int32 scIndex;		/*Chosen security index */
+    afs_int32 secFlags;
     struct rx_securityClass *sc;	/*Generated security object */
     afs_int32 i;		/*Loop index */
 
@@ -157,56 +152,21 @@ InitThisModule(int a_noAuthFlag, char *a_confDir, char *a_cellName)
 #ifdef USS_VOL_DB
     printf("[%s] Getting tickets if needed\n", rn);
 #endif /* USS_VOL_DB */
-    if (!a_noAuthFlag) {
-	/*
-	 * We don't need tickets for unauthenticated connections.
-	 */
-	strcpy(sname.cell, info.name);
-	sname.instance[0] = 0;
-	strcpy(sname.name, "afs");
-	code = ktc_GetToken(&sname, &ttoken, sizeof(ttoken), NULL);
-	if (code) {
-	    fprintf(stderr,
-		    "%s: Couldn't get AFS tokens, running unauthenticated.\n",
-		    uss_whoami);
-	    scIndex = 0;
-	} else {
-	    /*
-	     * We got a ticket, go for an authenticated connection.
-	     */
-	    if (ttoken.kvno >= 0 && ttoken.kvno <= 256)
-		scIndex = 2;	/*Kerberos */
-	    else {
-		fprintf(stderr, "%s: Funny kvno (%d) in ticket, proceeding\n",
-			uss_whoami, ttoken.kvno);
-		scIndex = 2;
-	    }
-	}			/*Got a ticket */
-    } /*Authentication desired */
-    else
-	scIndex = 0;
 
-    /*
-     * Generate the appropriate security object for the connection.
-     */
-#ifdef USS_VOL_DB
-    printf("[%s] Generating Rx security object\n", rn);
-#endif /* USS_VOL_DB */
-    switch (scIndex) {
-    case 0:
-	sc = (struct rx_securityClass *)
-	    rxnull_NewClientSecurityObject();
-	break;
+    secFlags = AFSCONF_SECOPTS_FALLBACK_NULL;
+    if (a_noAuthFlag)
+	secFlags |= AFSCONF_SECOPTS_NOAUTH;
 
-    case 1:
-	break;
-
-    case 2:
-	sc = (struct rx_securityClass *)
-	    rxkad_NewClientSecurityObject(rxkad_clear, &ttoken.sessionKey,
-					  ttoken.kvno, ttoken.ticketLen,
-					  ttoken.ticket);
-	break;
+    code = afsconf_PickClientSecObj(tdir, secFlags, &info, a_cellName,
+				    &sc, &scIndex, NULL);
+    if (code) {
+	printf("%s: Can't create client security object\n", uss_whoami);
+        exit(1);
+    }
+    if (scIndex == RX_SECIDX_NULL && !a_noAuthFlag) {
+	fprintf(stderr,
+		"%s: Couldn't get AFS tokens, running unauthenticated.\n",
+		uss_whoami);
     }
 
     /*
@@ -369,10 +329,10 @@ afs_int32
 uss_vol_GetServer(char *a_name)
 {				/*uss_vol_GetServer */
 
-    register struct hostent *th;
+    struct hostent *th;
     afs_int32 addr;
-    char b1, b2, b3, b4;
-    register afs_int32 code;
+    afs_int32 b1, b2, b3, b4;
+    afs_int32 code;
 
     code = sscanf(a_name, "%d.%d.%d.%d", &b1, &b2, &b3, &b4);
     if (code == 4) {
@@ -380,9 +340,7 @@ uss_vol_GetServer(char *a_name)
 	 * Parsed as 128.2.9.4, or similar; return it in network
 	 * byte order (128 in byte 0).
 	 */
-	addr =
-	    (((afs_int32) b1) << 24) | (((afs_int32) b2) << 16) |
-	    (((afs_int32) b3) << 8) | (afs_int32) b4;
+	addr = (b1 << 24) | (b2 << 16) | (b3 << 8) | b4;
 	return htonl(addr);
     }
 
@@ -446,7 +404,7 @@ afs_int32
 uss_vol_GetPartitionID(char *a_name)
 {				/*uss_vol_GetPartitionID */
 
-    register char tc;
+    char tc;
     char ascii[3];
 
     tc = *a_name;
@@ -617,7 +575,8 @@ uss_vol_CreateVol(char *a_volname, char *a_server, char *a_partition,
     static char rn[] = "uss_vol_CreateVol";	/*Routine name */
 #endif
     afs_int32 pname;		/*Partition name */
-    afs_int32 volid, code;	/*Volume ID, return code */
+    afs_uint32 volid;		/*Volume ID */
+    afs_int32 code;		/*return code */
     afs_int32 saddr;		/*Socket info for server */
     int VolExistFlag = 0;	/*Does the volume exist? */
     int mpExistFlag = 0;	/*Does the mountpoint exist? */
@@ -767,7 +726,7 @@ uss_vol_CreateVol(char *a_volname, char *a_server, char *a_partition,
      * the code if we're doing a dry run.
      */
     if (VolExistFlag) {
-	if ((Oldmpoint = (char *)malloc(strlen(a_mpoint) + 50)) == NULL) {
+	if ((Oldmpoint = malloc(strlen(a_mpoint) + 50)) == NULL) {
 	    fprintf(stderr, "%s: No more memory!\n", uss_whoami);
 	    return (1);
 	}
@@ -863,12 +822,10 @@ uss_vol_CreateVol(char *a_volname, char *a_server, char *a_partition,
      * properly, as well as all ACLs of future subdirectories,as the very last
      * thing we do to the new account.
      */
-    new_dir = (struct uss_subdir *)malloc(sizeof(struct uss_subdir));
+    new_dir = malloc(sizeof(struct uss_subdir));
     new_dir->previous = uss_currentDir;
-    new_dir->path = (char *)malloc(strlen(a_mpoint) + 1);
-    strcpy(new_dir->path, a_mpoint);
-    new_dir->finalACL = (char *)malloc(strlen(a_acl) + 1);
-    strcpy(new_dir->finalACL, a_acl);
+    new_dir->path = strdup(a_mpoint);
+    new_dir->finalACL = strdup(a_acl);
     uss_currentDir = new_dir;
     sprintf(tmp_str, "%s %s all", a_mpoint, uss_AccountCreator);
 
@@ -906,7 +863,7 @@ uss_vol_DeleteVol(char *a_volName, afs_int32 a_volID, char *a_servName,
 #ifdef USS_VOL_DB
     static char rn[] = "uss_vol_DeleteVol";	/*Routine name */
 #endif
-    register afs_int32 code = 0;	/*Return code */
+    afs_int32 code = 0;	/*Return code */
 
     /*
      * Make sure we've initialized our VLDB connection(s) before
@@ -1003,7 +960,7 @@ static void
 ovlentry_to_nvlentry(struct vldbentry *oentryp,
                      struct nvldbentry *nentryp)
 {
-    register int i;
+    int i;
 
     memset(nentryp, 0, sizeof(struct nvldbentry));
     strncpy(nentryp->name, oentryp->name, sizeof(nentryp->name));
@@ -1078,7 +1035,7 @@ uss_vol_GetVolInfoFromMountPoint(char *a_mountpoint)
 #ifdef USS_VOL_DB
     static char rn[] = "uss_vol_GetVolInfoFromMountPoint";
 #endif
-    register afs_int32 code;	/*Return code */
+    afs_int32 code;	/*Return code */
     uss_VolumeStatus_t *statusP;	/*Ptr to returned status */
     afs_int32 volID;		/*Volume ID */
     struct nvldbentry vldbEntry;	/*VLDB entry for volume */
