@@ -27,15 +27,6 @@ extern int base32_to_int(char *s);
 extern char *int_to_base64(b64_string_t s, int a);
 extern int base64_to_int(char *s);
 
-/* config_file.c && krb5_locl.h */
-typedef struct afs_config_section_struct afs_config_section;
-extern int afs_config_parse_file_multi(const char *, afs_config_section **);
-extern int afs_config_parse_file(const char *, afs_config_section **);
-extern int afs_config_file_free(afs_config_section *s);
-extern const char* fs_config_get_string(const afs_config_section *, ...);
-extern int afs_config_get_bool(const afs_config_section *, ...);
-extern int afs_config_get_int(const afs_config_section *c, ...);
-
 /* dirpath.c */
 extern unsigned int initAFSDirPath(void);
 extern const char *getDirPath(afsdir_id_t string_id);
@@ -52,7 +43,6 @@ extern char* afs_exec_alt(int argc, char **argv, const char *prefix,
                           const char *suffix);
 
 /* fileutil.c */
-extern int renamefile(const char *oldname, const char *newname);
 extern void FilepathNormalizeEx(char *path, int slashType);
 extern void FilepathNormalize(char *path);
 
@@ -68,10 +58,6 @@ extern char *int64_to_flipbase64(lb64_string_t s, afs_uint64 a);
 extern afs_int64 flipbase64_to_int64(char *s);
 #endif
 
-/* get_krbrlm.c */
-extern int afs_krb_get_lrealm(char *r, int n);
-extern int afs_krb_exclusion(char *name);
-extern int afs_is_foreign_ticket_name(char *tname, char *tinst, char * tcell, char *localrealm);
 /* hostparse.c */
 extern struct hostent *hostutil_GetHostByName(char *ahost);
 extern char *hostutil_GetNameByINet(afs_uint32 addr);
@@ -116,33 +102,12 @@ extern afs_int32 ktime_DateToInt32(char *adate, afs_int32 * aint32);
 extern char *ktime_GetDateUsage(void);
 extern afs_int32 ktime_InterpretDate(struct ktime_date *akdate);
 
-/* netutils.c */
-extern afs_uint32 extract_Addr(char *line, int maxSize);
-extern int parseNetRestrictFile(afs_uint32 outAddrs[], afs_uint32 * mask,
-				afs_uint32 * mtu, afs_uint32 maxAddrs,
-				afs_uint32 * nAddrs, char reason[],
-				const char *fileName);
-extern int ParseNetInfoFile(afs_uint32 * final, afs_uint32 * mask,
-			    afs_uint32 * mtu, int max, char reason[],
-			    const char *fileName);
-extern int filterAddrs(afs_uint32 addr1[], afs_uint32 addr2[],
-		       afs_uint32 mask1[], afs_uint32 mask2[],
-		       afs_uint32 mtu1[], afs_uint32 mtu2[], int n1, int n2);
-extern int parseNetFiles(afs_uint32 addrbuf[], afs_uint32 maskbuf[],
-			 afs_uint32 mtubuf[], afs_uint32 max, char reason[],
-			 const char *niFileName, const char *nrFileName);
-
-
 /* pthread_glock.c */
 
 
 /* pthread_threadname.c */
 #if defined(AFS_PTHREAD_ENV) && !defined(AFS_NT40_ENV)
-extern void afs_pthread_setname(pthread_t thread, const char *threadname);
 extern void afs_pthread_setname_self(const char *threadname);
-#elif defined(AFS_NT40_ENV)
-# define afs_pthread_setname(thread, threadname) (void)0
-# define afs_pthread_setname_self(threadname) (void)0
 #else
 /* Allow unconditional references to afs_pthread_setname_self to
  * reduce #ifdef spaghetti.
