@@ -23,18 +23,18 @@
 #include <afs/procmgmt.h>	/* signal(), kill(), wait(), etc. */
 
 #include <roken.h>		/* Must come after procmgmt.h */
+#include <afs/opr.h>
 
+#include <afs/opr.h>
 #include "afsutil.h"
 #include "fileutil.h"
 #include <lwp.h>
 
 #if defined(AFS_PTHREAD_ENV)
-/* can't include rx when we are libutil; it's too early */
-#include <rx/rx.h>
 #include <pthread.h>
 static pthread_mutex_t serverLogMutex;
-#define LOCK_SERVERLOG() MUTEX_ENTER(&serverLogMutex)
-#define UNLOCK_SERVERLOG() MUTEX_EXIT(&serverLogMutex)
+#define LOCK_SERVERLOG() opr_Verify(pthread_mutex_lock(&serverLogMutex) == 0);
+#define UNLOCK_SERVERLOG() opr_Verify(pthread_mutex_unlock(&serverLogMutex) == 0);
 
 #ifdef AFS_NT40_ENV
 #define NULLDEV "NUL"
@@ -67,7 +67,6 @@ int serverLogSyslogFacility = LOG_DAEMON;
 char *serverLogSyslogTag = 0;
 #endif
 
-#include <stdarg.h>
 int LogLevel;
 int mrafsStyleLogs = 0;
 static int threadIdLogs = 0;
@@ -160,6 +159,8 @@ LogCommandLine(int argc, char **argv, const char *progname,
     int i, l;
     char *commandLine, *cx;
 
+    opr_Assert(argc > 0);
+
     for (l = i = 0; i < argc; i++)
 	l += strlen(argv[i]) + 1;
     if ((commandLine = malloc(l))) {
@@ -311,7 +312,7 @@ OpenLog(const char *fileName)
 	    goto makefilename;
 	}
 	if (!isfifo)
-	    renamefile(fileName, FileName);	/* don't check error code */
+	    rk_rename(fileName, FileName);	/* don't check error code */
 	tempfd = open(fileName, O_WRONLY | O_TRUNC | O_CREAT | (isfifo?O_NONBLOCK:0), 0666);
     } else {
 	strcpy(oldName, fileName);
@@ -319,7 +320,7 @@ OpenLog(const char *fileName)
 
 	/* don't check error */
 	if (!isfifo)
-	    renamefile(fileName, oldName);
+	    rk_rename(fileName, oldName);
 	tempfd = open(fileName, O_WRONLY | O_TRUNC | O_CREAT | (isfifo?O_NONBLOCK:0), 0666);
     }
 
@@ -337,7 +338,7 @@ OpenLog(const char *fileName)
 #endif
 
 #if defined(AFS_PTHREAD_ENV)
-    MUTEX_INIT(&serverLogMutex, "serverlog", MUTEX_DEFAULT, 0);
+    opr_Verify(pthread_mutex_init(&serverLogMutex, NULL) == 0);
 #endif /* AFS_PTHREAD_ENV */
 
     serverLogFD = tempfd;
