@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -9,43 +9,23 @@
 
 #include <afsconfig.h>
 #include <afs/param.h>
-
-RCSID
-    ("$Header: /cvs/openafs/src/viced/fsprobe.c,v 1.12.2.3 2007/11/26 21:08:45 shadow Exp $");
-
 #include <afs/stds.h>
+
+#include <roken.h>
+
 #include <afs/afsint.h>
-#include <sys/socket.h>
 #include <rx/rx_globals.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
-#include <netdb.h>
 #include <ubik.h>
-#include <string.h>
 
 struct ubik_client *cstruct;
 struct rx_connection *serverconns[MAXSERVERS];
 char *(args[50]);
 
-extern int AFS_FetchData(), AFS_StoreData(), AFS_StoreACL();
-extern int RXAFS_GetTime(), AFS_GetStatistics(), AFS_FetchStatus(),
-AFS_FetchACL();
-extern int AFS_StoreStatus(), AFS_RemoveFile(), AFS_CreateFile();
-extern int AFS_Rename(), AFS_Symlink(), AFS_HardLink(), AFS_MakeDir(),
-AFS_RemoveDir();
-extern int AFS_Readdir(), AFS_MakeMountPoint(), AFS_ReleaseTokens(),
-AFS_GetToken();
-extern int AFS_BulkStatus(), AFS_Lookup();
-extern int AFS_BulkStatus(), AFS_BulkLookup(), AFS_RenewAllTokens();
-extern int AFS_BulkFetchVV(), AFS_BulkKeepAlive();
-extern int AFS_Spare1(), AFS_Spare2(), AFS_Spare3(), AFS_Spare4(),
-AFS_Spare5(), AFS_Spare6();
-
 afs_int32
 pxclient_Initialize(int auth, afs_int32 serverAddr)
 {
     afs_int32 code;
-    afs_int32 scIndex;
+    rx_securityIndex scIndex;
     struct rx_securityClass *sc;
 
     code = rx_Init(htons(2115) /*0 */ );
@@ -53,25 +33,9 @@ pxclient_Initialize(int auth, afs_int32 serverAddr)
 	fprintf(stderr, "pxclient_Initialize:  Could not initialize rx.\n");
 	return code;
     }
-    scIndex = 0;
+    scIndex = RX_SECIDX_NULL;
     rx_SetRxDeadTime(50);
-    switch (scIndex) {
-    case 0:
-	sc = rxnull_NewClientSecurityObject();
-	break;
-
-#ifdef notdef			/* security */
-    case 1:
-	sc = rxvab_NewClientSecurityObject(&ttoken.sessionKey, ttoken.ticket,
-					   0);
-	break;
-
-    case 2:
-	sc = rxkad_NewClientSecurityObject(rxkad_clear, &ttoken.sessionKey,
-					   ttoken.kvno, ttoken.ticketLen,
-					   ttoken.ticket);
-#endif /* notdef */
-    }
+    sc = rxnull_NewClientSecurityObject();
     serverconns[0] =
 	rx_NewConnection(serverAddr, htons(7000), 1, sc, scIndex);
 
@@ -93,20 +57,18 @@ main(int argc, char **argv)
 {
     char **av = argv;
     struct sockaddr_in host;
-    register afs_int32 code;
-    extern struct hostent *gethostbyname();
+    afs_int32 code;
     struct hostent *hp;
-    char *hostname;
     char hnamebuf[200];
     struct timeval tv;
     int noAuth = 1;		/* Default is authenticated connections */
 
     argc--, av++;
     if (argc < 1) {
-	printf("usage: pxclient <serverHost>\n");
+	printf("usage: fsprobe <serverHost>\n");
 	exit(1);
     }
-    memset((char *)&host, 0, sizeof(struct sockaddr_in));
+    memset(&host, 0, sizeof(struct sockaddr_in));
     host.sin_family = AF_INET;
     host.sin_addr.s_addr = inet_addr(av[0]);
 #ifdef STRUCT_SOCKADDR_HAS_SA_LEN
@@ -114,13 +76,11 @@ main(int argc, char **argv)
 #endif
     if (host.sin_addr.s_addr != -1) {
 	strcpy(hnamebuf, av[0]);
-	hostname = hnamebuf;
     } else {
 	hp = gethostbyname(av[0]);
 	if (hp) {
 	    host.sin_family = hp->h_addrtype;
 	    memcpy((caddr_t) & host.sin_addr, hp->h_addr, hp->h_length);
-	    hostname = hp->h_name;
 	} else {
 	    printf("unknown server host %s\n", av[0]);
 	    exit(1);
@@ -131,10 +91,10 @@ main(int argc, char **argv)
 	exit(1);
     }
 
-    code = ubik_Call(RXAFS_GetTime, cstruct, 0, &tv.tv_sec, &tv.tv_usec);
+    code = RXAFS_GetTime(cstruct->conns[0], (afs_uint32 *)&tv.tv_sec, (afs_uint32 *)&tv.tv_usec);
     if (!code)
 	printf("AFS_GetTime on %s sec=%ld, usec=%ld\n", av[0], tv.tv_sec,
-	       tv.tv_usec);
+	       (long int)tv.tv_usec);
     else
 	printf("return code is %d\n", code);
 
@@ -146,7 +106,7 @@ main(int argc, char **argv)
 	printf("fs> ");
 	if (fgets(line, 499, stdin) != NULL) {
 	    char *oper;
-	    register char **argp = args;
+	    char **argp = args;
 	    GetArgs(line, argp, &nargs);
 	    oper = &argp[0][0];
 	    ++argp, --nargs;
@@ -258,11 +218,11 @@ main(int argc, char **argv)
 
 
 void
-GetArgs(register char *line, register char **args, register int *nargs)
+GetArgs(char *line, char **args, int *nargs)
 {
     *nargs = 0;
     while (*line) {
-	register char *last = line;
+	char *last = line;
 	while (*line == ' ')
 	    line++;
 	if (*last == ' ')
@@ -313,10 +273,9 @@ FetchData(char **argp)
 
 
 static afs_int32
-FetchProc(register struct rx_call *acall)
+FetchProc(struct rx_call *acall)
 {
-    extern char *malloc();
-    register char *tbuffer;
+    char *tbuffer;
     afs_int32 tlen, length, code;
 
     code = rx_Read(acall, &length, sizeof(afs_int32));
@@ -454,7 +413,7 @@ StoreData(char **argp)
 
 
 static afs_int32
-StoreProc(register struct rx_call *acall, char *string, int length)
+StoreProc(struct rx_call *acall, char *string, int length)
 {
     afs_int32 tlen, code;
 
@@ -872,10 +831,9 @@ Readdir(char **argp)
 
 
 static afs_int32
-FetchDir(register struct rx_call *acall)
+FetchDir(struct rx_call *acall)
 {
-    extern char *malloc();
-    register char *tbuffer;
+    char *tbuffer;
     afs_int32 tlen, length, code;
     struct dirent *dp;
 
