@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -9,43 +9,17 @@
 
 #include <afsconfig.h>
 #include <afs/param.h>
-
-RCSID
-    ("$Header: /cvs/openafs/src/vlserver/vlclient.c,v 1.16.2.1 2007/02/09 01:00:23 shadow Exp $");
-
 #include <afs/stds.h>
-#include <sys/types.h>
-#ifdef HAVE_TIME_H
-#include <time.h>
-#endif
-#ifdef HAVE_FCNTL_H
-#include <fcntl.h>
-#endif
+
+#include <roken.h>
+
 #ifdef AFS_NT40_ENV
-#include <winsock2.h>
 #include <WINNT/afsevent.h>
 #endif
-#ifdef HAVE_SYS_TIME_H
-#include <sys/time.h>
-#endif
 #ifdef HAVE_SYS_FILE_H
 #include <sys/file.h>
 #endif
-#ifdef HAVE_NETDB_H
-#include <netdb.h>
-#endif
-#ifdef HAVE_NETINET_IN_H
-#include <netinet/in.h>
-#endif
-#include <stdio.h>
-
-#ifdef HAVE_STRING_H
-#include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
+#include <ctype.h>
 
 #include <afs/afsutil.h>
 #include <rx/xdr.h>
@@ -57,19 +31,19 @@ RCSID
 #include <afs/cmd.h>
 #include <lock.h>
 #include <ubik.h>
-#include "vlserver.h"
-#include "vlclient.h"
 
-void fill_listattributes_entry();
-void display_listattributes_entry();
-void display_entry();
-void display_entryN();
-void display_update_entry();
-void dump_stats();
-void GetArgs();
-void print_usage();
-void fill_entry();
-void fill_update_entry();
+#include "vlserver.h"
+
+void fill_listattributes_entry(struct VldbListByAttributes *, char **, int);
+void display_listattributes_entry(struct VldbListByAttributes *,int);
+void display_entry(struct vldbentry *, int);
+void display_entryN(struct nvldbentry *, int);
+void display_update_entry(struct VldbUpdateEntry *, int);
+void dump_stats(vldstats *, vital_vlheader *);
+void GetArgs(char *, char **, int *);
+void print_usage(void);
+void fill_entry(struct vldbentry *, char **, int);
+void fill_update_entry(struct VldbUpdateEntry *, char **, int);
 
 #define	VL_NUMBER_OPCODESX	34
 static char *opcode_names[VL_NUMBER_OPCODESX] = {
@@ -129,11 +103,10 @@ char confdir[AFSDIR_PATH_MAX];
 char *(args[50]);
 
 struct Vlent *
-GetVolume(vol, entry)
-     struct vldbentry *entry;
+GetVolume(int vol, struct vldbentry *entry)
 {
-    register int i;
-    register struct Vlent *vl;
+    int i;
+    struct Vlent *vl;
 
     if (!vol)
 	return NULL;
@@ -161,22 +134,21 @@ GetVolume(vol, entry)
 
 /* Almost identical's to pr_Initialize in vlserver/pruser.c */
 afs_int32
-vl_Initialize(int auth, char *confDir, int server, char *cellp)
+vl_Initialize(char *confDir, int secFlags, int server, char *cellp)
 {
-    return ugen_ClientInit(auth?0:1, confDir, cellp, 0,
-			  &cstruct, NULL, "vl_Initialize", rxkad_clear, 
-			  MAXSERVERS, AFSCONF_VLDBSERVICE, 50, server,
-			  htons(AFSCONF_VLDBPORT), USER_SERVICE_ID);
+    return ugen_ClientInitServer(confDir, cellp, secFlags, &cstruct,
+				 MAXSERVERS, AFSCONF_VLDBSERVICE, 90,
+				 server, htons(AFSCONF_VLDBPORT));
 }
 
 /* return host address in network byte order */
 afs_int32
 GetServer(char *aname)
 {
-    register struct hostent *th;
-    afs_int32 addr;
+    struct hostent *th;
+    afs_uint32 addr;
     int b1, b2, b3, b4;
-    register afs_int32 code;
+    afs_int32 code;
 
     code = sscanf(aname, "%d.%d.%d.%d", &b1, &b2, &b3, &b4);
     if (code == 4) {
@@ -191,26 +163,25 @@ GetServer(char *aname)
 }
 
 
-static
-handleit(as)
-     struct cmd_syndesc *as;
+static int
+handleit(struct cmd_syndesc *as, void *arock)
 {
-    register struct cmd_item *ti;
-    register afs_int32 code, server = 0, sawserver = 0;
+    struct cmd_item *ti;
+    afs_int32 code, server = 0, sawserver = 0;
     afs_int32 id, voltype;
     struct vldbentry entry;
-    char *cmd = 0, *cellp = 0;
+    char *cellp = 0;
     struct VldbUpdateEntry updateentry;
     struct VldbListByAttributes listbyattributes;
-    int noAuth = 1;		/* Default is authenticated connections */
+    int secFlags = AFSCONF_SECOPTS_FALLBACK_NULL;
 
-    if (ti = as->parms[0].items)	/* -cellpath <dir> */
+    if ((ti = as->parms[0].items))	/* -cellpath <dir> */
 	strcpy(confdir, ti->data);
     if (as->parms[1].items)	/* -server */
 	strcpy(confdir, AFSDIR_SERVER_ETC_DIRPATH);
     if (as->parms[2].items)	/* -noauth */
-	noAuth = 0;
-    if (ti = as->parms[3].items) {	/* -host */
+	secFlags |= AFSCONF_SECOPTS_NOAUTH;
+    if ((ti = as->parms[3].items)) {	/* -host */
 	server = GetServer(ti->data);
 	if (server == 0) {
 	    printf("server '%s' not found in host table\n", ti->data);
@@ -218,15 +189,15 @@ handleit(as)
 	}
 	sawserver = 1;
     }
-    if (!sawserver && noAuth && (!(ti = as->parms[4].items))) {
+    if (sawserver && !as->parms[2].items && (!(ti = as->parms[4].items))) {
 	printf
 	    ("Must also specify the -cell' option along with -host for authenticated conns\n");
 	exit(1);
     }
-    if (ti = as->parms[4].items) {	/* -cell */
+    if ((ti = as->parms[4].items)) {	/* -cell */
 	cellp = ti->data;
     }
-    if (code = vl_Initialize(noAuth, confdir, server, cellp)) {
+    if ((code = vl_Initialize(confdir, secFlags, server, cellp))) {
 	printf("Couldn't initialize vldb library (code=%d).\n", code);
 	exit(1);
     }
@@ -234,7 +205,7 @@ handleit(as)
     if (as->parms[5].items) {	/* -gstats */
 	vldstats stats;
 	vital_vlheader vital_header;
-	code = ubik_Call(VL_GetStats, cstruct, 0, &stats, &vital_header);
+	code = ubik_VL_GetStats(cstruct, 0, &stats, &vital_header);
 	if (!code)
 	    dump_stats(&stats, &vital_header);
 	exit(0);
@@ -252,20 +223,22 @@ handleit(as)
 	    exit(0);
 	} else {
 	    char *oper, *vname;
-	    register char **argp = args;
+	    char **argp = args;
+
 	    GetArgs(line, argp, &nargs);
 	    oper = &argp[0][0];
 	    ++argp, --nargs;
-	    if (!strcmp(oper, "cr")) {
+	    if (!*line) {
+	    } else if (!strcmp(oper, "cr")) {
 		fill_entry(&entry, argp, nargs);
 		display_entry(&entry, 0);
-		code = ubik_Call(VL_CreateEntry, cstruct, 0, &entry);
+		code = ubik_VL_CreateEntry(cstruct, 0, &entry);
 		printf("return code is %d\n", code);
 	    } else if (!strcmp(oper, "rm")) {
 		sscanf(&(*argp)[0], "%d", &id);
 		++argp, --nargs;
 		sscanf(&(*argp)[0], "%d", &voltype);
-		code = ubik_Call(VL_DeleteEntry, cstruct, 0, id, voltype);
+		code = ubik_VL_DeleteEntry(cstruct, 0, id, voltype);
 		printf("return code is %d\n", code);
 	    } else if (!strcmp(oper, "re")) {
 		sscanf(&(*argp)[0], "%d", &id);
@@ -277,7 +250,8 @@ handleit(as)
 		fill_entry(&entry, argp, nargs);
 		display_entry(&entry, 0);
 		code =
-		    ubik_Call(VL_ReplaceEntry, cstruct, 0, id, voltype,
+		    ubik_VL_ReplaceEntry(
+                    cstruct, 0, id, voltype,
 			      &entry, releasetype);
 		printf("return code is %d\n", code);
 	    } else if (!strcmp(oper, "up")) {
@@ -290,7 +264,7 @@ handleit(as)
 		fill_update_entry(&updateentry, argp, nargs);
 		display_update_entry(&updateentry, 0);
 		code =
-		    ubik_Call(VL_UpdateEntry, cstruct, 0, id, voltype,
+		    ubik_VL_UpdateEntry(cstruct, 0, id, voltype,
 			      &updateentry, releasetype);
 		printf("return code is %d\n", code);
 	    } else if (!strcmp(oper, "ls")) {
@@ -298,7 +272,7 @@ handleit(as)
 		for (index = 0; 1; index = next_index) {
 		    memset(&entry, 0, sizeof(entry));
 		    code =
-			ubik_Call(VL_ListEntry, cstruct, 0, index, &count,
+			ubik_VL_ListEntry(cstruct, 0, index, &count,
 				  &next_index, &entry);
 		    if (code) {
 			printf("VL_ListEntry returned code = %d\n", code);
@@ -312,8 +286,7 @@ handleit(as)
 		afs_int32 index, count, num = 0, num1 = 0, next_index;
 		struct Vlent *vl1;
 
-		VL = SVL =
-		    (struct Vlent *)malloc(ALLOCNT * sizeof(struct Vlent));
+		VL = SVL = malloc(ALLOCNT * sizeof(struct Vlent));
 		if (VL == NULL) {
 		    printf("Can't allocate memory...\n");
 		    exit(1);
@@ -322,7 +295,7 @@ handleit(as)
 		for (index = 0; 1; index = next_index) {
 		    memset(&entry, 0, sizeof(entry));
 		    code =
-			ubik_Call(VL_ListEntry, cstruct, 0, index, &count,
+			ubik_VL_ListEntry(cstruct, 0, index, &count,
 				  &next_index, &entry);
 		    if (code) {
 			printf("VL_ListEntry returned code = %d\n", code);
@@ -331,21 +304,21 @@ handleit(as)
 		    if (!next_index)
 			break;
 		    num++;
-		    if (vl1 = GetVolume(entry.volumeId[0], &entry)) {
+		    if ((vl1 = GetVolume(entry.volumeId[0], &entry))) {
 			num1++;
 			printf
 			    ("Duplicate entry is found for RW vol %u: [RW %u, RO %u, BA %u, name=%s]\n",
 			     entry.volumeId[0], vl1->rwid, vl1->roid,
 			     vl1->baid, vl1->name);
 		    }
-		    if (vl1 = GetVolume(entry.volumeId[1], &entry)) {
+		    if ((vl1 = GetVolume(entry.volumeId[1], &entry))) {
 			num1++;
 			printf
 			    ("Duplicate entry is found for RO vol %u: [RW %u, RO %u, BA %u, name=%s]\n",
 			     entry.volumeId[1], vl1->rwid, vl1->roid,
 			     vl1->baid, vl1->name);
 		    }
-		    if (vl1 = GetVolume(entry.volumeId[2], &entry)) {
+		    if ((vl1 = GetVolume(entry.volumeId[2], &entry))) {
 			num1++;
 			printf
 			    ("Duplicate entry is found for BA vol %u: [RW %u, RO %u, BA %u, name=%s]\n",
@@ -361,8 +334,7 @@ handleit(as)
 		    0, num31 = 0, num4 = 0, num41 = 0, next_index;
 		struct vldbentry tentry;
 
-		VL = SVL =
-		    (struct Vlent *)malloc(ALLOCNT * sizeof(struct Vlent));
+		VL = SVL = malloc(ALLOCNT * sizeof(struct Vlent));
 		if (VL == NULL) {
 		    printf("Can't allocate memory...\n");
 		    exit(1);
@@ -371,7 +343,7 @@ handleit(as)
 		for (index = 0; 1; index = next_index) {
 		    memset(&entry, 0, sizeof(entry));
 		    code =
-			ubik_Call(VL_ListEntry, cstruct, 0, index, &count,
+			ubik_VL_ListEntry(cstruct, 0, index, &count,
 				  &next_index, &entry);
 		    if (code) {
 			printf("VL_ListEntry returned code = %d\n", code);
@@ -381,7 +353,7 @@ handleit(as)
 			break;
 		    num++;
 		    code =
-			ubik_Call(VL_GetEntryByNameO, cstruct, 0, entry.name,
+			ubik_VL_GetEntryByNameO(cstruct, 0, entry.name,
 				  &tentry);
 		    if (code == VL_NOENT) {
 			num1++;
@@ -389,7 +361,7 @@ handleit(as)
 			       entry.name, entry.volumeId[RWVOL]);
 		    }
 		    code =
-			ubik_Call(VL_GetEntryByID, cstruct, 0,
+			ubik_VL_GetEntryByID(cstruct, 0,
 				  entry.volumeId[RWVOL], RWVOL, &tentry);
 		    if (code == VL_NOENT) {
 			num2++;
@@ -398,7 +370,7 @@ handleit(as)
 		    }
 		    if (entry.volumeId[BACKVOL]) {
 			code =
-			    ubik_Call(VL_GetEntryByID, cstruct, 0,
+			    ubik_VL_GetEntryByID(cstruct, 0,
 				      entry.volumeId[BACKVOL], BACKVOL,
 				      &tentry);
 			num31++;
@@ -410,7 +382,7 @@ handleit(as)
 		    }
 		    if (entry.volumeId[ROVOL]) {
 			code =
-			    ubik_Call(VL_GetEntryByID, cstruct, 0,
+			    ubik_VL_GetEntryByID(cstruct, 0,
 				      entry.volumeId[ROVOL], ROVOL, &tentry);
 			num41++;
 			if (code == VL_NOENT) {
@@ -434,8 +406,7 @@ handleit(as)
 		    0;
 		struct vldbentry tentry;
 
-		VL = SVL =
-		    (struct Vlent *)malloc(ALLOCNT * sizeof(struct Vlent));
+		VL = SVL = malloc(ALLOCNT * sizeof(struct Vlent));
 		if (VL == NULL) {
 		    printf("Can't allocate memory...\n");
 		    exit(1);
@@ -444,11 +415,9 @@ handleit(as)
 		    ("Volumes not found in main hash tables in vldb will be fixed...\n");
 		memset(&updateentry, 0, sizeof(updateentry));
 		for (index = 0; 1; index = next_index) {
-		    /* FIXME: n2 is never changed for some reason */
-		    int n1 = 0, n2 = 0, n3 = 0, n4 = 0;
 		    memset(&entry, 0, sizeof(entry));
 		    code =
-			ubik_Call(VL_ListEntry, cstruct, 0, index, &count,
+			ubik_VL_ListEntry(cstruct, 0, index, &count,
 				  &next_index, &entry);
 		    if (code) {
 			printf("VL_ListEntry returned code = %d\n", code);
@@ -458,16 +427,15 @@ handleit(as)
 			break;
 		    num++;
 		    code =
-			ubik_Call(VL_GetEntryByNameO, cstruct, 0, entry.name,
+			ubik_VL_GetEntryByNameO(cstruct, 0, entry.name,
 				  &tentry);
 		    if (code == VL_NOENT) {
 			num1++;
-			n1 = 1;
 			updateentry.Mask = VLUPDATE_VOLNAMEHASH;
 			printf("\tVolume %s %d (not in namehash)\n",
 			       entry.name, entry.volumeId[RWVOL]);
 			code =
-			    ubik_Call(VL_UpdateEntry, cstruct, 0,
+			    ubik_VL_UpdateEntry(cstruct, 0,
 				      entry.volumeId[RWVOL], -1, &updateentry,
 				      0);
 			if (code) {
@@ -477,7 +445,7 @@ handleit(as)
 			}
 		    }
 		    code =
-			ubik_Call(VL_GetEntryByID, cstruct, 0,
+			ubik_VL_GetEntryByID(cstruct, 0,
 				  entry.volumeId[RWVOL], RWVOL, &tentry);
 		    if (code == VL_NOENT) {
 			num1++;
@@ -487,7 +455,7 @@ handleit(as)
 			printf("\tVolume %s %d (not in rw id hash)\n",
 			       entry.name, entry.volumeId[RWVOL]);
 			code =
-			    ubik_Call(VL_UpdateEntryByName, cstruct, 0,
+			    ubik_VL_UpdateEntryByName(cstruct, 0,
 				      entry.name, &updateentry, 0);
 			if (code) {
 			    printf("\tFailed to update volume %s (err=%d)\n",
@@ -496,20 +464,19 @@ handleit(as)
 			}
 			x++;
 		    }
-		    if (entry.volumeId[BACKVOL] && !n2) {
+		    if (entry.volumeId[BACKVOL]) {
 			code =
-			    ubik_Call(VL_GetEntryByID, cstruct, 0,
+			    ubik_VL_GetEntryByID(cstruct, 0,
 				      entry.volumeId[BACKVOL], BACKVOL,
 				      &tentry);
 			if (code == VL_NOENT) {
-			    n3 = 1;
 			    num1++;
 			    updateentry.Mask = VLUPDATE_BACKUPID;
 			    updateentry.BackupId = entry.volumeId[BACKVOL];
 			    printf("\tVolume %s %d (not in backup id hash)\n",
 				   entry.name, entry.volumeId[BACKVOL]);
 			    code =
-				ubik_Call(VL_UpdateEntry, cstruct, 0,
+				ubik_VL_UpdateEntry(cstruct, 0,
 					  entry.volumeId[RWVOL], -1,
 					  &updateentry, 0);
 			    if (code) {
@@ -520,19 +487,18 @@ handleit(as)
 			    }
 			}
 		    }
-		    if (entry.volumeId[ROVOL && !n2]) {
+		    if (entry.volumeId[ROVOL]) {
 			code =
-			    ubik_Call(VL_GetEntryByID, cstruct, 0,
+			    ubik_VL_GetEntryByID(cstruct, 0,
 				      entry.volumeId[ROVOL], ROVOL, &tentry);
 			if (code == VL_NOENT) {
-			    n4 = 1;
 			    num1++;
 			    updateentry.Mask = VLUPDATE_READONLYID;
 			    updateentry.ReadOnlyId = entry.volumeId[ROVOL];
 			    printf("\tVolume %s %d (not in RO id hash)\n",
 				   entry.name, entry.volumeId[ROVOL]);
 			    code =
-				ubik_Call(VL_UpdateEntry, cstruct, 0,
+				ubik_VL_UpdateEntry(cstruct, 0,
 					  entry.volumeId[RWVOL], -1,
 					  &updateentry, 0);
 			    if (code) {
@@ -556,7 +522,7 @@ handleit(as)
 		fill_listattributes_entry(&listbyattributes, argp, nargs);
 		display_listattributes_entry(&listbyattributes, 0);
 		code =
-		    ubik_Call(VL_ListAttributes, cstruct, 0,
+		    ubik_VL_ListAttributes(cstruct, 0,
 			      &listbyattributes, &nentries, &entries);
 		if (code) {
 		    printf("VL_ListAttributes returned code = %d\n", code);
@@ -566,7 +532,7 @@ handleit(as)
 		for (i = 0; i < nentries; i++, entry++)
 		    display_entry(entry, 0);
 		if (entries.bulkentries_val)
-		    free((char *)entries.bulkentries_val);
+		    free(entries.bulkentries_val);
 	    } else if (!strcmp(oper, "lan2")) {
 		int nentries, i, si, nsi, t = 0;
 		nbulkentries entries;
@@ -589,7 +555,7 @@ handleit(as)
 		    nentries = 0;
 		    memset(&entries, 0, sizeof(entries));
 		    code =
-			ubik_Call(VL_ListAttributesN2, cstruct, 0,
+			ubik_VL_ListAttributesN2(cstruct, 0,
 				  &listbyattributes, name, si, &nentries,
 				  &entries, &nsi);
 		    if (code) {
@@ -603,7 +569,7 @@ handleit(as)
 		    for (i = 0; i < nentries; i++, entry++)
 			display_entryN(entry, 0);
 		    if (entries.nbulkentries_val)
-			free((char *)entries.nbulkentries_val);
+			free(entries.nbulkentries_val);
 		}
 		printf("--- %d volumes ---\n", t);
 	    } else if (!strcmp(oper, "ln")) {
@@ -615,7 +581,7 @@ handleit(as)
 		display_listattributes_entry(&listbyattributes, 0);
 		memset(&linkedvldbs, 0, sizeof(vldb_list));
 		code =
-		    ubik_Call(VL_LinkedList, cstruct, 0, &listbyattributes,
+		    ubik_VL_LinkedList(cstruct, 0, &listbyattributes,
 			      &netries, &linkedvldbs);
 		if (code) {
 		    printf("VL_LinkedList returned code = %d\n", code);
@@ -625,8 +591,8 @@ handleit(as)
 		       (netries == 1 ? "y" : "ies"));
 		for (vllist = linkedvldbs.node; vllist; vllist = vllist1) {
 		    vllist1 = vllist->next_vldb;
-		    display_entry(&vllist->VldbEntry, 0);
-		    free((char *)vllist);
+		    display_entry((struct vldbentry *) &vllist->VldbEntry, 0);
+		    free(vllist);
 		}
 	    } else if (!strcmp(oper, "lnn")) {
 		int netries;
@@ -637,7 +603,7 @@ handleit(as)
 		display_listattributes_entry(&listbyattributes, 0);
 		memset(&linkedvldbs, 0, sizeof(vldb_list));
 		code =
-		    ubik_Call(VL_LinkedListN, cstruct, 0, &listbyattributes,
+		    ubik_VL_LinkedListN(cstruct, 0, &listbyattributes,
 			      &netries, &linkedvldbs);
 		if (code) {
 		    printf("VL_LinkedList returned code = %d\n", code);
@@ -647,15 +613,15 @@ handleit(as)
 		       (netries == 1 ? "y" : "ies"));
 		for (vllist = linkedvldbs.node; vllist; vllist = vllist1) {
 		    vllist1 = vllist->next_vldb;
-		    display_entry(&vllist->VldbEntry, 0);
-		    free((char *)vllist);
+		    display_entry((struct vldbentry *)&vllist->VldbEntry, 0);
+		    free(vllist);
 		}
 	    } else if (!strcmp(oper, "di")) {
 		sscanf(&(*argp)[0], "%d", &id);
 		++argp, --nargs;
 		sscanf(&(*argp)[0], "%d", &voltype);
 		code =
-		    ubik_Call(VL_GetEntryByID, cstruct, 0, id, voltype,
+		    ubik_VL_GetEntryByID(cstruct, 0, id, voltype,
 			      &entry);
 		display_entry(&entry, code);
 		printf("return code is %d.\n", code);
@@ -664,7 +630,7 @@ handleit(as)
 		++argp, --nargs;
 		sscanf(&(*argp)[0], "%d", &voltype);
 		code =
-		    ubik_Call(VL_GetEntryByID, cstruct, 0, id, voltype,
+		    ubik_VL_GetEntryByID(cstruct, 0, id, voltype,
 			      &entry);
 		display_entry(&entry, code);
 		memset(&updateentry, 0, sizeof(updateentry));
@@ -672,7 +638,7 @@ handleit(as)
 		printf("\tRehashing namehash table for %s (%d)\n", entry.name,
 		       entry.volumeId[RWVOL]);
 		code =
-		    ubik_Call(VL_UpdateEntry, cstruct, 0,
+		    ubik_VL_UpdateEntry(cstruct, 0,
 			      entry.volumeId[RWVOL], -1, &updateentry, 0);
 		if (code) {
 		    printf("\tFailed to update volume %s (err=%d)\n",
@@ -694,7 +660,7 @@ handleit(as)
 		for (index = 0; 1; index = next_index) {
 		    memset(&entry, 0, sizeof(entry));
 		    code =
-			ubik_Call(VL_ListEntry, cstruct, 0, index, &count,
+			ubik_VL_ListEntry(cstruct, 0, index, &count,
 				  &next_index, &entry);
 		    if (code) {
 			printf("VL_ListEntry returned code = %d\n", code);
@@ -715,7 +681,7 @@ handleit(as)
 				("\tUndeleting vldb entry for vol %d (%s)\n",
 				 id, entry.name);
 			    code =
-				ubik_Call(VL_UpdateEntry, cstruct, 0, id, -1,
+				ubik_VL_UpdateEntry(cstruct, 0, id, -1,
 					  &updateentry, 0);
 			    if (code) {
 				printf
@@ -731,22 +697,22 @@ handleit(as)
 	    } else if (!strcmp(oper, "dn")) {
 		vname = &argp[0][0];
 		code =
-		    ubik_Call(VL_GetEntryByNameO, cstruct, 0, vname, &entry);
+		    ubik_VL_GetEntryByNameO(cstruct, 0, vname, &entry);
 		display_entry(&entry, code);
 		printf("return code is %d.\n", code);
 	    } else if (!strcmp(oper, "nv")) {
-		int newvolid;
+		unsigned int newvolid;
 		sscanf(&(*argp)[0], "%d", &id);
 		code =
-		    ubik_Call(VL_GetNewVolumeId, cstruct, 0, id, &newvolid);
+		    ubik_VL_GetNewVolumeId(cstruct, 0, id, &newvolid);
 		if (!code)
-		    printf("Current Max volid is (in hex):%X\n", newvolid);
+		    printf("Current Max volid is (in hex):0x%x\n", newvolid);
 		printf("return code is %d\n", code);
 	    } else if (!strcmp(oper, "gs")) {
 		vldstats stats;
 		vital_vlheader vital_header;
 		code =
-		    ubik_Call(VL_GetStats, cstruct, 0, &stats, &vital_header);
+		    ubik_VL_GetStats(cstruct, 0, &stats, &vital_header);
 		if (!code)
 		    dump_stats(&stats, &vital_header);
 		printf("return code is %d.\n", code);
@@ -758,7 +724,7 @@ handleit(as)
 
 		addrs.bulkaddrs_val = 0;
 		addrs.bulkaddrs_len = 0;
-		code = ubik_Call(VL_GetAddrs, cstruct, 0, 0 /*Handle */ ,
+		code = ubik_VL_GetAddrs(cstruct, 0, 0 /*Handle */ ,
 				 0 /*spare2 */ , &vlcb,
 				 &nentries, &addrs);
 		if (code) {
@@ -774,7 +740,7 @@ handleit(as)
 			printf("[0x%x %u] %s\n", *addrp, *addrp,
 			       hostutil_GetNameByINet(ntohl(*addrp)));
 		}
-		free((char *)addrs.bulkaddrs_val);
+		free(addrs.bulkaddrs_val);
 	    } else if (!strcmp(oper, "gau")) {
 		int nentries, i, j;
 		afs_uint32 *addrp;
@@ -783,7 +749,7 @@ handleit(as)
 
 		addrs.bulkaddrs_val = 0;
 		addrs.bulkaddrs_len = 0;
-		code = ubik_Call(VL_GetAddrs, cstruct, 0, 0 /*Handle */ ,
+		code = ubik_VL_GetAddrs(cstruct, 0, 0 /*Handle */ ,
 				 0 /*spare2 */ , &vlcb,
 				 &nentries, &addrs);
 		if (code) {
@@ -808,7 +774,7 @@ handleit(as)
 			attrs.index = *addrp & 0x00ffffff;
 
 			code =
-			    ubik_Call(VL_GetAddrsU, cstruct, 0, &attrs, &uuid,
+			    ubik_VL_GetAddrsU(cstruct, 0, &attrs, &uuid,
 				      &unique, &mhnentries, &mhaddrs);
 			if (code) {
 			    printf("VL_GetAddrsU returned code = %d\n", code);
@@ -830,15 +796,15 @@ handleit(as)
 				   hostutil_GetNameByINet(mhaddrp[j]));
 			}
 			if (mhaddrs.bulkaddrs_val)
-			    free((char *)mhaddrs.bulkaddrs_val);
+			    free(mhaddrs.bulkaddrs_val);
 		    } else {
 			printf("[0x%x %u] %s\n", *addrp, *addrp,
 			       hostutil_GetNameByINet(ntohl(*addrp)));
 		    }
 		}
-		free((char *)addrs.bulkaddrs_val);
+		free(addrs.bulkaddrs_val);
 	    } else if (!strcmp(oper, "mhc")) {
-		afs_int32 serveraddrs[MAXSERVERID + 1][VL_MAXIPADDRS_PERMH];
+		afs_uint32 serveraddrs[MAXSERVERID + 1][VL_MAXIPADDRS_PERMH];
 		afs_int32 serveraddrtype[MAXSERVERID + 1];
 		int nentries1, nentries2, i, j, x, y, unique, found;
 		afs_uint32 *addrp1, *addrp2;
@@ -858,7 +824,7 @@ handleit(as)
 		addrs1.bulkaddrs_val = 0;
 		addrs1.bulkaddrs_len = 0;
 		code =
-		    ubik_Call(VL_GetAddrs, cstruct, 0, 0, 0, &vlcb,
+		    ubik_VL_GetAddrs(cstruct, 0, 0, 0, &vlcb,
 			      &nentries1, &addrs1);
 		if (code) {
 		    printf("VL_GetAddrs returned code = %d\n", code);
@@ -880,7 +846,7 @@ handleit(as)
 			attrs.Mask = VLADDR_INDEX;
 			attrs.index = (base * VL_MHSRV_PERBLK) + index;
 			code =
-			    ubik_Call(VL_GetAddrsU, cstruct, 0, &attrs, &uuid,
+			    ubik_VL_GetAddrsU(cstruct, 0, &attrs, &uuid,
 				      &unique, &nentries2, &addrs2);
 			if (code) {
 			    printf("VL_GetAddrsU returned code = %d\n", code);
@@ -891,7 +857,7 @@ handleit(as)
 			for (j = 0; j < nentries2; j++) {
 			    serveraddrs[i][j] = ntohl(addrp2[j]);
 			}
-			free((char *)addrs2.bulkaddrs_val);
+			free(addrs2.bulkaddrs_val);
 		    }
 
 		    if (nargs) {
@@ -906,7 +872,7 @@ handleit(as)
 			}
 		    }
 		}
-		free((char *)addrs1.bulkaddrs_val);
+		free(addrs1.bulkaddrs_val);
 
 		/* Look for any duplicates */
 		for (i = 0; i < MAXSERVERID + 1; i++) {
@@ -955,7 +921,7 @@ handleit(as)
 		    printf("Illegal # entries = %d\n", nargs);
 		    continue;
 		}
-		addrp = (afs_uint32 *) malloc(20 * 4);
+		addrp = malloc(20 * 4);
 		addrs.bulkaddrs_val = addrp;
 		addrs.bulkaddrs_len = nargs;
 		while (nargs > 0) {
@@ -964,14 +930,13 @@ handleit(as)
 		    ++argp, --nargs;
 		}
 		code =
-		    ubik_Call(VL_RegisterAddrs, cstruct, 0, &uuid,
+		    ubik_VL_RegisterAddrs(cstruct, 0, &uuid,
 			      0 /*spare */ , &addrs);
 		if (code) {
 		    printf("VL_RegisterAddrs returned code = %d\n", code);
 		    continue;
 		}
 	    } else if (!strcmp(oper, "ca")) {
-		extern struct hostent *hostutil_GetHostByName();
 		struct hostent *h1, *h2;
 		afs_uint32 a1, a2;
 
@@ -996,7 +961,7 @@ handleit(as)
 
 		printf("changing 0x%x to 0x%x\n", ntohl(a1), ntohl(a2));
 		code =
-		    ubik_Call(VL_ChangeAddr, cstruct, 0, ntohl(a1),
+		    ubik_VL_ChangeAddr(cstruct, 0, ntohl(a1),
 			      ntohl(a2));
 		if (code) {
 		    printf("VL_ChangeAddr returned code = %d\n", code);
@@ -1010,17 +975,18 @@ handleit(as)
 		++argp, --nargs;
 		sscanf(&(*argp)[0], "%d", &a2);
 		printf(" to %d (0x%x)\n", a2, a2);
-		code = ubik_Call(VL_ChangeAddr, cstruct, 0, a1, a2);
+		code = ubik_VL_ChangeAddr(cstruct, 0, a1, a2);
 		if (code) {
 		    printf("VL_ChangeAddr returned code = %d\n", code);
 		    continue;
 		}
-	    } else if ((!strcmp(oper, "?")) || !strcmp(oper, "h"))
+	    } else if ((!strcmp(oper, "?")) || !strcmp(oper, "h") || !strcmp(oper, "help"))
 		print_usage();
 	    else if ((!strcmp(oper, "q")) || !strcmp(oper, "quit"))
 		exit(0);
 	    else {
-		printf("Unknown oper!\n");
+		printf("Unknown oper (%s)!\n", oper);
+		print_usage();
 	    }
 	}
     }
@@ -1029,15 +995,14 @@ handleit(as)
 
 #include "AFS_component_version_number.c"
 
-main(argc, argv)
-     int argc;
-     char **argv;
+int
+main(int argc, char **argv)
 {
-    register struct cmd_syndesc *ts;
+    struct cmd_syndesc *ts;
     afs_int32 code;
 
     strcpy(confdir, AFSDIR_CLIENT_ETC_DIRPATH);
-    ts = cmd_CreateSyntax("initcmd", handleit, 0, "initialize the program");
+    ts = cmd_CreateSyntax("initcmd", handleit, NULL, "initialize the program");
     cmd_AddParm(ts, "-cellpath", CMD_LIST, CMD_OPTIONAL,
 		"Cell configuration directory");
     cmd_AddParm(ts, "-server", CMD_LIST, CMD_OPTIONAL,
@@ -1056,10 +1021,7 @@ main(argc, argv)
 
 
 void
-fill_entry(entry, argp, nargs)
-     struct vldbentry *entry;
-     char **argp;
-     int nargs;
+fill_entry(struct vldbentry *entry, char **argp, int nargs)
 {
     char *name;
     int i;
@@ -1084,19 +1046,16 @@ fill_entry(entry, argp, nargs)
     }
     for (i = 0; i < MAXTYPES; i++) {
 	++argp, --nargs;
-	sscanf(&(*argp)[0], "%d", &entry->volumeId[i]);
+	sscanf(&(*argp)[0], "%u", &entry->volumeId[i]);
     }
     ++argp, --nargs;
     sscanf(&(*argp)[0], "%d", &entry->flags);
     ++argp, --nargs;
-    sscanf(&(*argp)[0], "%d", &entry->cloneId);
+    sscanf(&(*argp)[0], "%u", &entry->cloneId);
 }
 
 void
-fill_update_entry(entry, argp, nargs)
-     struct VldbUpdateEntry *entry;
-     char **argp;
-     int nargs;
+fill_update_entry(struct VldbUpdateEntry *entry, char **argp, int nargs)
 {
     int i, Mask;
     char *name;
@@ -1112,15 +1071,15 @@ fill_update_entry(entry, argp, nargs)
     if (entry->flags != -1)
 	entry->Mask |= VLUPDATE_FLAGS;
     ++argp, --nargs;
-    sscanf(&(*argp)[0], "%d", &entry->cloneId);
+    sscanf(&(*argp)[0], "%u", &entry->cloneId);
     if (entry->flags != -1)
 	entry->Mask |= VLUPDATE_CLONEID;
     ++argp, --nargs;
-    sscanf(&(*argp)[0], "%d", &entry->ReadOnlyId);
+    sscanf(&(*argp)[0], "%u", &entry->ReadOnlyId);
     if (entry->ReadOnlyId != -1)
 	entry->Mask |= VLUPDATE_READONLYID;
     ++argp, --nargs;
-    sscanf(&(*argp)[0], "%d", &entry->BackupId);
+    sscanf(&(*argp)[0], "%u", &entry->BackupId);
     if (entry->BackupId != -1)
 	entry->Mask |= VLUPDATE_BACKUPID;
     ++argp, --nargs;
@@ -1161,10 +1120,8 @@ fill_update_entry(entry, argp, nargs)
 }
 
 void
-fill_listattributes_entry(entry, argp, nargs)
-     struct VldbListByAttributes *entry;
-     char **argp;
-     int nargs;
+fill_listattributes_entry(struct VldbListByAttributes *entry, char **argp,
+			  int nargs)
 {
     entry->Mask = 0;
 
@@ -1198,9 +1155,7 @@ fill_listattributes_entry(entry, argp, nargs)
 }
 
 void
-display_listattributes_entry(entry, error)
-     struct VldbListByAttributes *entry;
-     int error;
+display_listattributes_entry(struct VldbListByAttributes *entry, int error)
 {
     if (error)
 	return;
@@ -1221,9 +1176,7 @@ display_listattributes_entry(entry, error)
 #define	volumetype_string(type) (type == RWVOL? "read/write":type == ROVOL? "readonly":type == BACKVOL? "backup":"unknown")
 
 void
-display_entry(entry, error)
-     struct vldbentry *entry;
-     int error;
+display_entry(struct vldbentry *entry, int error)
 {
     int i;
 
@@ -1242,9 +1195,7 @@ display_entry(entry, error)
 }
 
 void
-display_entryN(entry, error)
-     struct nvldbentry *entry;
-     int error;
+display_entryN(struct nvldbentry *entry, int error)
 {
     int i, et, ei;
 
@@ -1270,9 +1221,7 @@ display_entryN(entry, error)
 }
 
 void
-display_update_entry(entry, error)
-     struct VldbUpdateEntry *entry;
-     int error;
+display_update_entry(struct VldbUpdateEntry *entry, int error)
 {
     int i;
 
@@ -1286,19 +1235,19 @@ display_update_entry(entry, error)
     if (entry->Mask & VLUPDATE_CLONEID)
 	printf("\tNew CloneId: %X\n", entry->cloneId);
     if (entry->Mask & VLUPDATE_READONLYID)
-	printf("\tNew RO id: %D\n", entry->ReadOnlyId);
+	printf("\tNew RO id: %d\n", entry->ReadOnlyId);
     if (entry->Mask & VLUPDATE_BACKUPID)
-	printf("\tNew BACKUP id: %D\n", entry->BackupId);
+	printf("\tNew BACKUP id: %d\n", entry->BackupId);
     if (entry->Mask & VLUPDATE_REPSITES) {
 	printf("\tRepsites info:\n");
 	printf("\tFlag\tTServer\tTPart\tNServer\tNPart\tNFlag\n");
 	for (i = 0; i < entry->nModifiedRepsites; i++) {
-	    printf("\t%4x\t%7U\t%5d", entry->RepsitesMask[i],
+	    printf("\t%4x\t%7u\t%5d", entry->RepsitesMask[i],
 		   entry->RepsitesTargetServer[i],
 		   entry->RepsitesTargetPart[i]);
 	    if ((entry->RepsitesMask[i] & VLUPDATE_REPS_ADD)
 		|| (entry->RepsitesMask[i] & VLUPDATE_REPS_MODSERV))
-		printf("\t%7U", entry->RepsitesNewServer[i]);
+		printf("\t%7u", entry->RepsitesNewServer[i]);
 	    else
 		printf("\t-------");
 	    if ((entry->RepsitesMask[i] & VLUPDATE_REPS_ADD)
@@ -1316,16 +1265,15 @@ display_update_entry(entry, error)
 }
 
 void
-dump_stats(stats, vital_header)
-     vldstats *stats;
-     vital_vlheader *vital_header;
+dump_stats(vldstats *stats, vital_vlheader *vital_header)
 {
     int i;
     char strg[30];
     time_t start_time = stats->start_time;
+    struct tm tm;
 
-    strncpy(strg, ctime(&start_time), sizeof(strg));
-    strg[strlen(strg) - 1] = 0;
+    strftime(strg, sizeof(strg), "%a %b %d %H:%M:%S %Y",
+	     localtime_r(&start_time, &tm));
     printf("Dynamic statistics stats (starting time: %s):\n", strg);
     printf("OpcodeName\t# Requests\t# Aborts\n");
     for (i = 0; i < VL_NUMBER_OPCODESX; i++)
@@ -1342,28 +1290,25 @@ dump_stats(stats, vital_header)
 }
 
 void
-GetArgs(line, args, nargs)
-     register char *line;
-     register char **args;
-     register int *nargs;
+GetArgs(char *line, char **args, int *nargs)
 {
     *nargs = 0;
     while (*line) {
-	register char *last = line;
-	while (*line == ' ')
+	char *last = line;
+	while (isspace(*line))
 	    line++;
-	if (*last == ' ')
+	if (isspace(*last))
 	    *last = 0;
 	if (!*line)
 	    break;
 	*args++ = line, (*nargs)++;
-	while (*line && *line != ' ')
+	while (*line && !isspace(*line))
 	    line++;
     }
 }
 
 void
-print_usage()
+print_usage(void)
 {
     printf("Valid Commands:\n");
 
