@@ -37,8 +37,13 @@
 
 #include <roken.h>
 
-#include <rx/xdr.h>
+#include <afs/opr.h>
+#ifdef AFS_PTHREAD_ENV
+# include <opr/lock.h>
+#endif
+
 #include <afs/afsint.h>
+#include <rx/rx_queue.h>
 #include <afs/errors.h>
 #include <afs/afssyscalls.h>
 
@@ -69,8 +74,8 @@ static SYNC_client_state fssync_state =
 #ifdef AFS_PTHREAD_ENV
 static pthread_mutex_t vol_fsync_mutex;
 static volatile int vol_fsync_mutex_init = 0;
-#define VFSYNC_LOCK MUTEX_ENTER(&vol_fsync_mutex)
-#define VFSYNC_UNLOCK MUTEX_EXIT(&vol_fsync_mutex)
+#define VFSYNC_LOCK opr_mutex_enter(&vol_fsync_mutex)
+#define VFSYNC_UNLOCK opr_mutex_exit(&vol_fsync_mutex)
 #else
 #define VFSYNC_LOCK
 #define VFSYNC_UNLOCK
@@ -82,7 +87,7 @@ FSYNC_clientInit(void)
 #ifdef AFS_PTHREAD_ENV
     /* this is safe since it gets called with VOL_LOCK held, or before we go multithreaded */
     if (!vol_fsync_mutex_init) {
-	MUTEX_INIT(&vol_fsync_mutex, "vol fsync", MUTEX_DEFAULT, 0);
+	opr_mutex_init(&vol_fsync_mutex);
 	vol_fsync_mutex_init = 1;
     }
 #endif
@@ -117,7 +122,7 @@ FSYNC_askfs(SYNC_command * com, SYNC_response * res)
 	break;
     case SYNC_COM_ERROR:
     case SYNC_BAD_COMMAND:
-	Log("FSYNC_askfs: fatal FSSYNC protocol error; volume management functionality disabled until next fileserver restart\n");
+	Log("FSYNC_askfs: internal FSSYNC protocol error %d\n", code);
 	break;
     case SYNC_DENIED:
 	Log("FSYNC_askfs: FSSYNC request denied for reason=%d\n", res->hdr.reason);
