@@ -29,7 +29,11 @@
 #include <direct.h>
 #endif
 
-#include <afs/afs_assert.h>
+#include <afs/opr.h>
+#include <rx/rx_queue.h>
+#ifdef AFS_PTHREAD_ENV
+# include <opr/lock.h>
+#endif
 #include <lock.h>
 #include <afs/afsutil.h>
 #include <lwp.h>
@@ -78,7 +82,10 @@ namei_iread(IHandle_t * h, afs_foff_t offset, char *buf, afs_fsize_t size)
 	return -1;
 
     nBytes = FDH_PREAD(fdP, buf, size, offset);
-    FDH_CLOSE(fdP);
+    if (nBytes < 0)
+	FDH_REALLYCLOSE(fdP);
+    else
+	FDH_CLOSE(fdP);
     return nBytes;
 }
 
@@ -93,7 +100,10 @@ namei_iwrite(IHandle_t * h, afs_foff_t offset, char *buf, afs_fsize_t size)
 	return -1;
 
     nBytes = FDH_PWRITE(fdP, buf, size, offset);
-    FDH_CLOSE(fdP);
+    if (nBytes < 0)
+	FDH_REALLYCLOSE(fdP);
+    else
+	FDH_CLOSE(fdP);
     return nBytes;
 }
 
@@ -443,7 +453,7 @@ delTree(char *root, char *tree, int *errp)
 	    cp = tree + strlen(tree);	/* move cp to the end of string tree */
 
 	/* now delete all entries in this dir */
-	if ((ds = opendir(root)) != (DIR *) NULL) {
+	if ((ds = opendir(root)) != NULL) {
 	    errno = 0;
 	    while ((dirp = readdir(ds))) {
 		/* ignore . and .. */
@@ -613,26 +623,76 @@ SetOGM(FD_t fd, int parm, int tag)
 }
 
 static int
-CheckOGM(namei_t *name, FdHandle_t *fdP, int p1)
+SetWinOGM(FD_t fd, int p1, int p2)
 {
-    WIN32_FIND_DATA info;
-    HANDLE dirH;
+    BOOL code;
+    FILETIME ftime;
 
-    dirH =
-	FindFirstFileEx(name->n_path, FindExInfoStandard, &info,
-			FindExSearchNameMatch, NULL,
-			FIND_FIRST_EX_CASE_SENSITIVE);
+    ftime.dwHighDateTime = p1;
+    ftime.dwLowDateTime = p2;
 
-    if (!dirH)
-	return -1;          /* Can't get info, leave alone */
-
-    FindClose(dirH);
-
-    if (info.ftCreationTime.dwHighDateTime != (unsigned int)p1)
+    code = SetFileTime(fd, &ftime, NULL /*access*/, NULL /*write*/);
+    if (!code)
 	return -1;
+    return 0;
+}
+
+static int
+GetWinOGM(FD_t fd, int *p1, int *p2)
+{
+    BOOL code;
+    FILETIME ftime;
+
+    code = GetFileTime(fd, &ftime, NULL /*access*/, NULL /*write*/);
+    if (!code)
+	return -1;
+
+    *p1 = ftime.dwHighDateTime;
+    *p2 = ftime.dwLowDateTime;
 
     return 0;
 }
+
+static int
+CheckOGM(FdHandle_t *fdP, int p1)
+{
+    int ogm_p1, ogm_p2;
+
+    if (GetWinOGM(fdP->fd_fd, &ogm_p1, &ogm_p2)) {
+	return -1;
+    }
+
+    if (ogm_p1 != p1) {
+	return -1;
+    }
+
+    return 0;
+}
+
+static int
+FixSpecialOGM(FdHandle_t *fdP, int check)
+{
+    Inode ino = fdP->fd_ih->ih_ino;
+    VnodeId vno = NAMEI_VNODESPECIAL, ogm_vno;
+    int ogm_volid;
+
+    if (GetWinOGM(fdP->fd_fd, &ogm_volid, &ogm_vno)) {
+	return -1;
+    }
+
+    /* the only thing we can check is the vnode number; for the volid we have
+     * nothing else to compare against */
+    if (vno != ogm_vno) {
+	if (check) {
+	    return -1;
+	}
+	if (SetWinOGM(fdP->fd_fd, ogm_volid, vno)) {
+	    return -1;
+	}
+    }
+    return 0;
+}
+
 #else /* AFS_NT40_ENV */
 /* SetOGM - set owner group and mode bits from parm and tag */
 static int
@@ -667,21 +727,89 @@ GetOGMFromStat(struct afs_stat_st *status, int *parm, int *tag)
 }
 
 static int
-CheckOGM(namei_t *name, FdHandle_t *fdP, int p1)
+GetOGM(FdHandle_t *fdP, int *parm, int *tag)
 {
     struct afs_stat_st status;
-    int parm, tag;
     if (afs_fstat(fdP->fd_fd, &status) < 0)
 	return -1;
+    GetOGMFromStat(&status, parm, tag);
+    return 0;
+}
 
-    GetOGMFromStat(&status, &parm, &tag);
+static int
+CheckOGM(FdHandle_t *fdP, int p1)
+{
+    int parm, tag;
+
+    if (GetOGM(fdP, &parm, &tag) < 0)
+	return -1;
     if (parm != p1)
 	return -1;
 
     return 0;
 }
+
+static int
+FixSpecialOGM(FdHandle_t *fdP, int check)
+{
+    int inode_volid, ogm_volid;
+    int inode_type, ogm_type;
+    Inode ino = fdP->fd_ih->ih_ino;
+
+    inode_volid = ((ino >> NAMEI_UNIQSHIFT) & NAMEI_UNIQMASK);
+    inode_type = (int)((ino >> NAMEI_TAGSHIFT) & NAMEI_TAGMASK);
+
+    if (GetOGM(fdP, &ogm_volid, &ogm_type) < 0) {
+	Log("Error retrieving OGM info\n");
+	return -1;
+    }
+
+    if (inode_volid != ogm_volid || inode_type != ogm_type) {
+	Log("%sIncorrect OGM data (ino: vol %u type %d) (ogm: vol %u type %d)\n",
+	    check?"":"Fixing ", inode_volid, inode_type, ogm_volid, ogm_type);
+
+	if (check) {
+	    return -1;
+	}
+
+	if (SetOGM(fdP->fd_fd, inode_volid, inode_type) < 0) {
+	    Log("Error setting OGM data\n");
+	    return -1;
+	}
+    }
+    return 0;
+}
+
 #endif /* !AFS_NT40_ENV */
 
+/**
+ * Check/fix the OGM data for an inode
+ *
+ * @param[in] fdP   Open file handle for the inode to check
+ * @param[in] check 1 to just check the OGM data, and return an error if it
+ *                  is incorrect. 0 to fix the OGM data if it is incorrect.
+ *
+ * @pre fdP must be for a special inode
+ *
+ * @return status
+ *  @retval 0 success
+ *  @retval -1 error
+ */
+int
+namei_FixSpecialOGM(FdHandle_t *fdP, int check)
+{
+    int vnode;
+    Inode ino = fdP->fd_ih->ih_ino;
+
+    vnode = (int)(ino & NAMEI_VNODEMASK);
+    if (vnode != NAMEI_VNODESPECIAL) {
+	Log("FixSpecialOGM: non-special vnode %u\n", vnode);
+	return -1;
+    }
+
+    return FixSpecialOGM(fdP, check);
+}
+
 int big_vno = 0;		/* Just in case we ever do 64 bit vnodes. */
 
 /* Derive the name and create it O_EXCL. If that fails we have an error.
@@ -699,7 +827,7 @@ namei_icreate(IHandle_t * lh, char *part, afs_uint32 p1, afs_uint32 p2, afs_uint
     FdHandle_t *fdP;
     FdHandle_t tfd;
     int type, tag;
-    FILETIME ftime;
+    int ogm_p1, ogm_p2;
     char *p;
     b32_string_t str1;
 
@@ -719,8 +847,8 @@ namei_icreate(IHandle_t * lh, char *part, afs_uint32 p1, afs_uint32 p2, afs_uint
 	 * p3 - type
 	 * p4 - parent volume id
 	 */
-        ftime.dwHighDateTime = p1;
-        ftime.dwLowDateTime = p2;
+        ogm_p1 = p1;
+        ogm_p2 = p2;
 	type = p3;
 	tmp.ih_vid = p4;	/* Use parent volume id, where this file will be. */
 	tmp.ih_ino = namei_MakeSpecIno(p1, p3);
@@ -741,8 +869,8 @@ namei_icreate(IHandle_t * lh, char *part, afs_uint32 p1, afs_uint32 p2, afs_uint
 
 	tmp.ih_vid = p1;
 	tmp.ih_ino = (Inode) p2;
-	ftime.dwHighDateTime = p3;
-        ftime.dwLowDateTime = p4;
+	ogm_p1 = p3;
+        ogm_p2 = p4;
     }
 
     namei_HandleToName(&name, &tmp);
@@ -750,11 +878,11 @@ namei_icreate(IHandle_t * lh, char *part, afs_uint32 p1, afs_uint32 p2, afs_uint
     p++;
     for (tag = 0; tag < NAMEI_MAXVOLS; tag++) {
         *p = *int_to_base32(str1, tag);
-        fd = OS_OPEN((char *)&name.n_path, O_CREAT | O_RDWR | O_TRUNC | O_EXCL, 0666);
+        fd = OS_OPEN((char *)&name.n_path, O_CREAT | O_RDWR | O_EXCL, 0666);
         if (fd == INVALID_FD) {
             if (errno == ENOTDIR || errno == ENOENT) {
                 if (namei_CreateDataDirectories(&name, &created_dir) == 0)
-                    fd = OS_OPEN((char *)&name.n_path, O_CREAT | O_RDWR | O_TRUNC | O_EXCL, 0666);
+                    fd = OS_OPEN((char *)&name.n_path, O_CREAT | O_RDWR | O_EXCL, 0666);
             }
         }
 
@@ -771,7 +899,7 @@ namei_icreate(IHandle_t * lh, char *part, afs_uint32 p1, afs_uint32 p2, afs_uint
     tmp.ih_ino |= (((Inode) tag) << NAMEI_TAGSHIFT);
 
     if (!code) {
-        if (!SetFileTime((HANDLE) fd, &ftime, NULL, NULL)) {
+        if (SetWinOGM(fd, ogm_p1, ogm_p2)) {
 	    errno = OS_ERROR(EBADF);
             code = -1;
         }
@@ -886,12 +1014,12 @@ namei_icreate(IHandle_t * lh, char *part, afs_uint32 p1, afs_uint32 p2, afs_uint
     }
 
     namei_HandleToName(&name, &tmp);
-    fd = OS_OPEN(name.n_path, O_CREAT | O_EXCL | O_TRUNC | O_RDWR, 0);
+    fd = OS_OPEN(name.n_path, O_CREAT | O_EXCL | O_RDWR, 0);
     if (fd == INVALID_FD) {
 	if (errno == ENOTDIR || errno == ENOENT) {
 	    if (namei_CreateDataDirectories(&name, &created_dir) < 0)
 		goto bad;
-	    fd = OS_OPEN(name.n_path, O_CREAT | O_EXCL | O_TRUNC | O_RDWR,
+	    fd = OS_OPEN(name.n_path, O_CREAT | O_EXCL | O_RDWR,
 			  0);
 	    if (fd == INVALID_FD)
 		goto bad;
@@ -972,7 +1100,7 @@ namei_dec(IHandle_t * ih, Inode ino, int p1)
 	    return -1;
 	}
 
-	if (CheckOGM(&name, fdP, p1) < 0) {
+	if (CheckOGM(fdP, p1) < 0) {
 	    FDH_REALLYCLOSE(fdP);
 	    IH_RELEASE(tmp);
 	    errno = OS_ERROR(EINVAL);
@@ -1007,6 +1135,11 @@ namei_dec(IHandle_t * ih, Inode ino, int p1)
 		/* Try to remove directory. If it fails, that's ok.
 		 * Salvage will clean up.
 		 */
+		char *slash = strrchr(name.n_path, OS_DIRSEPC);
+		if (slash) {
+		    /* avoid an rmdir() on the file we just unlinked */
+		    *slash = '\0';
+		}
 		(void)namei_RemoveDataDirectories(&name);
 	    }
 	}
@@ -1138,7 +1271,7 @@ namei_copy_on_write(IHandle_t *h)
 	if (!fdP)
 	    return EIO;
 	snprintf(path, sizeof(path), "%s-tmp", name.n_path);
-	fd = OS_OPEN(path, O_CREAT | O_EXCL | O_TRUNC | O_RDWR, 0);
+	fd = OS_OPEN(path, O_CREAT | O_EXCL | O_RDWR, 0);
 	if (fd == INVALID_FD) {
 	    FDH_CLOSE(fdP);
 	    return EIO;
@@ -1323,8 +1456,8 @@ namei_GetLCOffsetAndIndexFromIno(Inode ino, afs_foff_t * offset, int *index)
 #ifdef AFS_PTHREAD_ENV
 /* XXX do static initializers work for WINNT/pthread? */
 pthread_mutex_t _namei_glc_lock = PTHREAD_MUTEX_INITIALIZER;
-#define NAMEI_GLC_LOCK MUTEX_ENTER(&_namei_glc_lock)
-#define NAMEI_GLC_UNLOCK MUTEX_EXIT(&_namei_glc_lock)
+#define NAMEI_GLC_LOCK opr_mutex_enter(&_namei_glc_lock)
+#define NAMEI_GLC_UNLOCK opr_mutex_exit(&_namei_glc_lock)
 #else /* !AFS_PTHREAD_ENV */
 #define NAMEI_GLC_LOCK
 #define NAMEI_GLC_UNLOCK
@@ -1367,6 +1500,9 @@ namei_GetLinkCount(FdHandle_t * h, Inode ino, int lockit, int fixup, int nowrite
     }
 
     rc = FDH_PREAD(h, (char*)&row, sizeof(row), offset);
+    if (rc == -1)
+	goto bad_getLinkByte;
+
     if ((rc == 0 || !((row >> index) & NAMEI_TAGMASK)) && fixup && nowrite)
         return 1;
     if (rc == 0 && fixup) {
@@ -1470,7 +1606,7 @@ GetFreeTag(IHandle_t * ih, int vno)
     if (FDH_PWRITE(fdP, (char *)&row, sizeof(row), offset) != sizeof(row)) {
 	goto badGetFreeTag;
     }
-    FDH_SYNC(fdP);
+    (void)FDH_SYNC(fdP);
     FDH_UNLOCKFILE(fdP, offset);
     FDH_CLOSE(fdP);
     return col;
@@ -1522,7 +1658,7 @@ namei_SetLinkCount(FdHandle_t * fdP, Inode ino, int count, int locked)
 	errno = OS_ERROR(EBADF);
 	goto bad_SetLinkCount;
     }
-    FDH_SYNC(fdP);
+    (void)FDH_SYNC(fdP);
 
     nBytes = 0;
 
@@ -1799,30 +1935,21 @@ _namei_examine_special(char * path1,
 {
     int ret = 0;
     struct ViceInodeInfo info;
-    afs_uint32 inode_vgid;
 
     if (DecodeInode(path1, dname, &info, myIH->ih_vid) < 0) {
 	ret = 0;
 	goto error;
     }
 
-#ifdef AFS_NT40_ENV
-    inode_vgid = myIH->ih_vid;
-#else
-    inode_vgid = (info.inodeNumber >> NAMEI_UNIQSHIFT) & NAMEI_UNIQMASK;
-#endif
-
     if (info.u.param[2] != VI_LINKTABLE) {
 	info.linkCount = 1;
-    } else if ((info.u.param[0] != myIH->ih_vid) ||
-	       (inode_vgid != myIH->ih_vid)) {
+    } else if (info.u.param[0] != myIH->ih_vid) {
 	/* VGID encoded in linktable filename and/or OGM data isn't
 	 * consistent with VGID encoded in namei path */
 	Log("namei_ListAFSSubDirs: warning: inconsistent linktable "
 	    "filename \"%s" OS_DIRSEP "%s\"; salvager will delete it "
-	    "(dir_vgid=%u, inode_vgid=%u, ogm_vgid=%u)\n",
+	    "(dir_vgid=%u, inode_vgid=%u)\n",
 	    path1, dname, myIH->ih_vid,
-	    (unsigned int)inode_vgid,
 	    info.u.param[0]);
     } else {
 	char path2[512];
@@ -2126,7 +2253,7 @@ static pthread_key_t wq_key;
 static void
 _namei_wq_keycreate(void)
 {
-    osi_Assert(pthread_key_create(&wq_key, NULL) == 0);
+    opr_Verify(pthread_key_create(&wq_key, NULL) == 0);
 }
 
 /**
@@ -2140,9 +2267,9 @@ _namei_wq_keycreate(void)
 void
 namei_SetWorkQueue(struct afs_work_queue *wq)
 {
-    osi_Assert(pthread_once(&wq_once, _namei_wq_keycreate) == 0);
+    opr_Verify(pthread_once(&wq_once, _namei_wq_keycreate) == 0);
 
-    osi_Assert(pthread_setspecific(wq_key, wq) == 0);
+    opr_Verify(pthread_setspecific(wq_key, wq) == 0);
 }
 
 /**
@@ -2318,7 +2445,7 @@ namei_ListAFSSubDirs(IHandle_t * dirIH,
 
     linkHandle.fd_fd = INVALID_FD;
 #ifdef AFS_SALSRV_ENV
-    osi_Assert(pthread_once(&wq_once, _namei_wq_keycreate) == 0);
+    opr_Verify(pthread_once(&wq_once, _namei_wq_keycreate) == 0);
 
     wq = pthread_getspecific(wq_key);
     if (!wq) {
@@ -2650,14 +2777,17 @@ DecodeInode(char *dpath, char *name, struct ViceInodeInfo *info,
     if (strcmp(name, check))
 	return -1;
 
-    GetOGMFromStat(&status, &parm, &tag);
     if ((info->inodeNumber & NAMEI_INODESPECIAL) == NAMEI_INODESPECIAL) {
+	parm = ((info->inodeNumber >> NAMEI_UNIQSHIFT) & NAMEI_UNIQMASK);
+	tag = (int)((info->inodeNumber >> NAMEI_TAGSHIFT) & NAMEI_TAGMASK);
+
 	/* p1 - vid, p2 - -1, p3 - type, p4 - rwvid */
 	info->u.param[0] = parm;
 	info->u.param[1] = -1;
 	info->u.param[2] = tag;
 	info->u.param[3] = volid;
     } else {
+	GetOGMFromStat(&status, &parm, &tag);
 	/* p1 - vid, p2 - vno, p3 - uniq, p4 - dv */
 	info->u.param[0] = volid;
 	info->u.param[1] = (int)(info->inodeNumber & NAMEI_VNODEMASK);
@@ -2769,6 +2899,8 @@ namei_ConvertROtoRWvolume(char *pname, afs_uint32 volumeId)
     struct DiskPartition64 *partP;
     struct ViceInodeInfo info;
     struct VolumeDiskHeader h;
+    char *rwpart, *rwname;
+    Error ec;
 # ifdef AFS_DEMAND_ATTACH_FS
     int locktype = 0;
 # endif /* AFS_DEMAND_ATTACH_FS */
@@ -2798,6 +2930,19 @@ namei_ConvertROtoRWvolume(char *pname, afs_uint32 volumeId)
 	goto done;
     }
 
+    /* check for existing RW on any partition on this server; */
+    /* if found, return EXDEV - invalid cross-device link */
+    VOL_LOCK;
+    VGetVolumePath(&ec, h.parent, &rwpart, &rwname);
+    if (ec == 0) {
+	Log("1 namei_ConvertROtoRWvolume: RW volume %lu already exists on server partition %s.\n",
+	    afs_printable_uint32_lu(h.parent), rwpart);
+	code = EXDEV;
+	VOL_UNLOCK;
+	goto done;
+    }
+    VOL_UNLOCK;
+
     FSYNC_VolOp(volumeId, pname, FSYNC_VOL_BREAKCBKS, 0, NULL);
 
     ino = namei_MakeSpecIno(h.parent, VI_LINKTABLE);
@@ -2891,7 +3036,7 @@ namei_ConvertROtoRWvolume(char *pname, afs_uint32 volumeId)
     }
     t_ih.ih_ino = namei_MakeSpecIno(ih->ih_vid, VI_VOLINFO);
     namei_HandleToName(&n, &t_ih);
-    fd2 = OS_OPEN(n.n_path, O_CREAT | O_EXCL | O_TRUNC | O_RDWR, 0);
+    fd2 = OS_OPEN(n.n_path, O_CREAT | O_EXCL | O_RDWR, 0);
     if (fd2 == INVALID_FD) {
 	Log("1 namei_ConvertROtoRWvolume: could not create RW info file: %s\n", n.n_path);
 	OS_CLOSE(fd);
@@ -3014,13 +3159,13 @@ static zlcList_t *zlcCur = NULL;
 static void
 AddToZLCDeleteList(char dir, char *name)
 {
-    osi_Assert(strlen(name) <= MAX_ZLC_NAMELEN - 3);
+    opr_Assert(strlen(name) <= MAX_ZLC_NAMELEN - 3);
 
     if (!zlcCur || zlcCur->zlc_n >= MAX_ZLC_NAMES) {
 	if (zlcCur && zlcCur->zlc_next)
 	    zlcCur = zlcCur->zlc_next;
 	else {
-	    zlcList_t *tmp = (zlcList_t *) malloc(sizeof(zlcList_t));
+	    zlcList_t *tmp = malloc(sizeof(zlcList_t));
 	    if (!tmp)
 		return;
 	    if (!zlcAnchor) {
