@@ -89,6 +89,7 @@
 #include <afs/dir.h>
 #include <afs/afsutil.h>
 #include <afs/fileutil.h>
+#include <rx/rx_queue.h>
 
 #include "nfs.h"
 #include "lwp.h"
@@ -152,7 +153,7 @@ handleit(struct cmd_syndesc *as, void *arock)
 
 #ifndef AFS_NT40_ENV
 	if (useSyslog)
-	    Log(msg);
+	    Log("%s", msg);
 	else
 #endif
 	    printf("%s\n", msg);
@@ -269,7 +270,7 @@ handleit(struct cmd_syndesc *as, void *arock)
 
 #ifndef AFS_NT40_ENV
 	if (useSyslog)
-	    Log(msg);
+	    Log("%s", msg);
 	else
 #endif
 	    printf("%s\n", msg);
