@@ -17,6 +17,7 @@
 #if defined(AFS_NAMEI_ENV) && !defined(AFS_NT40_ENV)
 #include <afs/dir.h>
 #include <rx/xdr.h>
+#include <rx/rx_queue.h>
 #include <afs/afsint.h>
 #include <afs/nfs.h>
 #include <lwp.h>
@@ -91,13 +92,11 @@ ExtractVnodes(struct Msg *m, Volume *vol, afs_int32 class,
 	goto Bad_Extract;
     }
     size = FDH_SIZE(fdP);
-    *list = (struct VnodeExtract *) malloc(size / vcp->diskSize
-					* sizeof(struct VnodeExtract));
+    *list = calloc(size / vcp->diskSize, sizeof(struct VnodeExtract));
     if (!(*list)) {
 	code = ENOMEM;
 	goto Bad_Extract;
     }
-    memset(*list, 0, size / vcp->diskSize * sizeof(struct VnodeExtract));
     stream = FDH_FDOPEN(fdP, "r");
     if (!stream) {
 	sprintf(m->line, "Couldn't stream open %s Index of volume %u\n",
@@ -377,7 +376,8 @@ afs_int32 copyVnodes(struct Msg *m, Volume *vol, Volume *newvol,
 		    code = EIO;
 		    goto Bad_Copy;
 	        }
-	        memcpy(parVnode, vnode, sizeof(struct VnodeDiskObject));
+		if (parVnode != NULL)
+		    memcpy(parVnode, vnode, sizeof(struct VnodeDiskObject));
 	    }
 	    if (e->flag & NEEDED && e->vN != where) {
 	        VNDISK_GET_LEN(size, vnode);
@@ -712,8 +712,7 @@ split_volume(struct rx_call *call, Volume *vol, Volume *newvol,
     afs_uint32 parent;
     struct Msg *m;
 
-    m = (struct Msg *) malloc(sizeof(struct Msg));
-    memset(m, 0, sizeof(struct Msg));
+    m = calloc(1, sizeof(struct Msg));
     m->call = call;
     m->verbose = verbose;
 
