@@ -10,7 +10,6 @@
 #ifndef _VOLSER_
 #define _VOLSER_ 1
 
-#include <afs/afs_assert.h>
 #ifdef AFS_PTHREAD_ENV
 #include <pthread.h>
 #endif
@@ -36,6 +35,7 @@
 #define volser_RW	0
 #define volser_RO	1
 #define	volser_BACK	2
+#define	volser_RWREPL	3
 
 #define	THOLD(tt)	((tt)->refCount++)
 
@@ -71,13 +71,13 @@ struct volser_trans {
 
 #ifdef AFS_PTHREAD_ENV
 #define VTRANS_OBJ_LOCK_INIT(tt) \
-  MUTEX_INIT(&((tt)->lock), "vtrans obj", MUTEX_DEFAULT, 0);
+  opr_mutex_init(&((tt)->lock));
 #define VTRANS_OBJ_LOCK_DESTROY(tt) \
-  MUTEX_DESTROY(&((tt)->lock))
+  opr_mutex_destroy(&((tt)->lock))
 #define VTRANS_OBJ_LOCK(tt) \
-  MUTEX_ENTER(&((tt)->lock))
+  opr_mutex_enter(&((tt)->lock))
 #define VTRANS_OBJ_UNLOCK(tt) \
-  MUTEX_EXIT(&((tt)->lock))
+  opr_mutex_exit(&((tt)->lock))
 #else
 #define VTRANS_OBJ_LOCK_INIT(tt)
 #define VTRANS_OBJ_LOCK_DESTROY(tt)
@@ -118,8 +118,6 @@ extern struct volser_trans *QI_GlobalWriteTrans;
 #define VMOVED		111	/* Volume has moved to another server; do a VGetVolumeInfo
 				 * to THIS server to find out where */
 
-#define MyPort 5003
-#define NameLen 80
 #define VLDB_MAXSERVERS 10
 #define VOLSERVICE_ID 4
 #define INVALID_BID 0
@@ -155,6 +153,7 @@ struct partList {		/*used by the backup system */
 #define ITSROVOL    0x02
 #define ITSRWVOL    0x04
 #define ITSBACKVOL  0x08
+#define ITSRWREPL   0x10
 #define RO_DONTUSE  0x20
 
 #define VLOP_RESTORE 0x100	/*this is bogus, clashes with VLOP_DUMP */
@@ -184,14 +183,14 @@ struct partList {		/*used by the backup system */
 #define RV_NOVLDB	0x040000
 #define RV_NOCLONE	0x080000
 #define RV_NODEL        0x100000
+#define RV_RWONLY	0x200000
 
 struct ubik_client;
 extern afs_uint32 vsu_GetVolumeID(char *astring, struct ubik_client *acstruct, afs_int32 *errp);
 extern int vsu_ExtractName(char rname[], char name[]);
-extern afs_int32 vsu_ClientInit(int noAuthFlag, const char *confDir,
-				char *cellName, afs_int32 sauth,
-				struct ubik_client **uclientp,
-				int (*secproc)(struct rx_securityClass *, afs_int32));
-extern void vsu_SetCrypt(int cryptflag);
-
+extern afs_int32 vsu_ClientInit(const char *confDir, char *cellName,
+				int secFlags,
+				int (*secproc)(struct rx_securityClass *,
+					       afs_int32),
+				struct ubik_client **uclientp);
 #endif /* _VOLSER_ */
